@@ -20,7 +20,6 @@
  */
 /**
  * SECTION: gesasset
- * @title: GESAsset
  * @short_description: Represents usable resources inside the GStreamer Editing Services
  *
  * The Assets in the GStreamer Editing Services represent the resources
@@ -78,19 +77,14 @@
  *
  * ]|
  *
- * You can request the creation of #GESAssets using either ges_asset_request() or
- * ges_asset_request_async(). All the #GESAssets are cached and thus any asset that has already
+ * You can request the creation of #GESAssets using either #ges_asset_request_async or
+ * #ges_asset_request_async. All the #GESAssets are cached and thus any asset that has already
  * been created can be requested again without overhead.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges.h"
 #include "ges-internal.h"
 
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include <gst/gst.h>
 
 GST_DEBUG_CATEGORY_STATIC (ges_asset_debug);
@@ -124,14 +118,14 @@ struct _GESAssetPrivate
   GESAssetState state;
   GType extractable_type;
 
-  /* When an asset is proxied, instantiating it will
+  /* When an asset is proxied, instanciating it will
    * return the asset it points to */
   char *proxied_asset_id;
 
   GList *proxies;
   GESAsset *proxy_target;
 
-  /* The error that occurred when an asset has been initialized with error */
+  /* The error that accured when an asset has been initialized with error */
   GError *error;
 };
 
@@ -145,7 +139,7 @@ typedef struct
 } GESAssetCacheEntry;
 
 /* Also protect all the entries in the cache */
-G_LOCK_DEFINE_STATIC (asset_cache_lock);
+static GMutex asset_cache_lock;
 /* We are mapping entries by types and ID, such as:
  *
  * {
@@ -169,8 +163,8 @@ G_LOCK_DEFINE_STATIC (asset_cache_lock);
  * different extractable types.
  **/
 static GHashTable *type_entries_table = NULL;
-#define LOCK_CACHE   (G_LOCK (asset_cache_lock))
-#define UNLOCK_CACHE (G_UNLOCK (asset_cache_lock))
+#define LOCK_CACHE   (g_mutex_lock (&asset_cache_lock))
+#define UNLOCK_CACHE (g_mutex_unlock (&asset_cache_lock))
 
 static gchar *
 _check_and_update_parameters (GType * extractable_type, const gchar * id,
@@ -274,7 +268,6 @@ initable_iface_init (GInitableIface * initable_iface)
 }
 
 G_DEFINE_TYPE_WITH_CODE (GESAsset, ges_asset, G_TYPE_OBJECT,
-    G_ADD_PRIVATE (GESAsset)
     G_IMPLEMENT_INTERFACE (G_TYPE_ASYNC_INITABLE, async_initable_iface_init)
     G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, initable_iface_init)
     G_IMPLEMENT_INTERFACE (GES_TYPE_META_CONTAINER, NULL));
@@ -294,38 +287,15 @@ ges_asset_extract_default (GESAsset * asset, GError ** error)
   GESAssetPrivate *priv = asset->priv;
   GESExtractable *n_extractable;
 
-
   params = ges_extractable_type_get_parameters_from_id (priv->extractable_type,
       priv->id, &n_params);
 
-#if GLIB_CHECK_VERSION(2, 53, 1)
-  {
-    gint i;
-    GValue *values;
-    const gchar **names;
-
-    values = g_malloc0 (sizeof (GValue) * n_params);
-    names = g_malloc0 (sizeof (gchar *) * n_params);
-
-    for (i = 0; i < n_params; i++) {
-      values[i] = params[i].value;
-      names[i] = params[i].name;
-    }
-
-    n_extractable =
-        GES_EXTRACTABLE (g_object_new_with_properties (priv->extractable_type,
-            n_params, names, values));
-    g_free (names);
-    g_free (values);
-  }
-#else
   n_extractable = g_object_newv (priv->extractable_type, n_params, params);
-#endif
 
   while (n_params--)
     g_value_unset (&params[n_params].value);
-
-  g_free (params);
+  if (params)
+    g_free (params);
 
   return n_extractable;
 }
@@ -403,9 +373,6 @@ ges_asset_finalize (GObject * object)
   if (priv->error)
     g_error_free (priv->error);
 
-  if (priv->proxies)
-    g_list_free (priv->proxies);
-
   G_OBJECT_CLASS (ges_asset_parent_class)->finalize (object);
 }
 
@@ -413,6 +380,7 @@ void
 ges_asset_class_init (GESAssetClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  g_type_class_add_private (klass, sizeof (GESAssetPrivate));
 
   object_class->get_property = ges_asset_get_property;
   object_class->set_property = ges_asset_set_property;
@@ -450,7 +418,8 @@ ges_asset_class_init (GESAssetClass * klass)
 void
 ges_asset_init (GESAsset * self)
 {
-  self->priv = ges_asset_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_ASSET, GESAssetPrivate);
 
   self->priv->state = ASSET_INITIALIZING;
   self->priv->proxied_asset_id = NULL;
@@ -486,9 +455,6 @@ _lookup_entry (GType extractable_type, const gchar * id)
 static void
 _free_entries (gpointer entry)
 {
-  GESAssetCacheEntry *data = (GESAssetCacheEntry *) entry;
-  if (data->asset)
-    gst_object_unref (data->asset);
   g_slice_free (GESAssetCacheEntry, entry);
 }
 
@@ -511,7 +477,7 @@ _gtask_return_true (GTask * task, gpointer udata)
  *
  * Looks for asset with specified id in cache and it's completely loaded.
  *
- * Returns: (transfer none) (nullable): The #GESAsset found or %NULL
+ * Returns: (transfer none): The #GESAsset found or %NULL
  */
 GESAsset *
 ges_asset_cache_lookup (GType extractable_type, const gchar * id)
@@ -637,26 +603,18 @@ ges_asset_cache_put (GESAsset * asset, GTask * task)
 void
 ges_asset_cache_init (void)
 {
+  g_mutex_init (&asset_cache_lock);
   type_entries_table = g_hash_table_new_full (g_str_hash, g_str_equal,
-      g_free, (GDestroyNotify) g_hash_table_unref);
+      NULL, (GDestroyNotify) g_hash_table_unref);
 
   _init_formatter_assets ();
   _init_standard_transition_assets ();
 }
 
-void
-ges_asset_cache_deinit (void)
-{
-  g_hash_table_destroy (type_entries_table);
-  type_entries_table = NULL;
-}
-
 gboolean
 ges_asset_request_id_update (GESAsset * asset, gchar ** proposed_id,
     GError * error)
 {
-  g_return_val_if_fail (GES_IS_ASSET (asset), FALSE);
-
   return GES_ASSET_GET_CLASS (asset)->request_id_update (asset, proposed_id,
       error);
 }
@@ -666,8 +624,6 @@ ges_asset_try_proxy (GESAsset * asset, const gchar * new_id)
 {
   GESAssetClass *class;
 
-  g_return_val_if_fail (GES_IS_ASSET (asset), FALSE);
-
   if (g_strcmp0 (asset->priv->id, new_id) == 0) {
     GST_WARNING_OBJECT (asset, "Trying to proxy to itself (%s),"
         " NOT possible", new_id);
@@ -713,7 +669,8 @@ _lookup_proxied_asset (const gchar * id, GESAssetCacheEntry * entry,
  * of the same file) as proxy. Note that when an asset is proxied, calling
  * #ges_asset_request will actually return the proxy asset.
  *
- * Returns: %TRUE if @proxy has been set on @asset, %FALSE otherwise.
+ * Returns: The list of proxies @asset has. Note that the default asset to be
+ * used is always the first in that list.
  */
 gboolean
 ges_asset_set_proxy (GESAsset * asset, GESAsset * proxy)
@@ -734,7 +691,7 @@ ges_asset_set_proxy (GESAsset * asset, GESAsset * proxy)
     if (asset->priv->proxies) {
       GESAsset *old_proxy = GES_ASSET (asset->priv->proxies->data);
 
-      old_proxy->priv->proxy_target = NULL;
+      GES_ASSET (asset->priv->proxies->data)->priv->proxy_target = NULL;
       g_object_notify_by_pspec (G_OBJECT (old_proxy),
           _properties[PROP_PROXY_TARGET]);
     }
@@ -769,8 +726,6 @@ ges_asset_set_proxy (GESAsset * asset, GESAsset * proxy)
     GST_ERROR_OBJECT (asset,
         "Trying to use %s as a proxy, but it is already proxying %s",
         proxy->priv->id, proxy->priv->proxy_target->priv->id);
-
-    return FALSE;
   }
 
   if (g_list_find (proxy->priv->proxies, asset)) {
@@ -797,37 +752,6 @@ ges_asset_set_proxy (GESAsset * asset, GESAsset * proxy)
   return TRUE;
 }
 
-/**
- * ges_asset_unproxy:
- * @asset: The #GESAsset to stop proxying with @proxy
- * @proxy: The #GESAsset to stop considering as a proxy for @asset
- *
- * Removes @proxy from the list of known proxies for @asset.
- * If @proxy was the current proxy for @asset, stop using it.
- *
- * Returns: %TRUE if @proxy was a known proxy for @asset, %FALSE otherwise.
- */
-gboolean
-ges_asset_unproxy (GESAsset * asset, GESAsset * proxy)
-{
-  g_return_val_if_fail (GES_IS_ASSET (asset), FALSE);
-  g_return_val_if_fail (GES_IS_ASSET (proxy), FALSE);
-  g_return_val_if_fail (asset != proxy, FALSE);
-
-  if (!g_list_find (asset->priv->proxies, proxy)) {
-    GST_INFO_OBJECT (asset, "%s is not a proxy.", proxy->priv->id);
-
-    return FALSE;
-  }
-
-  if (asset->priv->proxies->data == proxy)
-    ges_asset_set_proxy (asset, NULL);
-
-  asset->priv->proxies = g_list_remove (asset->priv->proxies, proxy);
-
-  return TRUE;
-}
-
 /**
  * ges_asset_list_proxies:
  * @asset: The #GESAsset to get proxies from
@@ -847,7 +771,7 @@ ges_asset_list_proxies (GESAsset * asset)
  * ges_asset_get_proxy:
  * @asset: The #GESAsset to get currenlty used proxy
  *
- * Returns: (transfer none) (nullable): The proxy in use for @asset
+ * Returns: (transfer none): The proxy in use for @asset
  */
 GESAsset *
 ges_asset_get_proxy (GESAsset * asset)
@@ -865,7 +789,7 @@ ges_asset_get_proxy (GESAsset * asset)
  * ges_asset_get_proxy_target:
  * @proxy: The #GESAsset from which to get the the asset it proxies.
  *
- * Returns: (transfer none) (nullable): The #GESAsset that is proxied by @proxy
+ * Returns: (transfer none): The #GESAsset that is proxied by @proxy
  */
 GESAsset *
 ges_asset_get_proxy_target (GESAsset * proxy)
@@ -886,11 +810,7 @@ ges_asset_set_id (GESAsset * asset, const gchar * id)
 
   gpointer orig_id = NULL;
   GESAssetCacheEntry *entry = NULL;
-  GESAssetPrivate *priv = NULL;
-
-  g_return_if_fail (GES_IS_ASSET (asset));
-
-  priv = asset->priv;
+  GESAssetPrivate *priv = asset->priv;
 
   if (priv->state != ASSET_INITIALIZED) {
     GST_WARNING_OBJECT (asset, "Trying to rest ID on an object that is"
@@ -922,7 +842,7 @@ ges_asset_set_id (GESAsset * asset, const gchar * id)
 }
 
 static GESAsset *
-_ensure_asset_for_wrong_id (const gchar * wrong_id, GType extractable_type,
+_unsure_material_for_wrong_id (const gchar * wrong_id, GType extractable_type,
     GError * error)
 {
   GESAsset *asset;
@@ -957,8 +877,6 @@ _ensure_asset_for_wrong_id (const gchar * wrong_id, GType extractable_type,
 GType
 ges_asset_get_extractable_type (GESAsset * self)
 {
-  g_return_val_if_fail (GES_IS_ASSET (self), G_TYPE_INVALID);
-
   return self->priv->extractable_type;
 }
 
@@ -994,11 +912,9 @@ ges_asset_request (GType extractable_type, const gchar * id, GError ** error)
   if (real_id == NULL) {
     /* We create an asset for that wrong ID so we have a reference that the
      * user requested it */
-    _ensure_asset_for_wrong_id (id, extractable_type, lerr);
+    _unsure_material_for_wrong_id (id, extractable_type, lerr);
     real_id = g_strdup (id);
   }
-  if (lerr)
-    g_error_free (lerr);
 
   asset = ges_asset_cache_lookup (extractable_type, real_id);
   if (asset) {
@@ -1082,14 +998,6 @@ done:
  * method. This asset can not be used as is.
  * @user_data: The user data to pass when @callback is called
  *
- * The @callback will be called from a running #GMainLoop which is iterating a #GMainContext.
- * Note that, users should ensure the #GMainContext, since this method will notify
- * @callback from the thread which was associated with a thread default
- * #GMainContext at calling ges_init().
- * For example, if a user wants non-default #GMainContext to be associated
- * with @callback, ges_init() must be called after g_main_context_push_thread_default ()
- * with custom #GMainContext.
- *
  * Request a new #GESAsset asyncronously, @callback will be called when the materail is
  * ready to be used or if an error occured.
  *
@@ -1127,7 +1035,6 @@ ges_asset_request_async (GType extractable_type,
   gchar *real_id;
   GESAsset *asset;
   GError *error = NULL;
-  GTask *task = NULL;
 
   g_return_if_fail (g_type_is_a (extractable_type, G_TYPE_OBJECT));
   g_return_if_fail (g_type_is_a (extractable_type, GES_TYPE_EXTRACTABLE));
@@ -1138,20 +1045,22 @@ ges_asset_request_async (GType extractable_type,
 
   real_id = _check_and_update_parameters (&extractable_type, id, &error);
   if (error) {
-    _ensure_asset_for_wrong_id (id, extractable_type, error);
+    _unsure_material_for_wrong_id (id, extractable_type, error);
     real_id = g_strdup (id);
   }
 
   /* Check if we already have an asset for this ID */
   asset = ges_asset_cache_lookup (extractable_type, real_id);
   if (asset) {
-    task = g_task_new (asset, NULL, callback, user_data);
+    GTask *task = g_task_new (asset, NULL, callback, user_data);
 
     /* In the case of proxied asset, we will loop until we find the
      * last asset of the chain of proxied asset */
     while (TRUE) {
       switch (asset->priv->state) {
         case ASSET_INITIALIZED:
+          gst_object_ref (asset);
+
           GST_DEBUG_OBJECT (asset, "Asset in cache and initialized, "
               "using it");
 
@@ -1163,7 +1072,6 @@ ges_asset_request_async (GType extractable_type,
           GST_DEBUG_OBJECT (asset, "Asset in cache and but not "
               "initialized, setting a new callback");
           ges_asset_cache_append_task (extractable_type, real_id, task);
-          task = NULL;
 
           goto done;
         case ASSET_PROXIED:{
@@ -1182,15 +1090,12 @@ ges_asset_request_async (GType extractable_type,
         case ASSET_NEEDS_RELOAD:
           GST_DEBUG_OBJECT (asset, "Asset in cache and needs reload");
           ges_asset_cache_append_task (extractable_type, real_id, task);
-          task = NULL;
           GES_ASSET_GET_CLASS (asset)->start_loading (asset, &error);
 
           goto done;
         case ASSET_INITIALIZED_WITH_ERROR:
           g_task_return_error (task,
-              error ? g_error_copy (error) : g_error_copy (asset->priv->error));
-
-          g_clear_error (&error);
+              error ? error : g_error_copy (asset->priv->error));
 
           goto done;
         default:
@@ -1204,8 +1109,6 @@ ges_asset_request_async (GType extractable_type,
       (extractable_type), G_PRIORITY_DEFAULT, cancellable, callback, user_data,
       "id", real_id, "extractable-type", extractable_type, NULL);
 done:
-  if (task)
-    gst_object_unref (task);
   if (real_id)
     g_free (real_id);
 }
@@ -1237,7 +1140,7 @@ ges_asset_needs_reload (GType extractable_type, const gchar * id)
 
   real_id = _check_and_update_parameters (&extractable_type, id, &error);
   if (error) {
-    _ensure_asset_for_wrong_id (id, extractable_type, error);
+    _unsure_material_for_wrong_id (id, extractable_type, error);
     real_id = g_strdup (id);
   }
 
@@ -1252,7 +1155,6 @@ ges_asset_needs_reload (GType extractable_type, const gchar * id)
         "Asset with id %s switch state to ASSET_NEEDS_RELOAD",
         ges_asset_get_id (asset));
     asset->priv->state = ASSET_NEEDS_RELOAD;
-    g_clear_error (&asset->priv->error);
     return TRUE;
   }
 
@@ -1382,8 +1284,8 @@ ges_list_assets (GType filter)
  * ges_asset_get_error:
  * @self: The asset to retrieve the error from
  *
- * Returns: (transfer none) (nullable): The #GError of the asset or %NULL if
- * the asset was loaded without issue
+ * Returns: (transfer none): The #GError of the asset or %NULL if the asset was
+ * loaded without issue
  *
  * Since: 1.8
  */
