@@ -20,12 +20,8 @@
 
 /**
  * SECTION:gesaudiourisource
- * @title: GESAudioUriSource
  * @short_description: outputs a single audio stream from a given file
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-utils.h"
 #include "ges-internal.h"
@@ -36,7 +32,7 @@
 
 struct _GESAudioUriSourcePrivate
 {
-  GstElement *decodebin;        /* Reference owned by parent class */
+  void *nothing;
 };
 
 enum
@@ -45,26 +41,6 @@ enum
   PROP_URI
 };
 
-static void
-ges_audio_uri_source_track_set_cb (GESAudioUriSource * self,
-    GParamSpec * arg G_GNUC_UNUSED, gpointer nothing)
-{
-  GESTrack *track;
-  const GstCaps *caps = NULL;
-
-  if (!self->priv->decodebin)
-    return;
-
-  track = ges_track_element_get_track (GES_TRACK_ELEMENT (self));
-  if (!track)
-    return;
-
-  caps = ges_track_get_caps (track);
-
-  GST_INFO_OBJECT (self, "Setting caps to: %" GST_PTR_FORMAT, caps);
-  g_object_set (self->priv->decodebin, "caps", caps, NULL);
-}
-
 /* GESSource VMethod */
 static GstElement *
 ges_audio_uri_source_create_source (GESTrackElement * trksrc)
@@ -78,8 +54,7 @@ ges_audio_uri_source_create_source (GESTrackElement * trksrc)
 
   track = ges_track_element_get_track (trksrc);
 
-  self->priv->decodebin = decodebin =
-      gst_element_factory_make ("uridecodebin", NULL);
+  decodebin = gst_element_factory_make ("uridecodebin", NULL);
 
   if (track)
     caps = ges_track_get_caps (track);
@@ -121,7 +96,7 @@ ges_extractable_interface_init (GESExtractableInterface * iface)
 }
 
 G_DEFINE_TYPE_WITH_CODE (GESAudioUriSource, ges_audio_uri_source,
-    GES_TYPE_AUDIO_SOURCE, G_ADD_PRIVATE (GESAudioUriSource)
+    GES_TYPE_AUDIO_SOURCE,
     G_IMPLEMENT_INTERFACE (GES_TYPE_EXTRACTABLE,
         ges_extractable_interface_init));
 
@@ -179,6 +154,8 @@ ges_audio_uri_source_class_init (GESAudioUriSourceClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GESAudioSourceClass *source_class = GES_AUDIO_SOURCE_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GESAudioUriSourcePrivate));
+
   object_class->get_property = ges_audio_uri_source_get_property;
   object_class->set_property = ges_audio_uri_source_set_property;
   object_class->dispose = ges_audio_uri_source_dispose;
@@ -198,10 +175,8 @@ ges_audio_uri_source_class_init (GESAudioUriSourceClass * klass)
 static void
 ges_audio_uri_source_init (GESAudioUriSource * self)
 {
-  self->priv = ges_audio_uri_source_get_instance_private (self);
-
-  g_signal_connect (self, "notify::track",
-      G_CALLBACK (ges_audio_uri_source_track_set_cb), NULL);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_AUDIO_URI_SOURCE, GESAudioUriSourcePrivate);
 }
 
 /**
@@ -210,8 +185,8 @@ ges_audio_uri_source_init (GESAudioUriSource * self)
  *
  * Creates a new #GESAudioUriSource for the provided @uri.
  *
- * Returns: (transfer floating) (nullable): The newly created
- * #GESAudioUriSource, or %NULL if there was an error.
+ * Returns: The newly created #GESAudioUriSource, or %NULL if there was an
+ * error.
  */
 GESAudioUriSource *
 ges_audio_uri_source_new (gchar * uri)
