@@ -22,7 +22,6 @@
 
 /**
  * SECTION:gesclip
- * @title: GESClip
  * @short_description: Base Class for objects in a GESLayer
  *
  * A #GESClip is a 'natural' object which controls one or more
@@ -31,9 +30,6 @@
  * Keeps a reference to the #GESTrackElement(s) it created and
  * sets/updates their properties.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-clip.h"
 #include "ges.h"
@@ -41,8 +37,7 @@
 
 #include <string.h>
 
-static GList *ges_clip_create_track_elements_func (GESClip * clip,
-    GESTrackType type);
+GList *ges_clip_create_track_elements_func (GESClip * clip, GESTrackType type);
 static gboolean _ripple (GESTimelineElement * element, GstClockTime start);
 static gboolean _ripple_end (GESTimelineElement * element, GstClockTime end);
 static gboolean _roll_start (GESTimelineElement * element, GstClockTime start);
@@ -50,12 +45,20 @@ static gboolean _roll_end (GESTimelineElement * element, GstClockTime end);
 static gboolean _trim (GESTimelineElement * element, GstClockTime start);
 static void _compute_height (GESContainer * container);
 
+G_DEFINE_ABSTRACT_TYPE (GESClip, ges_clip, GES_TYPE_CONTAINER);
+
 struct _GESClipPrivate
 {
   /*< public > */
   GESLayer *layer;
 
   /*< private > */
+
+  /* Set to TRUE when the clip is doing updates of track element
+   * properties so we don't end up in infinite property update loops
+   */
+  gboolean is_moving;
+
   guint nb_effects;
 
   GList *copied_track_elements;
@@ -81,15 +84,10 @@ enum
 
 static GParamSpec *properties[PROP_LAST];
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GESClip, ges_clip, GES_TYPE_CONTAINER);
-
 /****************************************************
  *              Listen to our children              *
  ****************************************************/
 
-/* @min_priority: The absolute minimum priority a child of @container should have
- * @max_priority: The absolute maximum priority a child of @container should have
- */
 static void
 _get_priority_range (GESContainer * container, guint32 * min_priority,
     guint32 * max_priority)
@@ -97,10 +95,10 @@ _get_priority_range (GESContainer * container, guint32 * min_priority,
   GESLayer *layer = GES_CLIP (container)->priv->layer;
 
   if (layer) {
-    *min_priority = _PRIORITY (container) + layer->min_nle_priority;
+    *min_priority = layer->min_nle_priority;
     *max_priority = layer->max_nle_priority;
   } else {
-    *min_priority = _PRIORITY (container) + MIN_NLE_PRIO;
+    *min_priority = MIN_NLE_PRIO;
     *max_priority = G_MAXUINT32;
   }
 }
@@ -121,7 +119,7 @@ _child_priority_changed_cb (GESTimelineElement * child,
   _get_priority_range (container, &min_prio, &max_prio);
 
   _ges_container_set_priority_offset (container, child,
-      min_prio - _PRIORITY (child));
+      min_prio + _PRIORITY (container) - _PRIORITY (child));
 }
 
 /*****************************************************
@@ -134,22 +132,28 @@ static gboolean
 _set_start (GESTimelineElement * element, GstClockTime start)
 {
   GList *tmp;
+  GESTimeline *timeline;
   GESContainer *container = GES_CONTAINER (element);
 
   GST_DEBUG_OBJECT (element, "Setting children start, (initiated_move: %"
       GST_PTR_FORMAT ")", container->initiated_move);
 
-  element->start = start;
-  g_object_notify (G_OBJECT (element), "start");
   container->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
   for (tmp = container->children; tmp; tmp = g_list_next (tmp)) {
     GESTimelineElement *child = (GESTimelineElement *) tmp->data;
 
-    _set_start0 (GES_TIMELINE_ELEMENT (child), start);
+    if (child != container->initiated_move) {
+      /* Make the snapping happen if in a timeline */
+      timeline = GES_TIMELINE_ELEMENT_TIMELINE (child);
+      if (timeline == NULL || ges_timeline_move_object_simple (timeline, child,
+              NULL, GES_EDGE_NONE, start) == FALSE)
+        _set_start0 (GES_TIMELINE_ELEMENT (child), start);
+
+    }
   }
   container->children_control_mode = GES_CHILDREN_UPDATE;
 
-  return FALSE;
+  return TRUE;
 }
 
 static gboolean
@@ -175,6 +179,7 @@ static gboolean
 _set_duration (GESTimelineElement * element, GstClockTime duration)
 {
   GList *tmp;
+  GESTimeline *timeline;
 
   GESContainer *container = GES_CONTAINER (element);
 
@@ -182,8 +187,13 @@ _set_duration (GESTimelineElement * element, GstClockTime duration)
   for (tmp = container->children; tmp; tmp = g_list_next (tmp)) {
     GESTimelineElement *child = (GESTimelineElement *) tmp->data;
 
-    if (child != container->initiated_move)
-      _set_duration0 (GES_TIMELINE_ELEMENT (child), duration);
+    if (child != container->initiated_move) {
+      /* Make the snapping happen if in a timeline */
+      timeline = GES_TIMELINE_ELEMENT_TIMELINE (child);
+      if (timeline == NULL || ges_timeline_trim_object_simple (timeline, child,
+              NULL, GES_EDGE_END, _START (child) + duration, TRUE) == FALSE)
+        _set_duration0 (GES_TIMELINE_ELEMENT (child), duration);
+    }
   }
   container->children_control_mode = GES_CHILDREN_UPDATE;
 
@@ -212,9 +222,9 @@ _set_priority (GESTimelineElement * element, guint32 priority)
 
   _get_priority_range (container, &min_prio, &max_prio);
 
-  container->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
+  container->children_control_mode = GES_CHILDREN_UPDATE_OFFSETS;
   for (tmp = container->children; tmp; tmp = g_list_next (tmp)) {
-    guint32 track_element_prio;
+    guint32 real_tck_prio;
     GESTimelineElement *child = (GESTimelineElement *) tmp->data;
     gint off = _ges_container_get_priority_offset (container, child);
 
@@ -226,21 +236,16 @@ _set_priority (GESTimelineElement * element, guint32 priority)
       off = 0;
     }
 
-    /* We need to remove our current priority from @min_prio
-     * as it is the absolute minimum priority @child could have
-     * before we set @container to @priority.
-     */
-    track_element_prio = min_prio - _PRIORITY (container) + priority - off;
+    real_tck_prio = min_prio + priority - off;
 
-    if (track_element_prio > max_prio) {
+    if (real_tck_prio > max_prio) {
       GST_WARNING ("%p priority of %i, is outside of the its containing "
           "layer space. (%d/%d) setting it to the maximum it can be",
-          container, priority, min_prio - _PRIORITY (container) + priority,
-          max_prio);
+          container, priority, min_prio, max_prio);
 
-      track_element_prio = max_prio;
+      real_tck_prio = max_prio;
     }
-    _set_priority0 (child, track_element_prio);
+    _set_priority0 (child, real_tck_prio);
   }
   container->children_control_mode = GES_CHILDREN_UPDATE;
   _compute_height (container);
@@ -248,17 +253,6 @@ _set_priority (GESTimelineElement * element, guint32 priority)
   return TRUE;
 }
 
-static guint32
-_get_layer_priority (GESTimelineElement * element)
-{
-  GESClip *clip = GES_CLIP (element);
-
-  if (clip->priv->layer == NULL)
-    return GES_TIMELINE_ELEMENT_NO_LAYER_PRIORITY;
-
-  return ges_layer_get_priority (clip->priv->layer);
-}
-
 /****************************************************
  *                                                  *
  *  GESContainer virtual methods implementation     *
@@ -304,7 +298,7 @@ _add_child (GESContainer * container, GESTimelineElement * element)
 
   /* If the TrackElement is an effect:
    *  - We add it on top of the list of TrackEffect
-   *  - We put all TrackElements present in the Clip
+   *  - We put all TrackObject present in the TimelineObject
    *    which are not BaseEffect on top of them
    * FIXME: Let the full control over priorities to the user
    */
@@ -314,7 +308,8 @@ _add_child (GESContainer * container, GESTimelineElement * element)
 
     GST_DEBUG_OBJECT (container, "Adding %ith effect: %" GST_PTR_FORMAT
         " Priority %i", priv->nb_effects + 1, element,
-        min_prio + priv->nb_effects);
+        min_prio + GES_TIMELINE_ELEMENT_PRIORITY (container) +
+        priv->nb_effects);
 
     tmp = g_list_nth (GES_CONTAINER_CHILDREN (container), priv->nb_effects);
     container->children_control_mode = GES_CHILDREN_UPDATE_OFFSETS;
@@ -323,12 +318,14 @@ _add_child (GESContainer * container, GESTimelineElement * element)
           GES_TIMELINE_ELEMENT_PRIORITY (tmp->data) + 1);
     }
 
-    _set_priority0 (element, min_prio + priv->nb_effects);
+    _set_priority0 (element, min_prio +
+        GES_TIMELINE_ELEMENT_PRIORITY (container) + priv->nb_effects);
     container->children_control_mode = mode;
     priv->nb_effects++;
   } else {
     /* We add the track element on top of the effect list */
-    _set_priority0 (element, min_prio + priv->nb_effects);
+    _set_priority0 (element, min_prio +
+        GES_TIMELINE_ELEMENT_PRIORITY (container) + priv->nb_effects);
   }
 
   /* We set the timing value of the child to ours, we avoid infinite loop
@@ -345,9 +342,8 @@ _add_child (GESContainer * container, GESTimelineElement * element)
 static gboolean
 _remove_child (GESContainer * container, GESTimelineElement * element)
 {
-  if (GES_IS_BASE_EFFECT (element)) {
+  if (GES_IS_BASE_EFFECT (element))
     GES_CLIP (container)->priv->nb_effects--;
-  }
 
   GST_FIXME_OBJECT (container, "We should set other children prios");
 
@@ -370,28 +366,6 @@ _child_removed (GESContainer * container, GESTimelineElement * element)
   g_signal_handlers_disconnect_by_func (element, _child_priority_changed_cb,
       container);
 
-  if (GES_IS_BASE_EFFECT (element)) {
-    GList *tmp;
-    guint32 priority;
-    GESChildrenControlMode mode = container->children_control_mode;
-
-    GST_DEBUG_OBJECT (container, "Resyncing effects priority.");
-
-    container->children_control_mode = GES_CHILDREN_UPDATE_OFFSETS;
-    tmp = GES_CONTAINER_CHILDREN (container);
-    priority =
-        ges_timeline_element_get_priority (GES_TIMELINE_ELEMENT (element));
-    for (; tmp; tmp = tmp->next) {
-      if (ges_timeline_element_get_priority (GES_TIMELINE_ELEMENT (tmp->data)) >
-          priority) {
-        ges_timeline_element_set_priority (GES_TIMELINE_ELEMENT (tmp->data),
-            GES_TIMELINE_ELEMENT_PRIORITY (tmp->data) - 1);
-      }
-    }
-
-    container->children_control_mode = mode;
-  }
-
   _compute_height (container);
 }
 
@@ -619,8 +593,9 @@ static gboolean
 _edit (GESContainer * container, GList * layers,
     gint new_layer_priority, GESEditMode mode, GESEdge edge, guint64 position)
 {
-  GESTimeline *timeline = GES_TIMELINE_ELEMENT_TIMELINE (container);
-  GESTimelineElement *element = GES_TIMELINE_ELEMENT (container);
+  GList *tmp;
+  gboolean ret = TRUE;
+  GESLayer *layer;
 
   if (!G_UNLIKELY (GES_CONTAINER_CHILDREN (container))) {
     GST_WARNING_OBJECT (container, "Trying to edit, but not containing"
@@ -628,35 +603,31 @@ _edit (GESContainer * container, GList * layers,
     return FALSE;
   }
 
-  if (!timeline) {
-    GST_WARNING_OBJECT (container, "Trying to edit, but not in any"
-        "timeline.");
-    return FALSE;
+  for (tmp = GES_CONTAINER_CHILDREN (container); tmp; tmp = g_list_next (tmp)) {
+    if (GES_IS_SOURCE (tmp->data) || GES_IS_TRANSITION (tmp->data)) {
+      ret &= ges_track_element_edit (tmp->data, layers, mode, edge, position);
+      break;
+    }
   }
 
-  switch (mode) {
-    case GES_EDIT_MODE_RIPPLE:
-      return timeline_ripple_object (timeline, element,
-          new_layer_priority <
-          0 ? GES_TIMELINE_ELEMENT_LAYER_PRIORITY (container) :
-          new_layer_priority, layers, edge, position);
-    case GES_EDIT_MODE_TRIM:
-      return timeline_trim_object (timeline, element,
-          new_layer_priority <
-          0 ? GES_TIMELINE_ELEMENT_LAYER_PRIORITY (container) :
-          new_layer_priority, layers, edge, position);
-    case GES_EDIT_MODE_NORMAL:
-      return timeline_move_object (timeline, element,
-          new_layer_priority <
-          0 ? GES_TIMELINE_ELEMENT_LAYER_PRIORITY (container) :
-          new_layer_priority, layers, edge, position);
-    case GES_EDIT_MODE_ROLL:
-      return timeline_roll_object (timeline, element, layers, edge, position);
-    case GES_EDIT_MODE_SLIDE:
-      GST_ERROR ("Sliding not implemented.");
+  /* Moving to layer */
+  if (new_layer_priority == -1) {
+    GST_DEBUG_OBJECT (container, "Not moving new prio %d", new_layer_priority);
+  } else {
+    gint priority_offset;
+
+    layer = GES_CLIP (container)->priv->layer;
+    if (layer == NULL) {
+      GST_WARNING_OBJECT (container, "Not in any layer yet, not moving");
+
       return FALSE;
+    }
+    priority_offset = new_layer_priority - ges_layer_get_priority (layer);
+
+    ret &= timeline_context_to_layer (layer->timeline, priority_offset);
   }
-  return FALSE;
+
+  return ret;
 }
 
 static void
@@ -665,16 +636,16 @@ _deep_copy (GESTimelineElement * element, GESTimelineElement * copy)
   GList *tmp;
   GESClip *self = GES_CLIP (element), *ccopy = GES_CLIP (copy);
 
-  if (!self->priv->layer)
-    return;
-
   for (tmp = GES_CONTAINER_CHILDREN (element); tmp; tmp = tmp->next) {
     ccopy->priv->copied_track_elements =
         g_list_append (ccopy->priv->copied_track_elements,
         ges_timeline_element_copy (tmp->data, TRUE));
   }
 
-  ccopy->priv->copied_layer = g_object_ref (self->priv->layer);
+  if (self->priv->copied_layer)
+    ccopy->priv->copied_layer = g_object_ref (self->priv->copied_layer);
+  else if (self->priv->layer)
+    ccopy->priv->copied_layer = g_object_ref (self->priv->layer);
 }
 
 static GESTimelineElement *
@@ -688,16 +659,12 @@ _paste (GESTimelineElement * element, GESTimelineElement * ref,
   if (self->priv->copied_layer)
     nclip->priv->copied_layer = g_object_ref (self->priv->copied_layer);
 
+  ges_clip_set_moving_from_layer (nclip, TRUE);
+  if (self->priv->copied_layer)
+    ges_layer_add_clip (self->priv->copied_layer, nclip);
+  ges_clip_set_moving_from_layer (nclip, FALSE);
+
   ges_timeline_element_set_start (GES_TIMELINE_ELEMENT (nclip), paste_position);
-  if (self->priv->copied_layer) {
-    if (!ges_layer_add_clip (self->priv->copied_layer, nclip)) {
-      GST_INFO ("%" GES_FORMAT " could not be pasted to %" GST_TIME_FORMAT,
-          GES_ARGS (element), GST_TIME_ARGS (paste_position));
-
-      return NULL;
-    }
-
-  }
 
   for (tmp = self->priv->copied_track_elements; tmp; tmp = tmp->next) {
     GESTrackElement *new_trackelement, *trackelement =
@@ -724,23 +691,6 @@ _paste (GESTimelineElement * element, GESTimelineElement * ref,
   return GES_TIMELINE_ELEMENT (nclip);
 }
 
-static gboolean
-_lookup_child (GESTimelineElement * self, const gchar * prop_name,
-    GObject ** child, GParamSpec ** pspec)
-{
-  GList *tmp;
-
-  if (GES_TIMELINE_ELEMENT_CLASS (ges_clip_parent_class)->lookup_child (self,
-          prop_name, child, pspec))
-    return TRUE;
-
-  for (tmp = GES_CONTAINER_CHILDREN (self); tmp; tmp = tmp->next) {
-    if (ges_timeline_element_lookup_child (tmp->data, prop_name, child, pspec))
-      return TRUE;
-  }
-
-  return FALSE;
-}
 
 /****************************************************
  *                                                  *
@@ -781,18 +731,15 @@ ges_clip_set_property (GObject * object, guint property_id,
 }
 
 static void
-ges_clip_dispose (GObject * object)
+ges_clip_finalize (GObject * object)
 {
   GESClip *self = GES_CLIP (object);
 
   g_list_free_full (self->priv->copied_track_elements, g_object_unref);
-  self->priv->copied_track_elements = NULL;
-  g_clear_object (&self->priv->copied_layer);
 
-  G_OBJECT_CLASS (ges_clip_parent_class)->dispose (object);
+  G_OBJECT_CLASS (ges_clip_parent_class)->finalize (object);
 }
 
-
 static void
 ges_clip_class_init (GESClipClass * klass)
 {
@@ -800,9 +747,11 @@ ges_clip_class_init (GESClipClass * klass)
   GESContainerClass *container_class = GES_CONTAINER_CLASS (klass);
   GESTimelineElementClass *element_class = GES_TIMELINE_ELEMENT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GESClipPrivate));
+
   object_class->get_property = ges_clip_get_property;
   object_class->set_property = ges_clip_set_property;
-  object_class->dispose = ges_clip_dispose;
+  object_class->finalize = ges_clip_finalize;
   klass->create_track_elements = ges_clip_create_track_elements_func;
   klass->create_track_element = NULL;
 
@@ -844,8 +793,6 @@ ges_clip_class_init (GESClipClass * klass)
   element_class->set_max_duration = _set_max_duration;
   element_class->paste = _paste;
   element_class->deep_copy = _deep_copy;
-  element_class->lookup_child = _lookup_child;
-  element_class->get_layer_priority = _get_layer_priority;
 
   container_class->add_child = _add_child;
   container_class->remove_child = _remove_child;
@@ -860,9 +807,12 @@ ges_clip_class_init (GESClipClass * klass)
 static void
 ges_clip_init (GESClip * self)
 {
-  self->priv = ges_clip_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_CLIP, GESClipPrivate);
+  /* FIXME, check why it was done this way _DURATION (self) = GST_SECOND; */
   self->priv->layer = NULL;
   self->priv->nb_effects = 0;
+  self->priv->is_moving = FALSE;
 }
 
 /**
@@ -874,8 +824,8 @@ ges_clip_init (GESClip * self)
  * keep a reference to the newly created trackelement, you therefore need to
  * call @ges_container_remove when you are done with it.
  *
- * Returns: (transfer none) (nullable): A #GESTrackElement. Returns NULL if
- * the #GESTrackElement could not be created.
+ * Returns: (transfer none): A #GESTrackElement. Returns NULL if the #GESTrackElement could not
+ * be created.
  */
 GESTrackElement *
 ges_clip_create_track_element (GESClip * clip, GESTrackType type)
@@ -919,10 +869,9 @@ ges_clip_create_track_element (GESClip * clip, GESTrackType type)
 GList *
 ges_clip_create_track_elements (GESClip * clip, GESTrackType type)
 {
-  GList *result = NULL, *tmp, *children;
+  GList *result, *tmp;
   GESClipClass *klass;
   guint max_prio, min_prio;
-  gboolean readding_effects_only = TRUE;
 
   g_return_val_if_fail (GES_IS_CLIP (clip), NULL);
 
@@ -935,25 +884,7 @@ ges_clip_create_track_elements (GESClip * clip, GESTrackType type)
 
   GST_DEBUG_OBJECT (clip, "Creating TrackElements for type: %s",
       ges_track_type_name (type));
-  children = ges_container_get_children (GES_CONTAINER (clip), TRUE);
-  for (tmp = children; tmp; tmp = tmp->next) {
-    GESTrackElement *child = GES_TRACK_ELEMENT (tmp->data);
-
-    if (!ges_track_element_get_track (child)
-        && ges_track_element_get_track_type (child) & type) {
-
-      GST_DEBUG_OBJECT (clip, "Removing for reusage: %" GST_PTR_FORMAT, child);
-      result = g_list_append (result, g_object_ref (child));
-      ges_container_remove (GES_CONTAINER (clip), tmp->data);
-      if (!GES_IS_BASE_EFFECT (child))
-        readding_effects_only = FALSE;
-    }
-  }
-  g_list_free_full (children, gst_object_unref);
-
-  if (readding_effects_only) {
-    result = g_list_concat (result, klass->create_track_elements (clip, type));
-  }
+  result = klass->create_track_elements (clip, type);
 
   _get_priority_range (GES_CONTAINER (clip), &min_prio, &max_prio);
   for (tmp = result; tmp; tmp = tmp->next) {
@@ -967,7 +898,8 @@ ges_clip_create_track_elements (GESClip * clip, GESTrackType type)
       ges_timeline_element_set_max_duration (GES_TIMELINE_ELEMENT (elem),
           GES_TIMELINE_ELEMENT_MAX_DURATION (clip));
 
-    _set_priority0 (elem, min_prio + clip->priv->nb_effects);
+    _set_priority0 (elem, min_prio + GES_TIMELINE_ELEMENT_PRIORITY (clip)
+        + clip->priv->nb_effects);
 
     ges_container_add (GES_CONTAINER (clip), elem);
   }
@@ -1008,10 +940,19 @@ ges_clip_set_layer (GESClip * clip, GESLayer * layer)
    * it is actually the result of a move between layer (as we know
    * that it will be added to another layer right after, and this
    * is what imports here.) */
-  if (!ELEMENT_FLAG_IS_SET (clip, GES_CLIP_IS_MOVING))
+  if (!clip->priv->is_moving)
     g_object_notify_by_pspec (G_OBJECT (clip), properties[PROP_LAYER]);
 }
 
+guint32
+ges_clip_get_layer_priority (GESClip * clip)
+{
+  if (clip->priv->layer == NULL)
+    return -1;
+
+  return ges_layer_get_priority (clip->priv->layer);
+}
+
 /**
  * ges_clip_set_moving_from_layer:
  * @clip: a #GESClip
@@ -1027,10 +968,7 @@ ges_clip_set_moving_from_layer (GESClip * clip, gboolean is_moving)
 {
   g_return_if_fail (GES_IS_CLIP (clip));
 
-  if (is_moving)
-    ELEMENT_SET_FLAG (clip, GES_CLIP_IS_MOVING);
-  else
-    ELEMENT_UNSET_FLAG (clip, GES_CLIP_IS_MOVING);
+  clip->priv->is_moving = is_moving;
 }
 
 /**
@@ -1041,6 +979,7 @@ ges_clip_set_moving_from_layer (GESClip * clip, gboolean is_moving)
  * You might rather use the ges_clip_move_to_layer function to
  * move #GESClip-s from a layer to another.
  *
+ *
  * Returns: %TRUE if @clip is currently moving from its current layer
  * %FALSE otherwize
  **/
@@ -1049,7 +988,7 @@ ges_clip_is_moving_from_layer (GESClip * clip)
 {
   g_return_val_if_fail (GES_IS_CLIP (clip), FALSE);
 
-  return ELEMENT_FLAG_IS_SET (clip, GES_CLIP_IS_MOVING);
+  return clip->priv->is_moving;
 }
 
 /**
@@ -1071,19 +1010,6 @@ ges_clip_move_to_layer (GESClip * clip, GESLayer * layer)
   g_return_val_if_fail (GES_IS_CLIP (clip), FALSE);
   g_return_val_if_fail (GES_IS_LAYER (layer), FALSE);
 
-  ELEMENT_SET_FLAG (clip, GES_CLIP_IS_MOVING);
-  if (layer->timeline
-      && !timeline_tree_can_move_element (timeline_get_tree (layer->timeline),
-          GES_TIMELINE_ELEMENT (clip),
-          ges_layer_get_priority (layer),
-          GES_TIMELINE_ELEMENT_START (clip),
-          GES_TIMELINE_ELEMENT_DURATION (clip), NULL)) {
-    GST_INFO_OBJECT (layer, "Clip %" GES_FORMAT " can't move to layer %d",
-        GES_ARGS (clip), ges_layer_get_priority (layer));
-    ELEMENT_UNSET_FLAG (clip, GES_CLIP_IS_MOVING);
-    return FALSE;
-  }
-
   current_layer = clip->priv->layer;
 
   if (current_layer == NULL) {
@@ -1095,17 +1021,17 @@ ges_clip_move_to_layer (GESClip * clip, GESLayer * layer)
   GST_DEBUG_OBJECT (clip, "moving to layer %p, priority: %d", layer,
       ges_layer_get_priority (layer));
 
+  clip->priv->is_moving = TRUE;
   gst_object_ref (clip);
   ret = ges_layer_remove_clip (current_layer, clip);
 
   if (!ret) {
-    ELEMENT_UNSET_FLAG (clip, GES_CLIP_IS_MOVING);
     gst_object_unref (clip);
     return FALSE;
   }
 
   ret = ges_layer_add_clip (layer, clip);
-  ELEMENT_UNSET_FLAG (clip, GES_CLIP_IS_MOVING);
+  clip->priv->is_moving = FALSE;
 
   gst_object_unref (clip);
   g_object_notify_by_pspec (G_OBJECT (clip), properties[PROP_LAYER]);
@@ -1127,8 +1053,8 @@ ges_clip_move_to_layer (GESClip * clip, GESLayer * layer)
  * Note: If many objects match, then the one with the highest priority will be
  * returned.
  *
- * Returns: (transfer full) (nullable): The #GESTrackElement used by @track,
- * else %NULL. Unref after usage
+ * Returns: (transfer full): The #GESTrackElement used by @track, else %NULL,
+ * Unref after usage
  */
 
 GESTrackElement *
@@ -1164,7 +1090,7 @@ ges_clip_find_track_element (GESClip * clip, GESTrack * track, GType type)
  *
  * Get the #GESLayer to which this clip belongs.
  *
- * Returns: (transfer full) (nullable): The #GESLayer where this @clip is being
+ * Returns: (transfer full): The #GESLayer where this @clip is being
  * used, or %NULL if it is not used on any layer. The caller should unref it
  * usage.
  */
@@ -1228,7 +1154,8 @@ ges_clip_get_top_effect_index (GESClip * clip, GESBaseEffect * effect)
 
   _get_priority_range (GES_CONTAINER (clip), &min_prio, &max_prio);
 
-  return GES_TIMELINE_ELEMENT_PRIORITY (effect) - min_prio;
+  return GES_TIMELINE_ELEMENT_PRIORITY (effect) - min_prio +
+      GES_TIMELINE_ELEMENT_PRIORITY (clip);
 }
 
 /* TODO 2.0 remove as it is Deprecated */
@@ -1269,18 +1196,16 @@ ges_clip_set_top_effect_index (GESClip * clip, GESBaseEffect * effect,
   g_return_val_if_fail (GES_IS_CLIP (clip), FALSE);
 
   track_element = GES_TRACK_ELEMENT (effect);
-  if (G_UNLIKELY (GES_CLIP (GES_TIMELINE_ELEMENT_PARENT (track_element)) !=
-          clip))
-    return FALSE;
-
   current_prio = _PRIORITY (track_element);
 
   _get_priority_range (GES_CONTAINER (clip), &min_prio, &max_prio);
 
   newindex = newindex + min_prio;
   /*  We don't change the priority */
-  if (current_prio == newindex)
-    return TRUE;
+  if (current_prio == newindex ||
+      (G_UNLIKELY (GES_CLIP (GES_TIMELINE_ELEMENT_PARENT (track_element)) !=
+              clip)))
+    return FALSE;
 
   if (newindex > (clip->priv->nb_effects - 1 + min_prio)) {
     GST_DEBUG ("You are trying to make %p not a top effect", effect);
@@ -1340,8 +1265,8 @@ ges_clip_set_top_effect_index (GESClip * clip, GESBaseEffect * effect,
  * property must be registered using @ges_effect_class_register_rate_property;
  * for the 'pitch' plugin, this is already done.
  *
- * Returns: (transfer none) (nullable): The newly created #GESClip resulting
- * from the splitting or %NULL if the clip can't be split.
+ * Returns: (transfer none): The newly created #GESClip resulting from the
+ * splitting
  */
 GESClip *
 ges_clip_split (GESClip * clip, guint64 position)
@@ -1384,9 +1309,6 @@ ges_clip_split (GESClip * clip, guint64 position)
       inpoint + old_duration * media_duration_factor);
   _set_duration0 (GES_TIMELINE_ELEMENT (new_object), new_duration);
 
-  _DURATION (clip) = old_duration;
-  g_object_notify (G_OBJECT (clip), "duration");
-
   /* We do not want the timeline to create again TrackElement-s */
   ges_clip_set_moving_from_layer (new_object, TRUE);
   ges_layer_add_clip (clip->priv->layer, new_object);
@@ -1419,10 +1341,7 @@ ges_clip_split (GESClip * clip, guint64 position)
         position - start + inpoint);
   }
 
-  ELEMENT_SET_FLAG (clip, GES_TIMELINE_ELEMENT_SET_SIMPLE);
-  _DURATION (clip) = duration;
   _set_duration0 (GES_TIMELINE_ELEMENT (clip), old_duration);
-  ELEMENT_UNSET_FLAG (clip, GES_TIMELINE_ELEMENT_SET_SIMPLE);
 
   return new_object;
 }
@@ -1461,40 +1380,134 @@ ges_clip_get_supported_formats (GESClip * clip)
 gboolean
 _ripple (GESTimelineElement * element, GstClockTime start)
 {
-  return ges_container_edit (GES_CONTAINER (element), NULL,
-      ges_timeline_element_get_layer_priority (element),
-      GES_EDIT_MODE_RIPPLE, GES_EDGE_NONE, start);
+  gboolean ret = TRUE;
+  GESTimeline *timeline;
+  GESClip *clip = GES_CLIP (element);
+
+  timeline = ges_layer_get_timeline (clip->priv->layer);
+
+  if (timeline == NULL) {
+    GST_DEBUG ("Not in a timeline yet");
+    return FALSE;
+  }
+
+  if (start > _END (element))
+    start = _END (element);
+
+  if (GES_CONTAINER_CHILDREN (element)) {
+    GESTrackElement *track_element =
+        GES_TRACK_ELEMENT (GES_CONTAINER_CHILDREN (element)->data);
+
+    ret = timeline_ripple_object (timeline, track_element, NULL, GES_EDGE_NONE,
+        start);
+  }
+
+  return ret;
 }
 
 static gboolean
 _ripple_end (GESTimelineElement * element, GstClockTime end)
 {
-  return ges_container_edit (GES_CONTAINER (element), NULL,
-      ges_timeline_element_get_layer_priority (element),
-      GES_EDIT_MODE_RIPPLE, GES_EDGE_END, end);
+  gboolean ret = TRUE;
+  GESTimeline *timeline;
+  GESClip *clip = GES_CLIP (element);
+
+  timeline = ges_layer_get_timeline (clip->priv->layer);
+
+  if (timeline == NULL) {
+    GST_DEBUG ("Not in a timeline yet");
+    return FALSE;
+  }
+
+  if (GES_CONTAINER_CHILDREN (element)) {
+    GESTrackElement *track_element =
+        GES_TRACK_ELEMENT (GES_CONTAINER_CHILDREN (element)->data);
+
+    ret = timeline_ripple_object (timeline, track_element, NULL, GES_EDGE_END,
+        end);
+  }
+
+  return ret;
 }
 
 gboolean
 _roll_start (GESTimelineElement * element, GstClockTime start)
 {
-  return ges_container_edit (GES_CONTAINER (element), NULL,
-      ges_timeline_element_get_layer_priority (element),
-      GES_EDIT_MODE_ROLL, GES_EDGE_START, start);
+  gboolean ret = TRUE;
+  GESTimeline *timeline;
+
+  GESClip *clip = GES_CLIP (element);
+
+  timeline = ges_layer_get_timeline (clip->priv->layer);
+
+  if (timeline == NULL) {
+    GST_DEBUG ("Not in a timeline yet");
+    return FALSE;
+  }
+
+  if (GES_CONTAINER_CHILDREN (element)) {
+    GESTrackElement *track_element =
+        GES_TRACK_ELEMENT (GES_CONTAINER_CHILDREN (element)->data);
+
+    ret = timeline_roll_object (timeline, track_element, NULL, GES_EDGE_START,
+        start);
+  }
+
+  return ret;
 }
 
 gboolean
 _roll_end (GESTimelineElement * element, GstClockTime end)
 {
-  return ges_container_edit (GES_CONTAINER (element), NULL,
-      ges_timeline_element_get_layer_priority (element),
-      GES_EDIT_MODE_ROLL, GES_EDGE_END, end);
+  gboolean ret = TRUE;
+  GESTimeline *timeline;
+
+  GESClip *clip = GES_CLIP (element);
+
+  timeline = ges_layer_get_timeline (clip->priv->layer);
+  if (timeline == NULL) {
+    GST_DEBUG ("Not in a timeline yet");
+    return FALSE;
+  }
+
+
+  if (GES_CONTAINER_CHILDREN (element)) {
+    GESTrackElement *track_element =
+        GES_TRACK_ELEMENT (GES_CONTAINER_CHILDREN (element)->data);
+
+    ret = timeline_roll_object (timeline, track_element,
+        NULL, GES_EDGE_END, end);
+  }
+
+  return ret;
 }
 
 gboolean
 _trim (GESTimelineElement * element, GstClockTime start)
 {
-  return ges_container_edit (GES_CONTAINER (element), NULL, -1,
-      GES_EDIT_MODE_TRIM, GES_EDGE_START, start);
+  gboolean ret = TRUE;
+  GESTimeline *timeline;
+
+  GESClip *clip = GES_CLIP (element);
+
+  timeline = ges_layer_get_timeline (clip->priv->layer);
+
+  if (timeline == NULL) {
+    GST_DEBUG ("Not in a timeline yet");
+    return FALSE;
+  }
+
+  if (GES_CONTAINER_CHILDREN (element)) {
+    GESTrackElement *track_element =
+        GES_TRACK_ELEMENT (GES_CONTAINER_CHILDREN (element)->data);
+
+    GST_DEBUG_OBJECT (element, "Trimming child: %" GST_PTR_FORMAT,
+        track_element);
+    ret = timeline_trim_object (timeline, track_element, NULL, GES_EDGE_START,
+        start);
+  }
+
+  return ret;
 }
 
 /**
@@ -1573,7 +1586,7 @@ ges_clip_find_track_elements (GESClip * clip, GESTrack * track,
     tmptrack = ges_track_element_get_track (otmp);
     if (((track != NULL && tmptrack == track)) ||
         (track_type != GES_TRACK_TYPE_UNKNOWN
-            && ges_track_element_get_track_type (otmp) == track_type)) {
+            && tmptrack->type == track_type)) {
 
       foundElement = GES_TRACK_ELEMENT (tmp->data);
 
