@@ -20,13 +20,9 @@
 
 /**
  * SECTION:gescontainer
- * @title: GESContainer
  * @short_description: Base Class for objects responsible for controlling other
  * GESTimelineElement-s
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-container.h"
 #include "ges.h"
@@ -34,6 +30,10 @@
 
 #include <string.h>
 
+#define _GET_PRIV(o) (G_TYPE_INSTANCE_GET_PRIVATE (self, GES_TYPE_CONTAINER, GESContainerPrivate))
+
+G_DEFINE_ABSTRACT_TYPE (GESContainer, ges_container, GES_TYPE_TIMELINE_ELEMENT);
+
 GST_DEBUG_CATEGORY_STATIC (ges_container_debug);
 #undef GST_CAT_DEFAULT
 #define GST_CAT_DEFAULT ges_container_debug
@@ -76,6 +76,7 @@ struct _GESContainerPrivate
    * properties so we don't end up in infinite property update loops
    */
   GHashTable *mappings;
+  guint nb_effects;
 
   /* List of GESTimelineElement being in the "child-added" signal
    * emission stage */
@@ -93,9 +94,6 @@ enum
 
 static GParamSpec *properties[PROP_LAST];
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GESContainer, ges_container,
-    GES_TYPE_TIMELINE_ELEMENT);
-
 /************************
  *   Private  methods   *
  ************************/
@@ -215,17 +213,14 @@ _ges_container_add_child_properties (GESContainer * container,
 
   for (i = 0; i < n_props; i++) {
     GObject *prop_child;
-    gchar *prop_name = g_strdup_printf ("%s::%s",
-        g_type_name (child_props[i]->owner_type),
-        child_props[i]->name);
 
-    if (ges_timeline_element_lookup_child (child, prop_name, &prop_child, NULL)) {
+    if (ges_timeline_element_lookup_child (child, child_props[i]->name,
+            &prop_child, NULL)) {
       ges_timeline_element_add_child_property (GES_TIMELINE_ELEMENT (container),
           child_props[i], prop_child);
-      gst_object_unref (prop_child);
 
     }
-    g_free (prop_name);
+
     g_param_spec_unref (child_props[i]);
   }
 
@@ -244,18 +239,14 @@ _ges_container_remove_child_properties (GESContainer * container,
 
   for (i = 0; i < n_props; i++) {
     GObject *prop_child;
-    gchar *prop_name = g_strdup_printf ("%s::%s",
-        g_type_name (child_props[i]->owner_type),
-        child_props[i]->name);
 
-    if (ges_timeline_element_lookup_child (child, prop_name, &prop_child, NULL)) {
+    if (ges_timeline_element_lookup_child (child, child_props[i]->name,
+            &prop_child, NULL)) {
       ges_timeline_element_remove_child_property (GES_TIMELINE_ELEMENT
           (container), child_props[i]);
-      gst_object_unref (prop_child);
 
     }
 
-    g_free (prop_name);
     g_param_spec_unref (child_props[i]);
   }
 
@@ -282,13 +273,12 @@ _lookup_child (GESTimelineElement * self, const gchar * prop_name,
 {
   GList *tmp;
 
-  /* FIXME Implement a syntax to precisely get properties by path */
-  for (tmp = GES_CONTAINER_CHILDREN (self); tmp; tmp = tmp->next) {
-    if (ges_timeline_element_lookup_child (tmp->data, prop_name, child, pspec))
-      return TRUE;
-  }
+  for (tmp = GES_CONTAINER_CHILDREN (self); tmp; tmp = tmp->next)
+    _ges_container_add_child_properties (GES_CONTAINER (self), tmp->data);
 
-  return FALSE;
+  return
+      GES_TIMELINE_ELEMENT_CLASS (ges_container_parent_class)->lookup_child
+      (self, prop_name, child, pspec);
 }
 
 static GESTrackType
@@ -348,15 +338,6 @@ _paste (GESTimelineElement * element, GESTimelineElement * ref,
     nchild =
         ges_timeline_element_paste (map->child,
         paste_position - map->start_offset);
-
-    if (!nchild) {
-      while (ncontainer->children)
-        ges_container_remove (ncontainer, ncontainer->children->data);
-
-      g_object_unref (ncontainer);
-      return NULL;
-    }
-
     ges_timeline_element_set_timeline (GES_TIMELINE_ELEMENT (ncontainer),
         GES_TIMELINE_ELEMENT_TIMELINE (ref));
     ges_container_add (ncontainer, nchild);
@@ -374,18 +355,9 @@ _paste (GESTimelineElement * element, GESTimelineElement * ref,
 static void
 _dispose (GObject * object)
 {
-  GList *tmp;
   GESContainer *self = GES_CONTAINER (object);
-  GList *children;
 
-  _ges_container_sort_children (self);
-  children = ges_container_get_children (self, FALSE);
-
-  for (tmp = g_list_last (children); tmp; tmp = tmp->prev)
-    ges_container_remove (self, tmp->data);
-
-  g_list_free_full (children, gst_object_unref);
-  self->children = NULL;
+  g_hash_table_unref (self->priv->mappings);
 
   G_OBJECT_CLASS (ges_container_parent_class)->dispose (object);
 }
@@ -398,10 +370,7 @@ _finalize (GObject * object)
   g_list_free_full (self->priv->copied_children,
       (GDestroyNotify) _free_mapping);
 
-  if (self->priv->mappings)
-    g_hash_table_destroy (self->priv->mappings);
-
-  G_OBJECT_CLASS (ges_container_parent_class)->finalize (object);
+  G_OBJECT_CLASS (ges_container_parent_class)->dispose (object);
 }
 
 static void
@@ -438,6 +407,8 @@ ges_container_class_init (GESContainerClass * klass)
   GST_DEBUG_CATEGORY_INIT (ges_container_debug, "gescontainer",
       GST_DEBUG_FG_YELLOW, "ges container");
 
+  g_type_class_add_private (klass, sizeof (GESContainerPrivate));
+
   object_class->get_property = _get_property;
   object_class->set_property = _set_property;
   object_class->dispose = _dispose;
@@ -479,7 +450,7 @@ ges_container_class_init (GESContainerClass * klass)
    */
   ges_container_signals[CHILD_REMOVED_SIGNAL] =
       g_signal_new ("child-removed", G_TYPE_FROM_CLASS (klass),
-      G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GESContainerClass, child_removed),
+      G_SIGNAL_RUN_FIRST, G_STRUCT_OFFSET (GESContainerClass, child_removed),
       NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1,
       GES_TYPE_TIMELINE_ELEMENT);
 
@@ -505,7 +476,7 @@ ges_container_class_init (GESContainerClass * klass)
 static void
 ges_container_init (GESContainer * self)
 {
-  self->priv = ges_container_get_instance_private (self);
+  self->priv = _GET_PRIV (self);
 
   /* FIXME, check why default was GST_SECOND? (before the existend of
    * ges-container)
@@ -532,14 +503,10 @@ _child_start_changed_cb (GESTimelineElement * child,
 
   GESContainerPrivate *priv = container->priv;
   GESTimelineElement *element = GES_TIMELINE_ELEMENT (container);
-  GESChildrenControlMode pmode = container->children_control_mode;
 
   map = g_hash_table_lookup (priv->mappings, child);
   g_assert (map);
 
-  if (ELEMENT_FLAG_IS_SET (child, GES_TIMELINE_ELEMENT_SET_SIMPLE))
-    container->children_control_mode = GES_CHILDREN_UPDATE_ALL_VALUES;
-
   switch (container->children_control_mode) {
     case GES_CHILDREN_IGNORE_NOTIFIES:
       return;
@@ -552,8 +519,8 @@ _child_start_changed_cb (GESTimelineElement * child,
         _DURATION (container) = _END (container) - start;
         _START (container) = start;
 
-        GST_DEBUG_OBJECT (container, "Child move made us move %" GES_FORMAT,
-            GES_ARGS (container));
+        GST_DEBUG_OBJECT (container, "Child move made us "
+            "move to %" GST_TIME_FORMAT, GST_TIME_ARGS (_START (container)));
 
         g_object_notify (G_OBJECT (container), "start");
       }
@@ -572,9 +539,6 @@ _child_start_changed_cb (GESTimelineElement * child,
     default:
       break;
   }
-
-  if (ELEMENT_FLAG_IS_SET (child, GES_TIMELINE_ELEMENT_SET_SIMPLE))
-    container->children_control_mode = pmode;
 }
 
 static void
@@ -592,8 +556,7 @@ _child_inpoint_changed_cb (GESTimelineElement * child,
   map = g_hash_table_lookup (priv->mappings, child);
   g_assert (map);
 
-  if (container->children_control_mode == GES_CHILDREN_UPDATE_OFFSETS
-      || ELEMENT_FLAG_IS_SET (child, GES_TIMELINE_ELEMENT_SET_SIMPLE)) {
+  if (container->children_control_mode == GES_CHILDREN_UPDATE_OFFSETS) {
     map->inpoint_offset = _START (container) - _START (child);
 
     return;
@@ -615,14 +578,10 @@ _child_duration_changed_cb (GESTimelineElement * child,
   GstClockTime end = 0;
   GESContainerPrivate *priv = container->priv;
   GESTimelineElement *element = GES_TIMELINE_ELEMENT (container);
-  GESChildrenControlMode pmode = container->children_control_mode;
 
   if (container->children_control_mode == GES_CHILDREN_IGNORE_NOTIFIES)
     return;
 
-  if (ELEMENT_FLAG_IS_SET (child, GES_TIMELINE_ELEMENT_SET_SIMPLE))
-    container->children_control_mode = GES_CHILDREN_UPDATE_ALL_VALUES;
-
   map = g_hash_table_lookup (priv->mappings, child);
   g_assert (map);
 
@@ -652,9 +611,6 @@ _child_duration_changed_cb (GESTimelineElement * child,
     default:
       break;
   }
-
-  if (ELEMENT_FLAG_IS_SET (child, GES_TIMELINE_ELEMENT_SET_SIMPLE))
-    container->children_control_mode = pmode;
 }
 
 /****************************************************
@@ -785,8 +741,6 @@ ges_container_add (GESContainer * container, GESTimelineElement * child)
 
   if (ges_timeline_element_set_parent (child, GES_TIMELINE_ELEMENT (container))
       == FALSE) {
-    if (class->remove_child)
-      class->remove_child (container, child);
 
     g_hash_table_remove (priv->mappings, child);
     container->children = g_list_remove (container->children, child);
