@@ -19,13 +19,9 @@
 
 /**
  * SECTION:geseffect
- * @title: GESEffect
  * @short_description: adds an effect build from a parse-launch style
  * bin description to a stream in a GESSourceClip or a GESLayer
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-internal.h"
 #include "ges-extractable.h"
@@ -36,6 +32,10 @@
 
 static void ges_extractable_interface_init (GESExtractableInterface * iface);
 
+G_DEFINE_TYPE_WITH_CODE (GESEffect,
+    ges_effect, GES_TYPE_BASE_EFFECT,
+    G_IMPLEMENT_INTERFACE (GES_TYPE_EXTRACTABLE,
+        ges_extractable_interface_init));
 
 static void ges_effect_dispose (GObject * object);
 static void ges_effect_finalize (GObject * object);
@@ -52,11 +52,6 @@ enum
   PROP_BIN_DESCRIPTION,
 };
 
-G_DEFINE_TYPE_WITH_CODE (GESEffect,
-    ges_effect, GES_TYPE_BASE_EFFECT, G_ADD_PRIVATE (GESEffect)
-    G_IMPLEMENT_INTERFACE (GES_TYPE_EXTRACTABLE,
-        ges_extractable_interface_init));
-
 static gchar *
 extractable_check_id (GType type, const gchar * id, GError ** error)
 {
@@ -75,8 +70,6 @@ extractable_check_id (GType type, const gchar * id, GError ** error)
   else
     g_assert_not_reached ();
 
-  g_free (bin_desc);
-
   return real_id;
 }
 
@@ -118,39 +111,6 @@ ges_extractable_interface_init (GESExtractableInterface * iface)
   iface->get_id = extractable_get_id;
 }
 
-static int
-property_name_compare (gconstpointer s1, gconstpointer s2)
-{
-  return g_strcmp0 ((const gchar *) s1, (const gchar *) s2);
-}
-
-static void
-_set_child_property (GESTimelineElement * self G_GNUC_UNUSED, GObject * child,
-    GParamSpec * pspec, GValue * value)
-{
-  GESEffectClass *klass = GES_EFFECT_GET_CLASS (self);
-  gchar *full_property_name;
-
-  GES_TIMELINE_ELEMENT_CLASS
-      (ges_effect_parent_class)->set_child_property (self, child, pspec, value);
-
-  full_property_name = g_strdup_printf ("%s::%s", G_OBJECT_TYPE_NAME (child),
-      pspec->name);
-
-  if (g_list_find_custom (klass->rate_properties, full_property_name,
-          property_name_compare)) {
-    GstElement *nleobject =
-        ges_track_element_get_nleobject (GES_TRACK_ELEMENT (self));
-    gdouble media_duration_factor =
-        ges_timeline_element_get_media_duration_factor (self);
-
-    g_object_set (nleobject, "media-duration-factor", media_duration_factor,
-        NULL);
-  }
-
-  g_free (full_property_name);
-}
-
 static void
 ges_effect_get_property (GObject * object,
     guint property_id, GValue * value, GParamSpec * pspec)
@@ -186,11 +146,11 @@ ges_effect_class_init (GESEffectClass * klass)
 {
   GObjectClass *object_class;
   GESTrackElementClass *obj_bg_class;
-  GESTimelineElementClass *element_class;
 
   object_class = G_OBJECT_CLASS (klass);
   obj_bg_class = GES_TRACK_ELEMENT_CLASS (klass);
-  element_class = GES_TIMELINE_ELEMENT_CLASS (klass);
+
+  g_type_class_add_private (klass, sizeof (GESEffectPrivate));
 
   object_class->get_property = ges_effect_get_property;
   object_class->set_property = ges_effect_set_property;
@@ -198,13 +158,11 @@ ges_effect_class_init (GESEffectClass * klass)
   object_class->finalize = ges_effect_finalize;
 
   obj_bg_class->create_element = ges_effect_create_element;
-  element_class->set_child_property = _set_child_property;
 
   klass->rate_properties = NULL;
   ges_effect_class_register_rate_property (klass, "scaletempo", "rate");
   ges_effect_class_register_rate_property (klass, "pitch", "tempo");
   ges_effect_class_register_rate_property (klass, "pitch", "rate");
-  ges_effect_class_register_rate_property (klass, "videorate", "rate");
 
   /**
    * GESEffect:bin-description:
@@ -224,7 +182,8 @@ ges_effect_class_init (GESEffectClass * klass)
 static void
 ges_effect_init (GESEffect * self)
 {
-  self->priv = ges_effect_get_instance_private (self);
+  self->priv =
+      G_TYPE_INSTANCE_GET_PRIVATE (self, GES_TYPE_EFFECT, GESEffectPrivate);
 }
 
 static void
@@ -244,50 +203,15 @@ ges_effect_finalize (GObject * object)
   G_OBJECT_CLASS (ges_effect_parent_class)->finalize (object);
 }
 
-static void
-ghost_compatible_pads (GstElement * bin, GstElement * child,
-    GstCaps * valid_caps, gint * n_src, gint * n_sink)
-{
-  GList *tmp;
-
-  for (tmp = child->pads; tmp; tmp = tmp->next) {
-    GstCaps *caps;
-    GstPad *pad = tmp->data;
-
-    if (GST_PAD_PEER (pad))
-      continue;
-
-    caps = gst_pad_query_caps (pad, NULL);
-
-    if (gst_caps_can_intersect (caps, valid_caps)) {
-      gchar *name =
-          g_strdup_printf ("%s_%d", GST_PAD_IS_SINK (pad) ? "sink" : "src",
-          GST_PAD_IS_SINK (pad) ? *n_sink++ : *n_src++);
-
-      GST_DEBUG_OBJECT (bin, "Ghosting pad: %" GST_PTR_FORMAT, pad);
-      gst_element_add_pad (GST_ELEMENT (bin), gst_ghost_pad_new (name, pad));
-      g_free (name);
-    } else {
-      GST_DEBUG_OBJECT (pad, "Can't ghost pad %" GST_PTR_FORMAT, caps);
-    }
-
-    gst_caps_unref (caps);
-  }
-}
-
 static GstElement *
 ges_effect_create_element (GESTrackElement * object)
 {
-  GList *tmp;
   GstElement *effect;
   gchar *bin_desc;
-  GstCaps *valid_caps;
-  gint n_src = 0, n_sink = 0;
 
   GError *error = NULL;
   GESEffect *self = GES_EFFECT (object);
-  const gchar *blacklisted_factories[] =
-      { "audioconvert", "audioresample", "videoconvert", NULL };
+  const gchar *wanted_categories[] = { "Effect", NULL };
 
   GESTrackType type = ges_track_element_get_track_type (object);
 
@@ -295,47 +219,31 @@ ges_effect_create_element (GESTrackElement * object)
     bin_desc = g_strconcat ("videoconvert name=pre_video_convert ! ",
         self->priv->bin_description, " ! videoconvert name=post_video_convert",
         NULL);
-    valid_caps = gst_caps_from_string ("video/x-raw(ANY)");
   } else if (type == GES_TRACK_TYPE_AUDIO) {
     bin_desc =
         g_strconcat ("audioconvert ! audioresample !",
         self->priv->bin_description, NULL);
-    valid_caps = gst_caps_from_string ("audio/x-raw(ANY)");
   } else {
     g_assert_not_reached ();
   }
 
-  effect = gst_parse_bin_from_description (bin_desc, FALSE, &error);
+  effect = gst_parse_bin_from_description (bin_desc, TRUE, &error);
+
   g_free (bin_desc);
+
   if (error != NULL) {
     GST_ERROR ("An error occured while creating the GstElement: %s",
         error->message);
     g_error_free (error);
-    goto fail;
+    return NULL;
   }
 
-  for (tmp = GST_BIN_CHILDREN (effect); tmp; tmp = tmp->next) {
-    ghost_compatible_pads (effect, tmp->data, valid_caps, &n_src, &n_sink);
+  GST_DEBUG ("Created effect %p", effect);
 
-    if (n_src > 1) {
-      GST_ERROR ("More than 1 source pad in the effect, that is not possible");
-
-      goto fail;
-    }
-  }
-
-  ges_track_element_add_children_props (object, effect, NULL,
-      blacklisted_factories, NULL);
-
-done:
-  gst_clear_caps (&valid_caps);
+  ges_track_element_add_children_props (object, effect, wanted_categories,
+      NULL, NULL);
 
   return effect;
-
-fail:
-  gst_clear_object (&effect);
-
-  goto done;
 }
 
 /**
@@ -349,7 +257,7 @@ fail:
  * #ges_asset_request (GES_TYPE_EFFECT, "audio your ! bin ! description", NULL);
  * and extract that asset to be in full control.
  *
- * Returns: (nullable): a newly created #GESEffect, or %NULL if something went
+ * Returns: a newly created #GESEffect, or %NULL if something went
  * wrong.
  */
 GESEffect *
@@ -368,6 +276,12 @@ ges_effect_new (const gchar * bin_description)
   return effect;
 }
 
+static int
+property_name_compare (gconstpointer s1, gconstpointer s2)
+{
+  return g_strcmp0 ((const gchar *) s1, (const gchar *) s2);
+}
+
 /**
  * ges_effect_class_register_rate_property:
  * @klass: Instance of the GESEffectClass
