@@ -20,12 +20,8 @@
 
 /**
  * SECTION:ges-enums
- * @title: GES Enumerations
  * @short_description: Various enums for the Gstreamer Editing Services
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-enums.h"
 #include "ges-internal.h"
@@ -152,19 +148,6 @@ register_ges_edge (GType * id)
   *id = g_enum_register_static ("GESEdge", edges);
 }
 
-const gchar *
-ges_edge_name (GESEdge edge)
-{
-  switch (edge) {
-    case GES_EDGE_START:
-      return "start";
-    case GES_EDGE_END:
-      return "end";
-    default:
-      return "none";
-  }
-}
-
 GType
 ges_edge_get_type (void)
 {
@@ -407,8 +390,6 @@ _init_standard_transition_assets (void)
     ges_meta_container_register_meta_string (GES_META_CONTAINER (asset),
         GES_META_READABLE, GES_META_DESCRIPTION,
         transition_types[i].value_name);
-
-    gst_object_unref (asset);
   }
 
 }
@@ -440,8 +421,8 @@ ges_text_valign_get_type (void)
     {GES_TEXT_VALIGN_BOTTOM, "GES_TEXT_VALIGN_BOTTOM", "bottom"},
     {GES_TEXT_VALIGN_TOP, "GES_TEXT_VALIGN_TOP", "top"},
     {GES_TEXT_VALIGN_POSITION, "GES_TEXT_VALIGN_POSITION", "position"},
-    {GES_TEXT_VALIGN_CENTER, "GES_TEXT_VALIGN_CENTER", "center"},
     {GES_TEXT_VALIGN_ABSOLUTE, "GES_TEXT_VALIGN_ABSOLUTE", "absolute"},
+    {GES_TEXT_VALIGN_CENTER, "GES_TEXT_VALIGN_CENTER", "center"},
     {0, NULL, NULL},
   };
 
