@@ -19,7 +19,6 @@
 
 /**
  * SECTION:gesgroup
- * @title: GESGroup
  * @short_description: Class that permits to group GESClip-s in a timeline,
  * letting the user manage it a single GESTimelineElement
  *
@@ -31,9 +30,6 @@
  * should be instaciated to group the various #GESTimelineElement passed
  * in parametter.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-group.h"
 #include "ges.h"
@@ -42,6 +38,8 @@
 #include <string.h>
 
 #define parent_class ges_group_parent_class
+G_DEFINE_TYPE (GESGroup, ges_group, GES_TYPE_CONTAINER);
+
 #define GES_CHILDREN_INIBIT_SIGNAL_EMISSION (GES_CHILDREN_LAST + 1)
 #define GES_GROUP_SIGNALS_IDS_DATA_KEY_FORMAT "ges-group-signals-ids-%p"
 
@@ -77,8 +75,6 @@ enum
 
 static GParamSpec *properties[PROP_LAST] = { NULL, };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GESGroup, ges_group, GES_TYPE_CONTAINER);
-
 /****************************************************
  *              Our listening of children           *
  ****************************************************/
@@ -94,12 +90,7 @@ _update_our_values (GESGroup * group)
 
     if (GES_IS_CLIP (child)) {
       GESLayer *layer = ges_clip_get_layer (GES_CLIP (child));
-      gint32 prio;
-
-      if (!layer)
-        continue;
-
-      prio = ges_layer_get_priority (layer);
+      gint32 prio = ges_layer_get_priority (layer);
 
       min_layer_prio = MIN (prio, min_layer_prio);
       max_layer_prio = MAX (prio, max_layer_prio);
@@ -118,7 +109,7 @@ _update_our_values (GESGroup * group)
     for (tmp = GES_CONTAINER_CHILDREN (group); tmp; tmp = tmp->next) {
       GESTimelineElement *child = tmp->data;
       guint32 child_prio = GES_IS_CLIP (child) ?
-          GES_TIMELINE_ELEMENT_LAYER_PRIORITY (child) : _PRIORITY (child);
+          ges_clip_get_layer_priority (GES_CLIP (child)) : _PRIORITY (child);
 
       _ges_container_set_priority_offset (container,
           child, min_layer_prio - child_prio);
@@ -159,7 +150,7 @@ _child_clip_changed_layer_cb (GESTimelineElement * clip,
   ChildSignalIds *sigids;
   gchar *signals_ids_key;
   GESLayer *old_layer, *new_layer;
-  gint offset, layer_prio = GES_TIMELINE_ELEMENT_LAYER_PRIORITY (clip);
+  gint offset, layer_prio = ges_clip_get_layer_priority (GES_CLIP (clip));
   GESContainer *container = GES_CONTAINER (group);
 
   offset = _ges_container_get_priority_offset (container, clip);
@@ -191,14 +182,13 @@ _child_clip_changed_layer_cb (GESTimelineElement * clip,
     return;
   }
 
-  if (new_layer && old_layer && (layer_prio + offset < 0 ||
+  if (new_layer && (layer_prio + offset < 0 ||
           (GES_TIMELINE_ELEMENT_TIMELINE (group) &&
               layer_prio + offset + GES_CONTAINER_HEIGHT (group) - 1 >
               g_list_length (GES_TIMELINE_ELEMENT_TIMELINE (group)->layers)))) {
 
-    GST_INFO_OBJECT (container,
-        "Trying to move to a layer %" GST_PTR_FORMAT " outside of"
-        "the timeline layers, moving back to old layer (prio %i)", new_layer,
+    GST_INFO_OBJECT (container, "Trying to move to a layer outside of"
+        "the timeline layers, moving back to old layer (prio %i)",
         _PRIORITY (group) - offset);
 
     container->children_control_mode = GES_CHILDREN_INIBIT_SIGNAL_EMISSION;
@@ -208,12 +198,6 @@ _child_clip_changed_layer_cb (GESTimelineElement * clip,
     return;
   }
 
-  if (!new_layer || !old_layer) {
-    _update_our_values (group);
-
-    return;
-  }
-
   container->initiated_move = clip;
   _set_priority0 (GES_TIMELINE_ELEMENT (group), layer_prio + offset);
   container->initiated_move = NULL;
@@ -255,7 +239,11 @@ _child_group_priority_changed (GESTimelineElement * child,
 static gboolean
 _trim (GESTimelineElement * group, GstClockTime start)
 {
+  GList *tmp;
+  GstClockTime last_child_end = 0, oldstart = _START (group);
+  GESContainer *container = GES_CONTAINER (group);
   GESTimeline *timeline = GES_TIMELINE_ELEMENT_TIMELINE (group);
+  gboolean ret = TRUE, expending = (start < _START (group));
 
   if (timeline == NULL) {
     GST_DEBUG ("Not in a timeline yet");
@@ -263,9 +251,40 @@ _trim (GESTimelineElement * group, GstClockTime start)
     return FALSE;
   }
 
-  return timeline_tree_trim (timeline_get_tree (timeline), group,
-      0, GST_CLOCK_DIFF (start, _START (group)), GES_EDGE_START,
-      ges_timeline_get_snapping_distance (timeline));
+  container->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
+  for (tmp = GES_CONTAINER_CHILDREN (group); tmp; tmp = tmp->next) {
+    GESTimelineElement *child = tmp->data;
+
+    if (expending) {
+      /* If the start is bigger, we do not touch it (in case we are expending) */
+      if (_START (child) > oldstart) {
+        GST_DEBUG_OBJECT (child, "Skipping as not at begining of the group");
+        continue;
+      }
+
+      ret &= ges_timeline_element_trim (child, start);
+    } else {
+      if (start > _END (child))
+        ret &= ges_timeline_element_trim (child, _END (child));
+      else if (_START (child) < start && _DURATION (child))
+        ret &= ges_timeline_element_trim (child, start);
+
+    }
+  }
+
+  for (tmp = GES_CONTAINER_CHILDREN (group); tmp; tmp = tmp->next) {
+    if (_DURATION (tmp->data))
+      last_child_end =
+          MAX (GES_TIMELINE_ELEMENT_END (tmp->data), last_child_end);
+  }
+
+  GES_GROUP (group)->priv->setting_value = TRUE;
+  _set_start0 (group, start);
+  _set_duration0 (group, last_child_end - start);
+  GES_GROUP (group)->priv->setting_value = FALSE;
+  container->children_control_mode = GES_CHILDREN_UPDATE;
+
+  return ret;
 }
 
 static gboolean
@@ -274,44 +293,39 @@ _set_priority (GESTimelineElement * element, guint32 priority)
   GList *tmp, *layers;
   gint diff = priority - _PRIORITY (element);
   GESContainer *container = GES_CONTAINER (element);
-  GESTimeline *timeline = GES_TIMELINE_ELEMENT_TIMELINE (element);
 
   if (GES_GROUP (element)->priv->setting_value == TRUE)
     return TRUE;
 
   container->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
-
   layers = GES_TIMELINE_ELEMENT_TIMELINE (element) ?
       GES_TIMELINE_ELEMENT_TIMELINE (element)->layers : NULL;
+
   if (layers == NULL) {
     GST_WARNING_OBJECT (element, "Not any layer in the timeline, not doing"
         "anything, timeline: %" GST_PTR_FORMAT,
         GES_TIMELINE_ELEMENT_TIMELINE (element));
 
     return FALSE;
+  } else if (priority + GES_CONTAINER_HEIGHT (container) - 1 >
+      g_list_length (layers)) {
+    GST_WARNING_OBJECT (container, "Trying to move to a layer outside of"
+        "the timeline layers");
+    return FALSE;
   }
 
   for (tmp = GES_CONTAINER_CHILDREN (element); tmp; tmp = tmp->next) {
     GESTimelineElement *child = tmp->data;
 
-    if (child != container->initiated_move
-        || ELEMENT_FLAG_IS_SET (container, GES_TIMELINE_ELEMENT_SET_SIMPLE)) {
+    if (child != container->initiated_move) {
       if (GES_IS_CLIP (child)) {
-        guint32 layer_prio = GES_TIMELINE_ELEMENT_LAYER_PRIORITY (child) + diff;
-        GESLayer *layer =
-            g_list_nth_data (GES_TIMELINE_GET_LAYERS (timeline), layer_prio);
+        guint32 layer_prio =
+            ges_clip_get_layer_priority (GES_CLIP (child)) + diff;
 
-        if (layer == NULL) {
-          do {
-            layer = ges_timeline_append_layer (timeline);
-          } while (ges_layer_get_priority (layer) < layer_prio);
-        }
-
-        GST_DEBUG ("%" GES_FORMAT "moving from layer: %i to %i",
-            GES_ARGS (child), GES_TIMELINE_ELEMENT_LAYER_PRIORITY (child),
-            layer_prio);
-        ges_clip_move_to_layer (GES_CLIP (child), g_list_nth_data (layers,
-                layer_prio));
+        GST_DEBUG_OBJECT (child, "moving from layer: %i to %i",
+            ges_clip_get_layer_priority (GES_CLIP (child)), layer_prio);
+        ges_clip_move_to_layer (GES_CLIP (child),
+            g_list_nth_data (layers, layer_prio));
       } else if (GES_IS_GROUP (child)) {
         GST_DEBUG_OBJECT (child, "moving from %i to %i",
             _PRIORITY (child), diff + _PRIORITY (child));
@@ -329,31 +343,24 @@ _set_start (GESTimelineElement * element, GstClockTime start)
 {
   GList *tmp;
   gint64 diff = start - _START (element);
-  GESTimeline *timeline;
   GESContainer *container = GES_CONTAINER (element);
-  GESTimelineElement *toplevel =
-      ges_timeline_element_get_toplevel_parent (element);;
 
-  gst_object_unref (toplevel);
   if (GES_GROUP (element)->priv->setting_value == TRUE)
     /* Let GESContainer update itself */
     return GES_TIMELINE_ELEMENT_CLASS (parent_class)->set_start (element,
         start);
 
-  if (ELEMENT_FLAG_IS_SET (element, GES_TIMELINE_ELEMENT_SET_SIMPLE) ||
-      ELEMENT_FLAG_IS_SET (toplevel, GES_TIMELINE_ELEMENT_SET_SIMPLE)) {
-    container->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
-    for (tmp = GES_CONTAINER_CHILDREN (element); tmp; tmp = tmp->next)
-      _set_start0 (tmp->data, _START (tmp->data) + diff);
-    container->children_control_mode = GES_CHILDREN_UPDATE;
 
-    return TRUE;
+  container->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
+  for (tmp = GES_CONTAINER_CHILDREN (element); tmp; tmp = tmp->next) {
+    GESTimelineElement *child = (GESTimelineElement *) tmp->data;
+
+    if (child != container->initiated_move &&
+        (_END (child) > _START (element) || _END (child) > start)) {
+      _set_start0 (child, _START (child) + diff);
+    }
   }
-
-  timeline = GES_TIMELINE_ELEMENT_TIMELINE (element);
-  if (timeline)
-    return ges_timeline_move_object_simple (timeline, element, NULL,
-        GES_EDGE_NONE, start);
+  container->children_control_mode = GES_CHILDREN_UPDATE;
 
   return TRUE;
 }
@@ -377,12 +384,6 @@ _set_duration (GESTimelineElement * element, GstClockTime duration)
     return GES_TIMELINE_ELEMENT_CLASS (parent_class)->set_duration (element,
         duration);
 
-  if (element->timeline
-      && !timeline_tree_can_move_element (timeline_get_tree (element->timeline),
-          element, _PRIORITY (element), element->start, duration, NULL)) {
-    return FALSE;
-  }
-
   if (container->initiated_move == NULL) {
     gboolean expending = (_DURATION (element) < duration);
 
@@ -438,12 +439,9 @@ _child_added (GESContainer * group, GESTimelineElement * child)
   GESGroupPrivate *priv = GES_GROUP (group)->priv;
   GstClockTime last_child_end = 0, first_child_start = G_MAXUINT64;
 
-  if (!GES_TIMELINE_ELEMENT_TIMELINE (group)
-      && GES_TIMELINE_ELEMENT_TIMELINE (child)) {
+  if (!GES_TIMELINE_ELEMENT_TIMELINE (group)) {
     timeline_add_group (GES_TIMELINE_ELEMENT_TIMELINE (child),
         GES_GROUP (group));
-    timeline_emit_group_added (GES_TIMELINE_ELEMENT_TIMELINE (child),
-        GES_GROUP (group));
   }
 
   children = GES_CONTAINER_CHILDREN (group);
@@ -455,7 +453,6 @@ _child_added (GESContainer * group, GESTimelineElement * child)
   }
 
   priv->setting_value = TRUE;
-  ELEMENT_SET_FLAG (group, GES_TIMELINE_ELEMENT_SET_SIMPLE);
   if (first_child_start != GES_TIMELINE_ELEMENT_START (group)) {
     group->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
     _set_start0 (GES_TIMELINE_ELEMENT (group), first_child_start);
@@ -465,7 +462,6 @@ _child_added (GESContainer * group, GESTimelineElement * child)
     _set_duration0 (GES_TIMELINE_ELEMENT (group),
         last_child_end - first_child_start);
   }
-  ELEMENT_UNSET_FLAG (group, GES_TIMELINE_ELEMENT_SET_SIMPLE);
   priv->setting_value = FALSE;
 
   group->children_control_mode = GES_CHILDREN_UPDATE;
@@ -540,21 +536,18 @@ _child_removed (GESContainer * group, GESTimelineElement * child)
   g_free (signals_ids_key);
   if (children == NULL) {
     GST_FIXME_OBJECT (group, "Auto destroy myself?");
-    if (GES_TIMELINE_ELEMENT_TIMELINE (group))
-      timeline_remove_group (GES_TIMELINE_ELEMENT_TIMELINE (group),
-          GES_GROUP (group));
+    timeline_remove_group (GES_TIMELINE_ELEMENT_TIMELINE (group),
+        GES_GROUP (group));
     return;
   }
 
   priv->setting_value = TRUE;
-  ELEMENT_SET_FLAG (group, GES_TIMELINE_ELEMENT_SET_SIMPLE);
   first_child_start = GES_TIMELINE_ELEMENT_START (children->data);
   if (first_child_start > GES_TIMELINE_ELEMENT_START (group)) {
     group->children_control_mode = GES_CHILDREN_IGNORE_NOTIFIES;
     _set_start0 (GES_TIMELINE_ELEMENT (group), first_child_start);
     group->children_control_mode = GES_CHILDREN_UPDATE;
   }
-  ELEMENT_UNSET_FLAG (group, GES_TIMELINE_ELEMENT_SET_SIMPLE);
   priv->setting_value = FALSE;
 }
 
@@ -580,7 +573,8 @@ _ungroup (GESContainer * group, gboolean recursive)
   }
 
   if (timeline)
-    timeline_emit_group_removed (timeline, (GESGroup *) group, children_array);
+    ges_timeline_emit_group_removed (timeline, (GESGroup *) group,
+        children_array);
   g_ptr_array_free (children_array, TRUE);
   g_list_free_full (children, gst_object_unref);
 
@@ -625,12 +619,9 @@ _paste (GESTimelineElement * element, GESTimelineElement * ref,
       paste_position);
 
   if (ngroup) {
-    if (GES_CONTAINER_CHILDREN (ngroup)) {
+    if (GES_CONTAINER_CHILDREN (ngroup))
       timeline_add_group (GES_TIMELINE_ELEMENT_TIMELINE (GES_CONTAINER_CHILDREN
               (ngroup)->data), GES_GROUP (element));
-      timeline_emit_group_added (GES_TIMELINE_ELEMENT_TIMELINE
-          (GES_CONTAINER_CHILDREN (ngroup)->data), GES_GROUP (element));
-    }
   }
 
   return ngroup;
@@ -703,6 +694,8 @@ ges_group_class_init (GESGroupClass * klass)
   GESContainerClass *container_class = GES_CONTAINER_CLASS (klass);
   GESTimelineElementClass *element_class = GES_TIMELINE_ELEMENT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GESGroupPrivate));
+
   object_class->get_property = ges_group_get_property;
   object_class->set_property = ges_group_set_property;
 
@@ -780,7 +773,8 @@ ges_group_class_init (GESGroupClass * klass)
 static void
 ges_group_init (GESGroup * self)
 {
-  self->priv = ges_group_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_GROUP, GESGroupPrivate);
 
   self->priv->setting_value = FALSE;
 }
@@ -798,7 +792,7 @@ ges_group_init (GESGroup * self)
  * together, it is recommanded to use the #ges_container_group method so the
  * proper subclass is selected.
  *
- * Returns: (transfer floating): The new empty group.
+ * Returns: The new empty group.
  */
 GESGroup *
 ges_group_new (void)
