@@ -20,8 +20,8 @@
 
 #ifndef __GES_INTERNAL_H__
 #define __GES_INTERNAL_H__
+
 #include <gst/gst.h>
-#include <gst/pbutils/encoding-profile.h>
 #include <gio/gio.h>
 
 #include "ges-timeline.h"
@@ -30,16 +30,9 @@
 
 #include "ges-asset.h"
 #include "ges-base-xml-formatter.h"
-#include "ges-timeline-tree.h"
 
-G_BEGIN_DECLS
-
-#ifndef GST_CAT_DEFAULT
-#define GST_CAT_DEFAULT (_ges_debug ())
-#endif
-
-G_GNUC_INTERNAL
-GstDebugCategory * _ges_debug (void);
+GST_DEBUG_CATEGORY_EXTERN (_ges_debug);
+#define GST_CAT_DEFAULT _ges_debug
 
 /*  The first 2 NLE priorities are used for:
  *    0- The Mixing element
@@ -53,75 +46,62 @@ GstDebugCategory * _ges_debug (void);
 #define _DURATION(obj) GES_TIMELINE_ELEMENT_DURATION (obj)
 #define _MAXDURATION(obj) GES_TIMELINE_ELEMENT_MAX_DURATION (obj)
 #define _PRIORITY(obj) GES_TIMELINE_ELEMENT_PRIORITY (obj)
-#ifndef _END
 #define _END(obj) (_START (obj) + _DURATION (obj))
-#endif
 #define _set_start0 ges_timeline_element_set_start
 #define _set_inpoint0 ges_timeline_element_set_inpoint
 #define _set_duration0 ges_timeline_element_set_duration
 #define _set_priority0 ges_timeline_element_set_priority
 
 #define GES_TIMELINE_ELEMENT_FORMAT \
-    "s<%p>" \
-    " [ %" GST_TIME_FORMAT \
-    " (%" GST_TIME_FORMAT \
-    ") - %" GST_TIME_FORMAT "(%" GST_TIME_FORMAT") layer: %" G_GINT32_FORMAT "] "
+    "s:%p" \
+    " start: %" GST_TIME_FORMAT \
+    " inpoint: %" GST_TIME_FORMAT \
+    " duration: %" GST_TIME_FORMAT
 
 #define GES_TIMELINE_ELEMENT_ARGS(element) \
     GES_TIMELINE_ELEMENT_NAME(element), element, \
     GST_TIME_ARGS(GES_TIMELINE_ELEMENT_START(element)), \
     GST_TIME_ARGS(GES_TIMELINE_ELEMENT_INPOINT(element)), \
-    GST_TIME_ARGS(GES_TIMELINE_ELEMENT_DURATION(element)), \
-    GST_TIME_ARGS(GES_TIMELINE_ELEMENT_MAX_DURATION(element)), \
-    GES_TIMELINE_ELEMENT_LAYER_PRIORITY(element)
-
-#define GES_FORMAT GES_TIMELINE_ELEMENT_FORMAT
-#define GES_ARGS GES_TIMELINE_ELEMENT_ARGS
+    GST_TIME_ARGS(GES_TIMELINE_ELEMENT_DURATION(element))
 
 G_GNUC_INTERNAL gboolean
-timeline_ripple_object         (GESTimeline *timeline, GESTimelineElement *obj,
-                                gint new_layer_priority,
-                                GList * layers, GESEdge edge,
-                                guint64 position);
+timeline_ripple_object         (GESTimeline *timeline, GESTrackElement *obj,
+                                    GList * layers, GESEdge edge,
+                                    guint64 position);
 
 G_GNUC_INTERNAL gboolean
 timeline_slide_object          (GESTimeline *timeline, GESTrackElement *obj,
                                     GList * layers, GESEdge edge, guint64 position);
 
 G_GNUC_INTERNAL gboolean
-timeline_roll_object           (GESTimeline *timeline, GESTimelineElement *obj,
-                                GList * layers, GESEdge edge, guint64 position);
+timeline_roll_object           (GESTimeline *timeline, GESTrackElement *obj,
+                                    GList * layers, GESEdge edge, guint64 position);
 
 G_GNUC_INTERNAL gboolean
-timeline_trim_object           (GESTimeline *timeline, GESTimelineElement * object,
-                                guint32 new_layer_priority, GList * layers, GESEdge edge,
-                                guint64 position);
+timeline_trim_object           (GESTimeline *timeline, GESTrackElement * object,
+                                    GList * layers, GESEdge edge, guint64 position);
 G_GNUC_INTERNAL gboolean
 ges_timeline_trim_object_simple (GESTimeline * timeline, GESTimelineElement * obj,
-                                 guint32 new_layer_priority, GList * layers, GESEdge edge,
-                                 guint64 position, gboolean snapping);
+                                 GList * layers, GESEdge edge, guint64 position, gboolean snapping);
 
 G_GNUC_INTERNAL gboolean
 ges_timeline_move_object_simple (GESTimeline * timeline, GESTimelineElement * object,
                                  GList * layers, GESEdge edge, guint64 position);
 
 G_GNUC_INTERNAL gboolean
-timeline_move_object           (GESTimeline *timeline, GESTimelineElement * object,
-                                guint32 new_layer_priority, GList * layers, GESEdge edge,
-                                guint64 position);
+timeline_move_object           (GESTimeline *timeline, GESTrackElement * object,
+                                    GList * layers, GESEdge edge, guint64 position);
+
+G_GNUC_INTERNAL gboolean
+timeline_context_to_layer      (GESTimeline *timeline, gint offset);
 
 G_GNUC_INTERNAL void
 timeline_add_group             (GESTimeline *timeline,
                                 GESGroup *group);
-G_GNUC_INTERNAL void
+G_GNUC_INTERNAL
+void
 timeline_remove_group          (GESTimeline *timeline,
                                 GESGroup *group);
-G_GNUC_INTERNAL void
-timeline_emit_group_added      (GESTimeline *timeline,
-                                GESGroup *group);
-G_GNUC_INTERNAL void
-timeline_emit_group_removed    (GESTimeline * timeline,
-                                GESGroup * group, GPtrArray * array);
 
 G_GNUC_INTERNAL
 gboolean
@@ -132,20 +112,12 @@ gboolean
 timeline_remove_element       (GESTimeline *timeline,
                                GESTimelineElement *element);
 
-G_GNUC_INTERNAL
-GNode *
-timeline_get_tree           (GESTimeline *timeline);
-
-G_GNUC_INTERNAL
-void
-timeline_update_transition (GESTimeline *timeline);
-
 G_GNUC_INTERNAL
 void
 timeline_fill_gaps            (GESTimeline *timeline);
 
-G_GNUC_INTERNAL void
-timeline_create_transitions (GESTimeline * timeline, GESTrackElement * track_element);
+G_GNUC_INTERNAL GList *
+timeline_get_groups           (GESTimeline * timeline);
 
 G_GNUC_INTERNAL
 void
@@ -158,9 +130,6 @@ track_disable_last_gap        (GESTrack *track, gboolean disabled);
 G_GNUC_INTERNAL void
 ges_asset_cache_init (void);
 
-G_GNUC_INTERNAL void
-ges_asset_cache_deinit (void);
-
 G_GNUC_INTERNAL void
 ges_asset_set_id (GESAsset *asset, const gchar *id);
 
@@ -170,10 +139,10 @@ ges_asset_cache_put (GESAsset * asset, GTask *task);
 G_GNUC_INTERNAL gboolean
 ges_asset_cache_set_loaded(GType extractable_type, const gchar * id, GError *error);
 
-GES_API GESAsset*
+GESAsset*
 ges_asset_cache_lookup(GType extractable_type, const gchar * id);
 
-GES_API gboolean
+gboolean
 ges_asset_try_proxy (GESAsset *asset, const gchar *new_id);
 
 G_GNUC_INTERNAL gboolean
@@ -184,10 +153,6 @@ ges_effect_assect_id_get_type_and_bindesc (const char    *id,
                                            GESTrackType  *track_type,
                                            GError       **error);
 
-G_GNUC_INTERNAL void _ges_uri_asset_cleanup (void);
-
-G_GNUC_INTERNAL gboolean _ges_uri_asset_ensure_setup (gpointer uriasset_class);
-
 /* GESExtractable internall methods
  *
  * FIXME Check if that should be public later
@@ -238,7 +203,6 @@ G_GNUC_INTERNAL  gchar * ges_project_try_updating_id              (GESProject *s
 G_GNUC_INTERNAL  void ges_project_add_loading_asset               (GESProject *project,
                                                                    GType extractable_type,
                                                                    const gchar *id);
-G_GNUC_INTERNAL  gchar* ges_uri_asset_try_update_id               (GError *error, GESAsset *wrong_asset);
 /************************************************
  *                                              *
  *   GESBaseXmlFormatter internal methods       *
@@ -257,7 +221,6 @@ G_GNUC_INTERNAL void ges_base_xml_formatter_add_clip (GESBaseXmlFormatter * self
                                                                  guint layer_prio,
                                                                  GESTrackType track_types,
                                                                  GstStructure *properties,
-                                                                 GstStructure * children_properties,
                                                                  const gchar *metadatas,
                                                                  GError **error);
 G_GNUC_INTERNAL void ges_base_xml_formatter_add_asset        (GESBaseXmlFormatter * self,
@@ -287,7 +250,6 @@ G_GNUC_INTERNAL void ges_base_xml_formatter_add_encoding_profile(GESBaseXmlForma
                                                                  const gchar * description,
                                                                  GstCaps * format,
                                                                  const gchar * preset,
-                                                                 GstStructure * preset_properties,
                                                                  const gchar * preset_name,
                                                                  guint id,
                                                                  guint presence,
@@ -313,8 +275,7 @@ G_GNUC_INTERNAL void ges_base_xml_formatter_add_source          (GESBaseXmlForma
 
 G_GNUC_INTERNAL void ges_base_xml_formatter_add_group           (GESBaseXmlFormatter *self,
                                                                  const gchar *name,
-                                                                 const gchar *properties,
-                                                                 const gchar *metadatas);
+                                                                 const gchar *properties);
 
 G_GNUC_INTERNAL void ges_base_xml_formatter_last_group_add_child(GESBaseXmlFormatter *self,
                                                                  const gchar * id,
@@ -330,10 +291,7 @@ G_GNUC_INTERNAL void ges_base_xml_formatter_add_control_binding (GESBaseXmlForma
 
 G_GNUC_INTERNAL gboolean set_property_foreach                   (GQuark field_id,
                                                                  const GValue * value,
-                                                                 GObject * object);
-
-G_GNUC_INTERNAL GstElement * get_element_for_encoding_profile   (GstEncodingProfile *prof,
-                                                                 GstElementFactoryListType type);
+                                                                 GObject * object);;
 
 /* Function to initialise GES */
 G_GNUC_INTERNAL void _init_standard_transition_assets        (void);
@@ -353,40 +311,36 @@ ges_base_xml_formatter_set_timeline_properties(GESBaseXmlFormatter * self,
 					       const gchar *properties,
 					       const gchar *metadatas);
 
+
+/****************************************************
+ *        GESCommandLineFormatter                   *
+ ****************************************************/
+G_GNUC_INTERNAL GOptionGroup *
+_ges_command_line_formatter_get_option_group (void);
+
 /****************************************************
  *              GESContainer                        *
  ****************************************************/
 G_GNUC_INTERNAL void _ges_container_sort_children         (GESContainer *container);
 G_GNUC_INTERNAL void _ges_container_sort_children_by_end  (GESContainer *container);
-G_GNUC_INTERNAL void _ges_container_set_height            (GESContainer * container,
-                                                           guint32 height);
-G_GNUC_INTERNAL gint  _ges_container_get_priority_offset  (GESContainer * container,
-                                                           GESTimelineElement *elem);
-G_GNUC_INTERNAL void _ges_container_set_priority_offset   (GESContainer * container,
-                                                           GESTimelineElement *elem,
-                                                           gint32 priority_offset);
-
 
 /****************************************************
  *                  GESClip                         *
  ****************************************************/
 G_GNUC_INTERNAL void              ges_clip_set_layer              (GESClip *clip, GESLayer  *layer);
 G_GNUC_INTERNAL gboolean          ges_clip_is_moving_from_layer   (GESClip *clip);
+G_GNUC_INTERNAL guint32           ges_clip_get_layer_priority     (GESClip *clip);
 G_GNUC_INTERNAL void              ges_clip_set_moving_from_layer  (GESClip *clip, gboolean is_moving);
 G_GNUC_INTERNAL GESTrackElement*  ges_clip_create_track_element   (GESClip *clip, GESTrackType type);
 G_GNUC_INTERNAL GList*            ges_clip_create_track_elements  (GESClip *clip, GESTrackType type);
 
-/****************************************************
- *              GESLayer                            *
- ****************************************************/
-G_GNUC_INTERNAL gboolean ges_layer_resync_priorities (GESLayer * layer);
-G_GNUC_INTERNAL void layer_set_priority               (GESLayer * layer, guint priority, gboolean emit);
 
 /****************************************************
  *              GESTrackElement                     *
  ****************************************************/
 #define         NLE_OBJECT_TRACK_ELEMENT_QUARK                  (g_quark_from_string ("nle_object_track_element_quark"))
 G_GNUC_INTERNAL gboolean  ges_track_element_set_track           (GESTrackElement * object, GESTrack * track);
+G_GNUC_INTERNAL guint32   _ges_track_element_get_layer_priority (GESTrackElement * element);
 G_GNUC_INTERNAL void ges_track_element_copy_properties          (GESTimelineElement * element,
                                                                  GESTimelineElement * elementcopy);
 
@@ -413,21 +367,7 @@ G_GNUC_INTERNAL GESVideoTestSource * ges_video_test_source_new (void);
 /****************************************************
  *              GESTimelineElement                  *
  ****************************************************/
-typedef enum
-{
-  GES_CLIP_IS_MOVING = (1 << 0),
-  GES_TIMELINE_ELEMENT_SET_SIMPLE = (1 << 1),
-} GESTimelineElementFlags;
-
 G_GNUC_INTERNAL gdouble ges_timeline_element_get_media_duration_factor(GESTimelineElement *self);
-G_GNUC_INTERNAL GESTimelineElement * ges_timeline_element_get_copied_from (GESTimelineElement *self);
-G_GNUC_INTERNAL GESTimelineElementFlags ges_timeline_element_flags (GESTimelineElement *self);
-G_GNUC_INTERNAL void                ges_timeline_element_set_flags (GESTimelineElement *self, GESTimelineElementFlags flags);
-
-#define ELEMENT_FLAGS(obj)             (ges_timeline_element_flags (GES_TIMELINE_ELEMENT(obj)))
-#define ELEMENT_SET_FLAG(obj,flag)     (ges_timeline_element_set_flags(GES_TIMELINE_ELEMENT(obj), (ELEMENT_FLAGS(obj) | (flag))))
-#define ELEMENT_UNSET_FLAG(obj,flag)   (ges_timeline_element_set_flags(GES_TIMELINE_ELEMENT(obj), (ELEMENT_FLAGS(obj) & ~(flag))))
-#define ELEMENT_FLAG_IS_SET(obj,flag)  ((ELEMENT_FLAGS (obj) & (flag)) == (flag))
 
 /******************************
  *  GESMultiFile internal API *
@@ -454,6 +394,4 @@ G_GNUC_INTERNAL gboolean ges_nle_composition_add_object (GstElement *comp, GstEl
 G_GNUC_INTERNAL gboolean ges_nle_composition_remove_object (GstElement *comp, GstElement *object);
 G_GNUC_INTERNAL gboolean ges_nle_object_commit (GstElement * nlesource, gboolean recurse);
 
-G_END_DECLS
-
 #endif /* __GES_INTERNAL_H__ */
