@@ -19,7 +19,6 @@
  */
 /**
  * SECTION: gesproject
- * @title: GESProject
  * @short_description: A GESAsset that is used to manage projects
  *
  * The #GESProject is used to control a set of #GESAsset and is a
@@ -44,10 +43,6 @@
  * a set of signals. Also it handles problem such as missing files/missing
  * #GstElement and lets you try to recover from those.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "ges.h"
 #include "ges-internal.h"
 
@@ -57,6 +52,8 @@ static GHashTable *tried_uris = NULL;
 /* TODO We should rely on both extractable_type and @id to identify
  * a Asset, not only @id
  */
+G_DEFINE_TYPE (GESProject, ges_project, GES_TYPE_ASSET);
+
 struct _GESProjectPrivate
 {
   GHashTable *assets;
@@ -89,8 +86,6 @@ enum
   LAST_SIGNAL
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GESProject, ges_project, GES_TYPE_ASSET);
-
 static guint _signals[LAST_SIGNAL] = { 0 };
 
 static guint nb_projects = 0;
@@ -317,13 +312,6 @@ ges_missing_uri_default (GESProject * self, GError * error,
 {
   guint i;
   const gchar *old_uri = ges_asset_get_id (wrong_asset);
-  gchar *new_id = NULL;
-
-  if (ges_asset_request_id_update (wrong_asset, &new_id, error) && new_id) {
-    GST_INFO_OBJECT (self, "Returned guessed new ID: %s", new_id);
-
-    return new_id;
-  }
 
   if (new_paths == NULL)
     return NULL;
@@ -353,12 +341,6 @@ ges_missing_uri_default (GESProject * self, GError * error,
   return NULL;
 }
 
-gchar *
-ges_uri_asset_try_update_id (GError * error, GESAsset * wrong_asset)
-{
-  return ges_missing_uri_default (NULL, error, wrong_asset);
-}
-
 static void
 ges_uri_assets_validate_uri (const gchar * nid)
 {
@@ -370,6 +352,7 @@ ges_uri_assets_validate_uri (const gchar * nid)
 static void
 _dispose (GObject * object)
 {
+  GList *tmp;
   GESProjectPrivate *priv = GES_PROJECT (object)->priv;
 
   if (priv->assets)
@@ -381,8 +364,8 @@ _dispose (GObject * object)
   if (priv->formatter_asset)
     gst_object_unref (priv->formatter_asset);
 
-  while (priv->formatters)
-    ges_project_remove_formatter (GES_PROJECT (object), priv->formatters->data);
+  for (tmp = priv->formatters; tmp; tmp = tmp->next)
+    ges_project_remove_formatter (GES_PROJECT (object), tmp->data);;
 
   G_OBJECT_CLASS (ges_project_parent_class)->dispose (object);
 }
@@ -394,7 +377,6 @@ _finalize (GObject * object)
 
   if (priv->uri)
     g_free (priv->uri);
-  g_list_free_full (priv->encoding_profiles, g_object_unref);
 
   G_OBJECT_CLASS (ges_project_parent_class)->finalize (object);
 }
@@ -432,6 +414,8 @@ ges_project_class_init (GESProjectClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GESProjectPrivate));
+
   klass->asset_added = NULL;
   klass->missing_uri = ges_missing_uri_default;
   klass->loading_error = NULL;
@@ -528,7 +512,7 @@ ges_project_class_init (GESProjectClass * klass)
   /**
    * GESProject::error-loading-asset:
    * @project: the #GESProject on which a problem happend when creted a #GESAsset
-   * @error: The #GError defining the error that occured, might be %NULL
+   * @error: The #GError defining the error that accured, might be %NULL
    * @id: The @id of the asset that failed loading
    * @extractable_type: The @extractable_type of the asset that
    * failed loading
@@ -553,7 +537,8 @@ static void
 ges_project_init (GESProject * project)
 {
   GESProjectPrivate *priv = project->priv =
-      ges_project_get_instance_private (project);
+      G_TYPE_INSTANCE_GET_PRIVATE (project,
+      GES_TYPE_PROJECT, GESProjectPrivate);
 
   priv->uri = NULL;
   priv->formatters = NULL;
@@ -602,10 +587,11 @@ ges_project_try_updating_id (GESProject * project, GESAsset * asset,
     return NULL;
   }
 
-  /* Always send the MISSING_URI signal if requesting new ID is possible
-   * so that subclasses of GESProject are aware of the missing-uri */
-  g_signal_emit (project, _signals[MISSING_URI_SIGNAL], 0, error, asset,
-      &new_id);
+  if (new_id == NULL) {
+    GST_DEBUG_OBJECT (project, "Sending 'missing-uri' signal for %s", id);
+    g_signal_emit (project, _signals[MISSING_URI_SIGNAL], 0, error, asset,
+        &new_id);
+  }
 
   if (new_id) {
     GST_DEBUG_OBJECT (project, "new id found: %s", new_id);
@@ -635,7 +621,6 @@ new_asset_cb (GESAsset * source, GAsyncResult * res, GESProject * project)
 
   if (error) {
     possible_id = ges_project_try_updating_id (project, source, error);
-    g_clear_error (&error);
 
     if (possible_id == NULL)
       return;
@@ -644,6 +629,7 @@ new_asset_cb (GESAsset * source, GAsyncResult * res, GESProject * project)
         ges_asset_get_extractable_type (source));
 
     g_free (possible_id);
+    g_error_free (error);
     return;
   }
 
@@ -746,7 +732,7 @@ ges_project_create_asset (GESProject * project, const gchar * id,
  * "asset-added" signal to get the asset when it finally gets added to
  * @project
  *
- * Returns: (transfer full) (nullable): The newly created #GESAsset or %NULL.
+ * Returns: (transfer full): The newly created #GESAsset
  */
 GESAsset *
 ges_project_create_asset_sync (GESProject * project, const gchar * id,
@@ -795,17 +781,16 @@ ges_project_create_asset_sync (GESProject * project, const gchar * id,
       GESAsset *tmpasset;
 
       tmpasset = ges_asset_cache_lookup (extractable_type, id);
+      g_signal_emit (project, _signals[ASSET_LOADING_SIGNAL], 0, tmpasset);
+      g_signal_emit (project, _signals[ERROR_LOADING_ASSET], 0, *error, id,
+          extractable_type);
       possible_id = ges_project_try_updating_id (project, tmpasset, *error);
 
-      if (possible_id == NULL) {
-        g_signal_emit (project, _signals[ASSET_LOADING_SIGNAL], 0, tmpasset);
-        g_signal_emit (project, _signals[ERROR_LOADING_ASSET], 0, *error, id,
-            extractable_type);
+      if (possible_id == NULL)
         return NULL;
-      }
 
-
-      g_clear_error (error);
+      if (error)
+        g_clear_error (error);
 
       id = possible_id;
     }
@@ -1074,7 +1059,7 @@ ges_project_load (GESProject * project, GESTimeline * timeline, GError ** error)
  *
  * Retrieve the uri that is currently set on @project
  *
- * Returns: (transfer full) (nullable): a newly allocated string representing uri.
+ * Returns: The uri that is set on @project
  */
 gchar *
 ges_project_get_uri (GESProject * project)
