@@ -18,7 +18,6 @@
 
 /**
  * SECTION:gestimelineelement
- * @title: GESTimelineElement
  * @short_description: Base Class for all elements that will be in a way or
  * another inside a GESTimeline.
  *
@@ -26,9 +25,6 @@
  * as priority. A GESTimelineElement can have a parent object which will be
  * responsible for controlling its timing properties.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-utils.h"
 #include "ges-timeline-element.h"
@@ -55,6 +51,11 @@ ges_extractable_interface_init (GESExtractableInterface * iface)
   iface->set_asset = extractable_set_asset;
 }
 
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (GESTimelineElement, ges_timeline_element,
+    G_TYPE_INITIALLY_UNOWNED,
+    G_IMPLEMENT_INTERFACE (GES_TYPE_EXTRACTABLE, ges_extractable_interface_init)
+    G_IMPLEMENT_INTERFACE (GES_TYPE_META_CONTAINER, NULL));
+
 enum
 {
   PROP_0,
@@ -96,29 +97,8 @@ struct _GESTimelineElementPrivate
   GHashTable *children_props;
 
   GESTimelineElement *copied_from;
-
-  GESTimelineElementFlags flags;
 };
 
-typedef struct
-{
-  GObject *child;
-  GParamSpec *arg;
-  GESTimelineElement *self;
-} EmitDeepNotifyInIdleData;
-
-G_DEFINE_ABSTRACT_TYPE_WITH_CODE (GESTimelineElement, ges_timeline_element,
-    G_TYPE_INITIALLY_UNOWNED, G_ADD_PRIVATE (GESTimelineElement)
-    G_IMPLEMENT_INTERFACE (GES_TYPE_EXTRACTABLE, ges_extractable_interface_init)
-    G_IMPLEMENT_INTERFACE (GES_TYPE_META_CONTAINER, NULL));
-
-static void
-_set_child_property (GESTimelineElement * self G_GNUC_UNUSED, GObject * child,
-    GParamSpec * pspec, GValue * value)
-{
-  g_object_set_property (child, pspec->name, value);
-}
-
 static gboolean
 _lookup_child (GESTimelineElement * self, const gchar * prop_name,
     GObject ** child, GParamSpec ** pspec)
@@ -144,8 +124,6 @@ _lookup_child (GESTimelineElement * self, const gchar * prop_name,
       ChildPropHandler *handler = (ChildPropHandler *) value;
       if (classename == NULL ||
           g_strcmp0 (G_OBJECT_TYPE_NAME (G_OBJECT (handler->child)),
-              classename) == 0 ||
-          g_strcmp0 (g_type_name (G_PARAM_SPEC (key)->owner_type),
               classename) == 0) {
         GST_DEBUG_OBJECT (self, "The %s property from %s has been found", name,
             classename);
@@ -270,14 +248,10 @@ ges_timeline_element_dispose (GObject * object)
 {
   GESTimelineElement *self = GES_TIMELINE_ELEMENT (object);
 
-  if (self->priv->children_props) {
+  if (self->priv->children_props)
     g_hash_table_unref (self->priv->children_props);
-    self->priv->children_props = NULL;
-  }
 
   g_clear_object (&self->priv->copied_from);
-
-  G_OBJECT_CLASS (ges_timeline_element_parent_class)->dispose (object);
 }
 
 static void
@@ -304,7 +278,8 @@ _child_prop_handler_free (ChildPropHandler * handler)
 static void
 ges_timeline_element_init (GESTimelineElement * self)
 {
-  self->priv = ges_timeline_element_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_TIMELINE_ELEMENT, GESTimelineElementPrivate);
 
   self->priv->serialize = TRUE;
 
@@ -319,6 +294,8 @@ ges_timeline_element_class_init (GESTimelineElementClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GESTimelineElementPrivate));
+
   object_class->get_property = _get_property;
   object_class->set_property = _set_property;
 
@@ -385,9 +362,6 @@ ges_timeline_element_class_init (GESTimelineElementClass * klass)
    * GESTimelineElement:priority:
    *
    * The priority of the object.
-   *
-   * Setting GESTimelineElement priorities is deprecated
-   * as all priority management is done by GES itself now.
    */
   properties[PROP_PRIORITY] = g_param_spec_uint ("priority", "Priority",
       "The priority of the object", 0, G_MAXUINT, 0, G_PARAM_READWRITE);
@@ -445,7 +419,6 @@ ges_timeline_element_class_init (GESTimelineElementClass * klass)
 
   klass->list_children_properties = default_list_children_properties;
   klass->lookup_child = _lookup_child;
-  klass->set_child_property = _set_child_property;
 }
 
 static void
@@ -527,8 +500,8 @@ _set_name (GESTimelineElement * self, const gchar * wanted_name)
  * @self: a #GESTimelineElement
  * @parent: new parent of self
  *
- * Sets the parent of @self to @parent. The parents needs to already
- * own a hard reference on @self.
+ * Sets the parent of @self to @parent. The object's reference count will
+ * be incremented, and any floating reference will be removed (see gst_object_ref_sink()).
  *
  * Returns: %TRUE if @parent could be set or %FALSE when @self
  * already had a parent or @self and @parent are the same.
@@ -544,8 +517,7 @@ ges_timeline_element_set_parent (GESTimelineElement * self,
   if (self == parent) {
     GST_INFO_OBJECT (self, "Trying to add %p in itself, not a good idea!",
         self);
-    gst_object_ref_sink (self);
-    gst_object_unref (self);
+
     return FALSE;
   }
 
@@ -568,8 +540,6 @@ ges_timeline_element_set_parent (GESTimelineElement * self,
 had_parent:
   {
     GST_WARNING_OBJECT (self, "set parent failed, object already had a parent");
-    gst_object_ref_sink (self);
-    gst_object_unref (self);
     return FALSE;
   }
 }
@@ -581,8 +551,8 @@ had_parent:
  * Returns the parent of @self. This function increases the refcount
  * of the parent object so you should gst_object_unref() it after usage.
  *
- * Returns: (transfer full) (nullable): parent of @self, this can be %NULL if
- * @self has no parent. unref after usage.
+ * Returns: (transfer full): parent of @self, this can be %NULL if @self
+ *   has no parent. unref after usage.
  */
 GESTimelineElement *
 ges_timeline_element_get_parent (GESTimelineElement * self)
@@ -657,8 +627,8 @@ had_timeline:
  * Returns the timeline of @self. This function increases the refcount
  * of the timeline so you should gst_object_unref() it after usage.
  *
- * Returns: (transfer full) (nullable): timeline of @self, this can be %NULL if
- * @self has no timeline. unref after usage.
+ * Returns: (transfer full): timeline of @self, this can be %NULL if @self
+ *   has no timeline. unref after usage.
  */
 GESTimeline *
 ges_timeline_element_get_timeline (GESTimelineElement * self)
@@ -679,23 +649,18 @@ ges_timeline_element_get_timeline (GESTimelineElement * self)
  * @self: a #GESTimelineElement
  * @start: the position in #GstClockTime
  *
- * Set the position of the object in its containing layer.
+ * Set the position of the object in its containing layer
  *
- * Note that if the snapping-distance property of the timeline containing
- * @self is set, @self will properly snap to the edges around @start.
- *
- * Returns: %TRUE if @start could be set.
+ * Note that if the timeline snap-distance property of the timeline containing
+ * @self is set, @self will properly snap to its neighboors.
  */
-gboolean
+void
 ges_timeline_element_set_start (GESTimelineElement * self, GstClockTime start)
 {
   GESTimelineElementClass *klass;
-  GESTimelineElement *toplevel_container, *parent;
+  GESTimelineElement *toplevel_container;
 
-  g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self), FALSE);
-
-  if (self->start == start)
-    return TRUE;
+  g_return_if_fail (GES_IS_TIMELINE_ELEMENT (self));
 
   klass = GES_TIMELINE_ELEMENT_GET_CLASS (self);
 
@@ -704,38 +669,30 @@ ges_timeline_element_set_start (GESTimelineElement * self, GstClockTime start)
       GST_TIME_ARGS (GES_TIMELINE_ELEMENT_START (self)), GST_TIME_ARGS (start));
 
   toplevel_container = ges_timeline_element_get_toplevel_parent (self);
-  parent = self->parent;
 
-  /* FIXME This should not belong to GESTimelineElement */
-  if (toplevel_container &&
-      ((gint64) (_START (toplevel_container) + start - _START (self))) < 0 &&
-      parent
-      && GES_CONTAINER (parent)->children_control_mode == GES_CHILDREN_UPDATE) {
-    GST_INFO_OBJECT (self,
-        "Can not move the object as it would imply its "
+  if (((gint64) (_START (toplevel_container) + start - _START (self))) < 0) {
+    GST_INFO_OBJECT (self, "Can not move the object as it would imply its"
         "container to have a negative start value");
 
     gst_object_unref (toplevel_container);
-    return FALSE;
+    return;
   }
 
   gst_object_unref (toplevel_container);
   if (klass->set_start) {
-    gboolean res = klass->set_start (self, start);
-    if (res) {
+    if (klass->set_start (self, start)) {
       self->start = start;
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_START]);
     }
 
     GST_DEBUG_OBJECT (self, "New start: %" GST_TIME_FORMAT,
         GST_TIME_ARGS (GES_TIMELINE_ELEMENT_START (self)));
-    return res;
+    return;
   }
 
   GST_WARNING_OBJECT (self, "No set_start virtual method implementation"
       " on class %s. Can not set start %" GST_TIME_FORMAT,
       G_OBJECT_CLASS_NAME (klass), GST_TIME_ARGS (start));
-  return FALSE;
 }
 
 /**
@@ -745,16 +702,14 @@ ges_timeline_element_set_start (GESTimelineElement * self, GstClockTime start)
  *
  * Set the in-point, that is the moment at which the @self will start
  * outputting data from its contents.
- *
- * Returns: %TRUE if @inpoint could be set.
  */
-gboolean
+void
 ges_timeline_element_set_inpoint (GESTimelineElement * self,
     GstClockTime inpoint)
 {
   GESTimelineElementClass *klass;
 
-  g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self), FALSE);
+  g_return_if_fail (GES_IS_TIMELINE_ELEMENT (self));
 
   GST_DEBUG_OBJECT (self, "current inpoint: %" GST_TIME_FORMAT
       " new inpoint: %" GST_TIME_FORMAT, GST_TIME_ARGS (inpoint),
@@ -763,20 +718,17 @@ ges_timeline_element_set_inpoint (GESTimelineElement * self,
   klass = GES_TIMELINE_ELEMENT_GET_CLASS (self);
 
   if (klass->set_inpoint) {
-    gboolean res = klass->set_inpoint (self, inpoint);
-    if (res) {
+    if (klass->set_inpoint (self, inpoint)) {
       self->inpoint = inpoint;
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_INPOINT]);
     }
 
-    return res;
+    return;
   }
 
-  GST_DEBUG_OBJECT (self, "No set_inpoint virtual method implementation"
+  GST_WARNING_OBJECT (self, "No set_inpoint virtual method implementation"
       " on class %s. Can not set inpoint %" GST_TIME_FORMAT,
       G_OBJECT_CLASS_NAME (klass), GST_TIME_ARGS (inpoint));
-
-  return FALSE;
 }
 
 /**
@@ -785,16 +737,14 @@ ges_timeline_element_set_inpoint (GESTimelineElement * self,
  * @maxduration: the maximum duration in #GstClockTime
  *
  * Set the maximun duration of the object
- *
- * Returns: %TRUE if @maxduration could be set.
  */
-gboolean
+void
 ges_timeline_element_set_max_duration (GESTimelineElement * self,
     GstClockTime maxduration)
 {
   GESTimelineElementClass *klass;
 
-  g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self), FALSE);
+  g_return_if_fail (GES_IS_TIMELINE_ELEMENT (self));
 
   klass = GES_TIMELINE_ELEMENT_GET_CLASS (self);
 
@@ -805,12 +755,11 @@ ges_timeline_element_set_max_duration (GESTimelineElement * self,
 
   if (klass->set_max_duration) {
     if (klass->set_max_duration (self, maxduration) == FALSE)
-      return FALSE;
+      return;
   }
 
   self->maxduration = maxduration;
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_MAX_DURATION]);
-  return TRUE;
 }
 
 /**
@@ -822,16 +771,14 @@ ges_timeline_element_set_max_duration (GESTimelineElement * self,
  *
  * Note that if the timeline snap-distance property of the timeline containing
  * @self is set, @self will properly snap to its neighboors.
- *
- * Returns: %TRUE if @duration could be set.
  */
-gboolean
+void
 ges_timeline_element_set_duration (GESTimelineElement * self,
     GstClockTime duration)
 {
   GESTimelineElementClass *klass;
 
-  g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self), FALSE);
+  g_return_if_fail (GES_IS_TIMELINE_ELEMENT (self));
 
   klass = GES_TIMELINE_ELEMENT_GET_CLASS (self);
 
@@ -841,19 +788,17 @@ ges_timeline_element_set_duration (GESTimelineElement * self,
       GST_TIME_ARGS (duration));
 
   if (klass->set_duration) {
-    gboolean res = klass->set_duration (self, duration);
-    if (res) {
+    if (klass->set_duration (self, duration)) {
       self->duration = duration;
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_DURATION]);
     }
 
-    return res;
+    return;
   }
 
   GST_WARNING_OBJECT (self, "No set_duration virtual method implementation"
       " on class %s. Can not set duration %" GST_TIME_FORMAT,
       G_OBJECT_CLASS_NAME (klass), GST_TIME_ARGS (duration));
-  return FALSE;
 }
 
 /**
@@ -932,19 +877,13 @@ ges_timeline_element_get_priority (GESTimelineElement * self)
  * @priority: the priority
  *
  * Sets the priority of the object within the containing layer
- *
- * Deprecated: All priority management is done by GES itself now.
- * To set #GESEffect priorities #ges_clip_set_top_effect_index should
- * be used.
- *
- * Returns: %TRUE if @priority could be set.
  */
-gboolean
+void
 ges_timeline_element_set_priority (GESTimelineElement * self, guint32 priority)
 {
   GESTimelineElementClass *klass;
 
-  g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self), FALSE);
+  g_return_if_fail (GES_IS_TIMELINE_ELEMENT (self));
 
   klass = GES_TIMELINE_ELEMENT_GET_CLASS (self);
 
@@ -952,19 +891,16 @@ ges_timeline_element_set_priority (GESTimelineElement * self, guint32 priority)
       self->priority, priority);
 
   if (klass->set_priority) {
-    gboolean res = klass->set_priority (self, priority);
-    if (res) {
+    if (klass->set_priority (self, priority)) {
       self->priority = priority;
       g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_PRIORITY]);
     }
-
-    return res;
+    return;
   }
 
   GST_WARNING_OBJECT (self, "No set_priority virtual method implementation"
       " on class %s. Can not set priority %d", G_OBJECT_CLASS_NAME (klass),
       priority);
-  return FALSE;
 }
 
 /**
@@ -1175,31 +1111,7 @@ ges_timeline_element_copy (GESTimelineElement * self, gboolean deep)
     }
   }
 
-#if GLIB_CHECK_VERSION(2, 53, 1)
-  {
-    gint i;
-    GValue *values;
-    const gchar **names;
-    values = g_malloc0 (sizeof (GValue) * n_specs);
-    names = g_malloc0 (sizeof (gchar *) * n_specs);
-
-    for (i = 0; i < n_params; i++) {
-      values[i] = params[i].value;
-      names[i] = params[i].name;
-    }
-
-    ret =
-        GES_TIMELINE_ELEMENT (g_object_new_with_properties (G_OBJECT_TYPE
-            (self), n_params, names, values));
-    g_free (names);
-    g_free (values);
-  }
-#else
   ret = g_object_newv (G_OBJECT_TYPE (self), n_params, params);
-#endif
-
-  while (n_params--)
-    g_value_unset (&params[n_params].value);
 
   g_free (specs);
   g_free (params);
@@ -1248,6 +1160,7 @@ ges_timeline_element_get_toplevel_parent (GESTimelineElement * self)
  * ges_timeline_element_get_name:
  * @self: a #GESTimelineElement
  *
+ *
  * Returns a copy of the name of @self.
  * Caller should g_free() the return value after usage.
  *
@@ -1266,6 +1179,7 @@ ges_timeline_element_get_name (GESTimelineElement * self)
  * @self: a #GESTimelineElement
  * @name: (allow-none): The name @self should take (if avalaible<)
  *
+ *
  * Sets the name of object, or gives @self a guaranteed unique name (if name is NULL).
  * This function makes a copy of the provided name, so the caller retains ownership
  * of the name it sent.
@@ -1311,59 +1225,30 @@ had_timeline:
   }
 }
 
-static gboolean
-emit_deep_notify_in_idle (EmitDeepNotifyInIdleData * data)
-{
-  g_signal_emit (data->self, ges_timeline_element_signals[DEEP_NOTIFY], 0,
-      data->child, data->arg);
-
-  gst_object_unref (data->child);
-  g_param_spec_unref (data->arg);
-  gst_object_unref (data->self);
-  g_slice_free (EmitDeepNotifyInIdleData, data);
-
-  return FALSE;
-}
-
 static void
 child_prop_changed_cb (GObject * child, GParamSpec * arg
     G_GNUC_UNUSED, GESTimelineElement * self)
 {
-  EmitDeepNotifyInIdleData *data;
-
-  /* Emit "deep-notify" right away if in main thread */
-  if (g_main_context_acquire (g_main_context_default ())) {
-    g_main_context_release (g_main_context_default ());
-    g_signal_emit (self, ges_timeline_element_signals[DEEP_NOTIFY], 0,
-        child, arg);
-    return;
-  }
-
-  data = g_slice_new (EmitDeepNotifyInIdleData);
-
-  data->child = gst_object_ref (child);
-  data->arg = g_param_spec_ref (arg);
-  data->self = gst_object_ref (self);
-
-  g_idle_add ((GSourceFunc) emit_deep_notify_in_idle, data);
+  g_signal_emit (self, ges_timeline_element_signals[DEEP_NOTIFY], 0,
+      child, arg);
 }
 
 gboolean
 ges_timeline_element_add_child_property (GESTimelineElement * self,
     GParamSpec * pspec, GObject * child)
 {
-  gchar *signame;
+  gchar *signame = g_strconcat ("notify::", pspec->name, NULL);
   ChildPropHandler *handler;
 
   if (g_hash_table_contains (self->priv->children_props, pspec)) {
     GST_INFO_OBJECT (self, "Child property already exists: %s", pspec->name);
+
     return FALSE;
   }
 
   GST_DEBUG_OBJECT (self, "Adding child property: %" GST_PTR_FORMAT "::%s",
       child, pspec->name);
 
-  signame = g_strconcat ("notify::", pspec->name, NULL);
   handler = (ChildPropHandler *) g_slice_new0 (ChildPropHandler);
   handler->child = gst_object_ref (child);
   handler->handler_id =
@@ -1418,21 +1303,16 @@ not_found:
  */
 void
 ges_timeline_element_set_child_property_by_pspec (GESTimelineElement * self,
-    GParamSpec * pspec, const GValue * value)
+    GParamSpec * pspec, GValue * value)
 {
-  ChildPropHandler *handler;
-  GESTimelineElementClass *klass;
+  GObject *child;
 
   g_return_if_fail (GES_IS_TRACK_ELEMENT (self));
 
-  handler = g_hash_table_lookup (self->priv->children_props, pspec);
-
-  if (!handler)
+  if (!ges_timeline_element_lookup_child (self, pspec->name, &child, &pspec))
     goto not_found;
 
-  klass = GES_TIMELINE_ELEMENT_GET_CLASS (self);
-  g_assert (klass->set_child_property);
-  klass->set_child_property (self, handler->child, pspec, (GValue *) value);
+  g_object_set_property (child, pspec->name, value);
 
   return;
 
@@ -1459,10 +1339,9 @@ not_found:
  */
 gboolean
 ges_timeline_element_set_child_property (GESTimelineElement * self,
-    const gchar * property_name, const GValue * value)
+    const gchar * property_name, GValue * value)
 {
   GParamSpec *pspec;
-  GESTimelineElementClass *klass;
   GObject *child;
 
   g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self), FALSE);
@@ -1470,9 +1349,7 @@ ges_timeline_element_set_child_property (GESTimelineElement * self,
   if (!ges_timeline_element_lookup_child (self, property_name, &child, &pspec))
     goto not_found;
 
-  klass = GES_TIMELINE_ELEMENT_GET_CLASS (self);
-  g_assert (klass->set_child_property);
-  klass->set_child_property (self, child, pspec, (GValue *) value);
+  g_object_set_property (child, pspec->name, value);
 
   gst_object_unref (child);
   g_param_spec_unref (pspec);
@@ -1614,7 +1491,6 @@ ges_timeline_element_set_child_property_valist (GESTimelineElement * self,
     g_object_set_property (child, pspec->name, &value);
 
     gst_object_unref (child);
-    g_param_spec_unref (pspec);
     g_value_unset (&value);
 
     name = va_arg (var_args, gchar *);
@@ -1631,8 +1507,6 @@ cant_copy:
     GST_WARNING_OBJECT (self, "error copying value %s in %p: %s", pspec->name,
         self, error);
 
-    gst_object_unref (child);
-    g_param_spec_unref (pspec);
     g_value_unset (&value);
     return;
   }
@@ -1698,7 +1572,6 @@ ges_timeline_element_get_child_property_valist (GESTimelineElement * self,
     g_value_init (&value, pspec->value_type);
     g_object_get_property (child, pspec->name, &value);
     gst_object_unref (child);
-    g_param_spec_unref (pspec);
 
     G_VALUE_LCOPY (&value, var_args, 0, &error);
     if (error)
@@ -1825,8 +1698,7 @@ ges_timeline_element_get_track_types (GESTimelineElement * self)
  * using ges_timeline_element_copy with recurse=TRUE set,
  * otherwise it will fail.
  *
- * Returns: (transfer full): New element resulting of pasting @self
- * or %NULL
+ * Returns: (transfer none): Paste @self copying the element
  *
  * Since: 1.6.0
  */
@@ -1854,29 +1726,7 @@ ges_timeline_element_paste (GESTimelineElement * self,
 
   g_clear_object (&self->priv->copied_from);
 
-  return res ? g_object_ref (res) : res;
-}
-
-/**
- * ges_timeline_element_get_layer_priority:
- * @self: A #GESTimelineElement
- *
- * Returns: The priority of the first layer the element is in (note that only
- * groups can span over several layers). %GES_TIMELINE_ELEMENT_NO_LAYER_PRIORITY
- * means that the element is not in a layer.
- *
- * Since: 1.16
- */
-guint32
-ges_timeline_element_get_layer_priority (GESTimelineElement * self)
-{
-  g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self),
-      GES_TIMELINE_ELEMENT_NO_LAYER_PRIORITY);
-
-  if (!GES_TIMELINE_ELEMENT_GET_CLASS (self)->get_layer_priority)
-    return self->priority;
-
-  return GES_TIMELINE_ELEMENT_GET_CLASS (self)->get_layer_priority (self);
+  return g_object_ref (res);
 }
 
 /* Internal */
@@ -1922,26 +1772,3 @@ ges_timeline_element_get_media_duration_factor (GESTimelineElement * self)
   g_type_class_unref (class);
   return media_duration_factor;
 }
-
-/* Internal */
-GESTimelineElement *
-ges_timeline_element_get_copied_from (GESTimelineElement * self)
-{
-  GESTimelineElement *copied_from = self->priv->copied_from;
-  self->priv->copied_from = NULL;
-  return copied_from;
-}
-
-GESTimelineElementFlags
-ges_timeline_element_flags (GESTimelineElement * self)
-{
-  return self->priv->flags;
-}
-
-void
-ges_timeline_element_set_flags (GESTimelineElement * self,
-    GESTimelineElementFlags flags)
-{
-  self->priv->flags = flags;
-
-}
