@@ -20,15 +20,11 @@
 
 /**
  * SECTION:gestitleclip
- * @title: GESTitleClip
  * @short_description: Render stand-alone titles in  GESLayer.
  *
  * Renders the given text in the specified font, at specified position, and
  * with the specified background pattern.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-internal.h"
 #include "ges-title-clip.h"
@@ -37,6 +33,8 @@
 #include "ges-title-source.h"
 #include <string.h>
 
+G_DEFINE_TYPE (GESTitleClip, ges_title_clip, GES_TYPE_SOURCE_CLIP);
+
 #define DEFAULT_TEXT ""
 #define DEFAULT_FONT_DESC "Serif 36"
 #define GES_TITLE_CLIP_VALIGN_TYPE (ges_title_clip_valign_get_type())
@@ -60,8 +58,6 @@ enum
   PROP_YPOS,
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GESTitleClip, ges_title_clip, GES_TYPE_SOURCE_CLIP);
-
 static GESTrackElement
     * ges_title_clip_create_track_element (GESClip * clip, GESTrackType type);
 
@@ -148,11 +144,11 @@ static void
 ges_title_clip_class_init (GESTitleClipClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
-  GESTimelineElementClass *timeline_element_class =
-      GES_TIMELINE_ELEMENT_CLASS (klass);
-  GESClipClass *clip_class = GES_CLIP_CLASS (klass);
+  GESClipClass *timobj_class = GES_CLIP_CLASS (klass);
   GESContainerClass *container_class = GES_CONTAINER_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GESTitleClipPrivate));
+
   object_class->get_property = ges_title_clip_get_property;
   object_class->set_property = ges_title_clip_set_property;
   object_class->dispose = ges_title_clip_dispose;
@@ -215,8 +211,7 @@ ges_title_clip_class_init (GESTitleClipClass * klass)
           G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS |
           GES_PARAM_NO_SERIALIZATION));
 
-  clip_class->create_track_element = ges_title_clip_create_track_element;
-  timeline_element_class->set_inpoint = NULL;
+  timobj_class->create_track_element = ges_title_clip_create_track_element;
 
   container_class->child_added = _child_added;
   container_class->child_removed = _child_removed;
@@ -281,7 +276,8 @@ ges_title_clip_class_init (GESTitleClipClass * klass)
 static void
 ges_title_clip_init (GESTitleClip * self)
 {
-  self->priv = ges_title_clip_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_TITLE_CLIP, GESTitleClipPrivate);
 
   GES_TIMELINE_ELEMENT (self)->duration = 0;
   /* Not 100% required since a new gobject's content will always be memzero'd */
@@ -656,9 +652,6 @@ _child_removed (GESContainer * container, GESTimelineElement * element)
     priv->track_titles = g_slist_remove (priv->track_titles, element);
     gst_object_unref (element);
   }
-
-  GES_CONTAINER_CLASS (ges_title_clip_parent_class)->child_removed (container,
-      element);
 }
 
 static void
@@ -671,9 +664,6 @@ _child_added (GESContainer * container, GESTimelineElement * element)
     priv->track_titles = g_slist_prepend (priv->track_titles,
         gst_object_ref (element));
   }
-
-  GES_CONTAINER_CLASS (ges_title_clip_parent_class)->child_added (container,
-      element);
 }
 
 static GESTrackElement *
@@ -695,8 +685,8 @@ ges_title_clip_create_track_element (GESClip * clip, GESTrackType type)
  *
  * Creates a new #GESTitleClip
  *
- * Returns: (transfer floating) (nullable): The newly created #GESTitleClip,
- * or %NULL if there was an error.
+ * Returns: The newly created #GESTitleClip, or NULL if there was an
+ * error.
  */
 GESTitleClip *
 ges_title_clip_new (void)
