@@ -20,16 +20,15 @@
 
 /**
  * SECTION:gestitlesource
- * @title: GESTitleSource
  * @short_description: render stand-alone text titles
- *
+ * 
  * #GESTitleSource is a GESTimelineElement that implements the notion
  * of titles in GES.
  *
- * ## Children Properties
- *
- * You can use the following children properties through the
- * #ges_track_element_set_child_property and alike set of methods:
+ * <refsect1 id="GESTileSource.children_properties" role="properties">
+ * <title role="children_properties.title">Children Properties</title>
+ * <para>You can use the following children properties through the
+ * #ges_track_element_set_child_property and alike set of methods:</para>
  * <informaltable frame="none">
  * <tgroup cols="3">
  * <colspec colname="properties_type" colwidth="150px"/>
@@ -86,10 +85,8 @@
  * </tbody>
  * </tgroup>
  * </informaltable>
+ * </refsect1>
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-internal.h"
 #include "ges-track-element.h"
@@ -99,6 +96,8 @@
 #define DEFAULT_TEXT ""
 #define DEFAULT_FONT_DESC "Serif 36"
 
+G_DEFINE_TYPE (GESTitleSource, ges_title_source, GES_TYPE_VIDEO_SOURCE);
+
 struct _GESTitleSourcePrivate
 {
   gchar *text;
@@ -113,9 +112,6 @@ struct _GESTitleSourcePrivate
   GstElement *background_el;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GESTitleSource, ges_title_source,
-    GES_TYPE_VIDEO_SOURCE);
-
 enum
 {
   PROP_0,
@@ -132,8 +128,8 @@ static void ges_title_source_set_property (GObject * object, guint
 static GstElement *ges_title_source_create_source (GESTrackElement * self);
 
 static gboolean
-_lookup_child (GESTimelineElement * object,
-    const gchar * prop_name, GObject ** element, GParamSpec ** pspec)
+_lookup_child (GESTrackElement * object,
+    const gchar * prop_name, GstElement ** element, GParamSpec ** pspec)
 {
   gboolean res;
 
@@ -147,7 +143,7 @@ _lookup_child (GESTimelineElement * object,
     clean_name = g_strdup (prop_name);
 
   res =
-      GES_TIMELINE_ELEMENT_CLASS (ges_title_source_parent_class)->lookup_child
+      GES_TRACK_ELEMENT_CLASS (ges_title_source_parent_class)->lookup_child
       (object, clean_name, element, pspec);
 
   g_free (clean_name);
@@ -160,23 +156,23 @@ ges_title_source_class_init (GESTitleSourceClass * klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GESVideoSourceClass *source_class = GES_VIDEO_SOURCE_CLASS (klass);
-  GESTimelineElementClass *timeline_element_class =
-      GES_TIMELINE_ELEMENT_CLASS (klass);
+  GESTrackElementClass *track_element_class = GES_TRACK_ELEMENT_CLASS (klass);
+
+  g_type_class_add_private (klass, sizeof (GESTitleSourcePrivate));
 
   object_class->get_property = ges_title_source_get_property;
   object_class->set_property = ges_title_source_set_property;
   object_class->dispose = ges_title_source_dispose;
+  track_element_class->lookup_child = _lookup_child;
 
-  timeline_element_class->set_inpoint = NULL;
-  timeline_element_class->lookup_child = _lookup_child;
-  source_class->ABI.abi.disable_scale_in_compositor = TRUE;
   source_class->create_source = ges_title_source_create_source;
 }
 
 static void
 ges_title_source_init (GESTitleSource * self)
 {
-  self->priv = ges_title_source_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_TITLE_SOURCE, GESTitleSourcePrivate);
 
   self->priv->text = g_strdup (DEFAULT_TEXT);
   self->priv->font_desc = g_strdup (DEFAULT_FONT_DESC);
@@ -300,7 +296,7 @@ ges_title_source_create_source (GESTrackElement * object)
  * @self: the #GESTitleSource* to set text on
  * @text: the text to render. an internal copy of this text will be
  * made.
- *
+ * 
  * Sets the text this track element will render.
  *
  * Deprecated: use ges_track_element_get/set_children_properties on the
@@ -324,7 +320,7 @@ ges_title_source_set_text (GESTitleSource * self, const gchar * text)
  * ges_title_source_set_font_desc:
  * @self: the #GESTitleSource
  * @font_desc: the pango font description
- *
+ * 
  * Set the pango font description this source will use to render
  * the text.
  */
@@ -450,10 +446,7 @@ ges_title_source_set_ypos (GESTitleSource * self, gdouble position)
  *
  * Get the text currently set on the @source.
  *
- * Returns: (transfer full): The text currently set on the @source.
- * 
- * Deprecated: 1.16: Use ges_timeline_element_get_child_property instead
- * (this actually returns a newly allocated string)
+ * Returns: (transfer none): The text currently set on the @source.
  */
 const gchar *
 ges_title_source_get_text (GESTitleSource * source)
@@ -472,11 +465,8 @@ ges_title_source_get_text (GESTitleSource * source)
  *
  * Get the pango font description used by @source.
  *
- * Returns: (transfer full): The pango font description used by this
+ * Returns: (transfer none): The pango font description used by this
  * @source.
- * 
- * Deprecated: 1.16: Use ges_timeline_element_get_child_property instead
- * (this actually returns a newly allocated string)
  */
 const gchar *
 ges_title_source_get_font_desc (GESTitleSource * source)
@@ -608,8 +598,8 @@ ges_title_source_get_ypos (GESTitleSource * source)
  *
  * Creates a new #GESTitleSource.
  *
- * Returns: (transfer floating) (nullable): The newly created #GESTitleSource,
- * or %NULL if there was an error.
+ * Returns: The newly created #GESTitleSource, or %NULL if there was an
+ * error.
  */
 GESTitleSource *
 ges_title_source_new (void)
