@@ -20,7 +20,6 @@
 
 /**
  * SECTION:gestrackelement
- * @title: GESTrackElement
  * @short_description: Base Class for objects contained in a GESTrack
  *
  * #GESTrackElement is the Base Class for any object that can be contained in a
@@ -30,16 +29,15 @@
  * its container, like the start position, the inpoint, the duration and the
  * priority.
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include "ges-internal.h"
 #include "ges-extractable.h"
 #include "ges-track-element.h"
 #include "ges-clip.h"
 #include "ges-meta-container.h"
 
+G_DEFINE_ABSTRACT_TYPE (GESTrackElement, ges_track_element,
+    GES_TYPE_TIMELINE_ELEMENT);
+
 struct _GESTrackElementPrivate
 {
   GESTrackType track_type;
@@ -76,9 +74,6 @@ enum
 
 static guint ges_track_element_signals[LAST_SIGNAL] = { 0 };
 
-G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GESTrackElement, ges_track_element,
-    GES_TYPE_TIMELINE_ELEMENT);
-
 static GstElement *ges_track_element_create_gnl_object_func (GESTrackElement *
     object);
 
@@ -122,15 +117,6 @@ strv_find_str (const gchar ** strv, const char *str)
   return FALSE;
 }
 
-static guint32
-_get_layer_priority (GESTimelineElement * element)
-{
-  if (!element->parent)
-    return GES_TIMELINE_ELEMENT_NO_LAYER_PRIORITY;
-
-  return ges_timeline_element_get_layer_priority (element->parent);
-}
-
 static void
 ges_track_element_get_property (GObject * object, guint property_id,
     GValue * value, GParamSpec * pspec)
@@ -258,6 +244,8 @@ ges_track_element_class_init (GESTrackElementClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GESTimelineElementClass *element_class = GES_TIMELINE_ELEMENT_CLASS (klass);
 
+  g_type_class_add_private (klass, sizeof (GESTrackElementPrivate));
+
   object_class->get_property = ges_track_element_get_property;
   object_class->set_property = ges_track_element_set_property;
   object_class->dispose = ges_track_element_dispose;
@@ -292,7 +280,7 @@ ges_track_element_class_init (GESTrackElementClass * klass)
    * @track_element: a #GESTrackElement
    * @control_binding: the #GstControlBinding that has been added
    *
-   * The control-binding-added signal is emitted each time a control binding
+   * The control-bunding-added  is emitted each time a control binding
    * is added for a child property of @track_element
    */
   ges_track_element_signals[CONTROL_BINDING_ADDED] =
@@ -303,13 +291,13 @@ ges_track_element_class_init (GESTrackElementClass * klass)
   /**
    * GESTrackElement::control-binding-removed:
    * @track_element: a #GESTrackElement
-   * @control_binding: the #GstControlBinding that has been removed
+   * @control_binding: the #GstControlBinding that has been added
    *
-   * The control-binding-removed signal is emitted each time a control binding
-   * is removed for a child property of @track_element
+   * The control-bunding-added  is emitted each time a control binding
+   * is added for a child property of @track_element
    */
   ges_track_element_signals[CONTROL_BINDING_REMOVED] =
-      g_signal_new ("control-binding-removed", G_TYPE_FROM_CLASS (klass),
+      g_signal_new ("control-binding-reomved", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST, 0, NULL, NULL, g_cclosure_marshal_generic,
       G_TYPE_NONE, 1, GST_TYPE_CONTROL_BINDING);
 
@@ -319,7 +307,6 @@ ges_track_element_class_init (GESTrackElementClass * klass)
   element_class->set_priority = _set_priority;
   element_class->get_track_types = _get_track_types;
   element_class->deep_copy = ges_track_element_copy_properties;
-  element_class->get_layer_priority = _get_layer_priority;
 
   klass->create_gnl_object = ges_track_element_create_gnl_object_func;
   klass->list_children_properties = default_list_children_properties;
@@ -329,8 +316,8 @@ ges_track_element_class_init (GESTrackElementClass * klass)
 static void
 ges_track_element_init (GESTrackElement * self)
 {
-  GESTrackElementPrivate *priv = self->priv =
-      ges_track_element_get_instance_private (self);
+  GESTrackElementPrivate *priv = self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_TRACK_ELEMENT, GESTrackElementPrivate);
 
   /* Sane default values */
   GES_TIMELINE_ELEMENT_START (self) = 0;
@@ -424,7 +411,6 @@ _update_control_bindings (GESTimelineElement * element, GstClockTime inpoint,
       if (next->timestamp > inpoint)
         break;
     }
-    g_list_free (values);
 
     value_at_pos =
         interpolate_values_for_position (first, next, inpoint, absolute);
@@ -444,7 +430,6 @@ _update_control_bindings (GESTimelineElement * element, GstClockTime inpoint,
         if (prev->timestamp < duration + inpoint)
           break;
       }
-      g_list_free (values);
 
       value_at_pos =
           interpolate_values_for_position (prev, last, duration + inpoint,
@@ -466,7 +451,6 @@ _update_control_bindings (GESTimelineElement * element, GstClockTime inpoint,
           && value->timestamp > duration + inpoint)
         gst_timed_value_control_source_unset (source, value->timestamp);
     }
-    g_list_free (values);
   }
 
   g_free (specs);
@@ -687,58 +671,6 @@ add_failure:
   }
 }
 
-static void
-ges_track_element_add_child_props (GESTrackElement * self,
-    GstElement * child, const gchar ** wanted_categories,
-    const gchar ** blacklist, const gchar ** whitelist)
-{
-  GstElementFactory *factory;
-  const gchar *klass;
-  GParamSpec **parray;
-  GObjectClass *gobject_klass;
-  gchar **categories;
-  guint i;
-
-  factory = gst_element_get_factory (child);
-  klass = gst_element_factory_get_metadata (factory,
-      GST_ELEMENT_METADATA_KLASS);
-
-  if (strv_find_str (blacklist, GST_OBJECT_NAME (factory))) {
-    GST_DEBUG_OBJECT (self, "%s blacklisted", GST_OBJECT_NAME (factory));
-    return;
-  }
-
-  GST_DEBUG_OBJECT (self, "Looking at element '%s' of klass '%s'",
-      GST_ELEMENT_NAME (child), klass);
-
-  categories = g_strsplit (klass, "/", 0);
-
-  for (i = 0; categories[i]; i++) {
-    if ((!wanted_categories ||
-            strv_find_str (wanted_categories, categories[i]))) {
-      guint i, nb_specs;
-
-      gobject_klass = G_OBJECT_GET_CLASS (child);
-      parray = g_object_class_list_properties (gobject_klass, &nb_specs);
-      for (i = 0; i < nb_specs; i++) {
-        if ((!whitelist && (parray[i]->flags & G_PARAM_WRITABLE))
-            || (strv_find_str (whitelist, parray[i]->name))) {
-          ges_timeline_element_add_child_property (GES_TIMELINE_ELEMENT
-              (self), parray[i], G_OBJECT (child));
-        }
-      }
-      g_free (parray);
-
-      GST_DEBUG
-          ("%d configurable properties of '%s' added to property hashtable",
-          nb_specs, GST_ELEMENT_NAME (child));
-      break;
-    }
-  }
-
-  g_strfreev (categories);
-}
-
 /**
  * ges_track_element_add_children_props:
  * @self: The #GESTrackElement to set chidlren props on
@@ -763,11 +695,39 @@ ges_track_element_add_children_props (GESTrackElement * self,
 {
   GValue item = { 0, };
   GstIterator *it;
+  GParamSpec **parray;
+  GObjectClass *class;
+  const gchar *klass;
+  GstElementFactory *factory;
   gboolean done = FALSE;
 
   if (!GST_IS_BIN (element)) {
-    ges_track_element_add_child_props (self, element, wanted_categories,
-        blacklist, whitelist);
+    guint i;
+    GParamSpec *pspec;
+
+    GObjectClass *class = G_OBJECT_GET_CLASS (element);
+
+    for (i = 0; whitelist[i]; i++) {
+
+      pspec = g_object_class_find_property (class, whitelist[i]);
+      if (!pspec) {
+        GST_WARNING ("no such property : %s in element : %s", whitelist[i],
+            gst_element_get_name (element));
+        continue;
+      }
+
+      if (pspec->flags) {
+        ges_timeline_element_add_child_property (GES_TIMELINE_ELEMENT (self),
+            pspec, G_OBJECT (element));
+        GST_LOG_OBJECT (self,
+            "added property %s to controllable properties successfully !",
+            whitelist[i]);
+      } else
+        GST_WARNING
+            ("the property %s for element %s exists but is not writable",
+            whitelist[i], gst_element_get_name (element));
+
+    }
     return;
   }
 
@@ -778,9 +738,48 @@ ges_track_element_add_children_props (GESTrackElement * self,
     switch (gst_iterator_next (it, &item)) {
       case GST_ITERATOR_OK:
       {
+        gchar **categories;
+        guint i;
         GstElement *child = g_value_get_object (&item);
-        ges_track_element_add_child_props (self, child, wanted_categories,
-            blacklist, whitelist);
+
+        factory = gst_element_get_factory (child);
+        klass = gst_element_factory_get_metadata (factory,
+            GST_ELEMENT_METADATA_KLASS);
+
+        if (strv_find_str (blacklist, GST_OBJECT_NAME (factory))) {
+          GST_DEBUG_OBJECT (self, "%s blacklisted", GST_OBJECT_NAME (factory));
+          continue;
+        }
+
+        GST_DEBUG ("Looking at element '%s' of klass '%s'",
+            GST_ELEMENT_NAME (child), klass);
+
+        categories = g_strsplit (klass, "/", 0);
+
+        for (i = 0; categories[i]; i++) {
+          if ((!wanted_categories ||
+                  strv_find_str (wanted_categories, categories[i]))) {
+            guint i, nb_specs;
+
+            class = G_OBJECT_GET_CLASS (child);
+            parray = g_object_class_list_properties (class, &nb_specs);
+            for (i = 0; i < nb_specs; i++) {
+              if ((parray[i]->flags & G_PARAM_WRITABLE) &&
+                  (!whitelist || strv_find_str (whitelist, parray[i]->name))) {
+                ges_timeline_element_add_child_property (GES_TIMELINE_ELEMENT
+                    (self), parray[i], G_OBJECT (child));
+              }
+            }
+            g_free (parray);
+
+            GST_DEBUG
+                ("%d configurable properties of '%s' added to property hashtable",
+                nb_specs, GST_ELEMENT_NAME (child));
+            break;
+          }
+        }
+
+        g_strfreev (categories);
         g_value_reset (&item);
         break;
       }
@@ -816,8 +815,6 @@ ges_track_element_set_track (GESTrackElement * object, GESTrack * track)
   object->priv->track = track;
 
   if (object->priv->track) {
-    ges_track_element_set_track_type (object, track->type);
-
     g_object_set (object->priv->nleobject,
         "caps", ges_track_get_caps (object->priv->track), NULL);
   }
@@ -841,14 +838,23 @@ ges_track_element_get_all_control_bindings (GESTrackElement * trackelement)
   return priv->bindings_hashtable;
 }
 
+guint32
+_ges_track_element_get_layer_priority (GESTrackElement * element)
+{
+  if (_PRIORITY (element) < LAYER_HEIGHT + MIN_NLE_PRIO)
+    return 0;
+
+  return (_PRIORITY (element) - MIN_NLE_PRIO) / LAYER_HEIGHT;
+}
+
 /**
  * ges_track_element_get_track:
  * @object: a #GESTrackElement
  *
  * Get the #GESTrack to which this object belongs.
  *
- * Returns: (transfer none) (nullable): The #GESTrack to which this object
- * belongs. Can be %NULL if it is not in any track
+ * Returns: (transfer none): The #GESTrack to which this object belongs. Can be %NULL if it
+ * is not in any track
  */
 GESTrack *
 ges_track_element_get_track (GESTrackElement * object)
@@ -961,7 +967,7 @@ ges_track_element_lookup_child (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_property_by_pspec: (skip):
+ * ges_track_element_set_child_property_by_pspec:
  * @object: a #GESTrackElement
  * @pspec: The #GParamSpec that specifies the property you want to set
  * @value: the value
@@ -983,7 +989,7 @@ ges_track_element_set_child_property_by_pspec (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_property_valist: (skip):
+ * ges_track_element_set_child_property_valist:
  * @object: The #GESTrackElement parent object
  * @first_property_name: The name of the first property to set
  * @var_args: value for the first property, followed optionally by more
@@ -1005,7 +1011,7 @@ ges_track_element_set_child_property_valist (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_properties: (skip):
+ * ges_track_element_set_child_properties:
  * @object: The #GESTrackElement parent object
  * @first_property_name: The name of the first property to set
  * @...: value for the first property, followed optionally by more
@@ -1033,7 +1039,7 @@ ges_track_element_set_child_properties (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_property_valist: (skip):
+ * ges_track_element_get_child_property_valist:
  * @object: The #GESTrackElement parent object
  * @first_property_name: The name of the first property to get
  * @var_args: value for the first property, followed optionally by more
@@ -1077,7 +1083,7 @@ ges_track_element_list_children_properties (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_properties: (skip):
+ * ges_track_element_get_child_properties:
  * @object: The origin #GESTrackElement
  * @first_property_name: The name of the first property to get
  * @...: return location for the first property, followed optionally by more
@@ -1102,7 +1108,7 @@ ges_track_element_get_child_properties (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_property_by_pspec: (skip):
+ * ges_track_element_get_child_property_by_pspec:
  * @object: a #GESTrackElement
  * @pspec: The #GParamSpec that specifies the property you want to get
  * @value: (out): return location for the value
@@ -1120,7 +1126,7 @@ ges_track_element_get_child_property_by_pspec (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_property: (skip):
+ * ges_track_element_set_child_property:
  * @object: The origin #GESTrackElement
  * @property_name: The name of the property
  * @value: the value
@@ -1144,7 +1150,7 @@ ges_track_element_set_child_property (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_property: (skip):
+ * ges_track_element_get_child_property:
  * @object: The origin #GESTrackElement
  * @property_name: The name of the property
  * @value: (out): return location for the property value, it will
@@ -1194,8 +1200,6 @@ ges_track_element_copy_properties (GESTimelineElement * element,
       ges_track_element_list_children_properties (GES_TRACK_ELEMENT (element),
       &n_specs);
   for (n = 0; n < n_specs; ++n) {
-    if (!(specs[n]->flags & G_PARAM_WRITABLE))
-      continue;
     g_value_init (&val, specs[n]->value_type);
     ges_track_element_get_child_property_by_pspec (GES_TRACK_ELEMENT (element),
         specs[n], &val);
@@ -1249,7 +1253,6 @@ _split_binding (GESTrackElement * element, GESTrackElement * new_element,
     last_value = value;
 
   }
-  g_list_free (values);
 }
 
 static void
@@ -1268,7 +1271,6 @@ _copy_binding (GESTrackElement * element, GESTrackElement * new_element,
     gst_timed_value_control_source_set (new_source, value->timestamp,
         value->value);
   }
-  g_list_free (values);
 }
 
 /* position == GST_CLOCK_TIME_NONE means that we do a simple copy
@@ -1334,7 +1336,7 @@ ges_track_element_copy_bindings (GESTrackElement * element,
  *  happen in, %NULL means that the edition is done in all the
  *  #GESLayers contained in the current timeline.
  *      FIXME: This is not implemented yet.
- * @mode: The #GESEditMode in which the edition will happen.
+ * @mode: The #GESEditMode in which the editition will happen.
  * @edge: The #GESEdge the edit should happen on.
  * @position: The position at which to edit @object (in nanosecond)
  *
@@ -1354,7 +1356,7 @@ ges_track_element_edit (GESTrackElement * object,
   g_return_val_if_fail (GES_IS_TRACK_ELEMENT (object), FALSE);
 
   if (G_UNLIKELY (!track)) {
-    GST_WARNING_OBJECT (object, "Trying to edit in %d mode but not in "
+    GST_WARNING_OBJECT (object, "Trying to edit in %d mode but not in"
         "any Track yet.", mode);
     return FALSE;
   }
@@ -1362,28 +1364,23 @@ ges_track_element_edit (GESTrackElement * object,
   timeline = GES_TIMELINE (ges_track_get_timeline (track));
 
   if (G_UNLIKELY (!timeline)) {
-    GST_WARNING_OBJECT (object, "Trying to edit in %d mode but "
-        "track %p is not in any timeline yet.", mode, track);
+    GST_WARNING_OBJECT (object, "Trying to edit in %d mode but not in"
+        "track %p no in any timeline yet.", mode, track);
     return FALSE;
   }
 
   switch (mode) {
     case GES_EDIT_MODE_NORMAL:
-      return timeline_move_object (timeline, GES_TIMELINE_ELEMENT (object), -1,
-          layers, edge, position);
+      return timeline_move_object (timeline, object, layers, edge, position);
       break;
     case GES_EDIT_MODE_TRIM:
-      return timeline_trim_object (timeline, GES_TIMELINE_ELEMENT (object), -1,
-          layers, edge, position);
+      return timeline_trim_object (timeline, object, layers, edge, position);
       break;
     case GES_EDIT_MODE_RIPPLE:
-      return timeline_ripple_object (timeline, GES_TIMELINE_ELEMENT (object),
-          GES_TIMELINE_ELEMENT_PRIORITY (object) / LAYER_HEIGHT,
-          layers, edge, position);
+      return timeline_ripple_object (timeline, object, layers, edge, position);
       break;
     case GES_EDIT_MODE_ROLL:
-      return timeline_roll_object (timeline, GES_TIMELINE_ELEMENT (object),
-          layers, edge, position);
+      return timeline_roll_object (timeline, object, layers, edge, position);
       break;
     case GES_EDIT_MODE_SLIDE:
       return timeline_slide_object (timeline, object, layers, edge, position);
@@ -1447,9 +1444,7 @@ ges_track_element_remove_control_binding (GESTrackElement * object,
  * @object: the #GESTrackElement on which to set a control binding
  * @source: the #GstControlSource to set on the binding.
  * @property_name: The name of the property to control.
- * @binding_type: The type of binding to create. Currently the following values are valid:
- *   - "direct": See #gst_direct_control_binding_new
- *   - "direct-absolute": See #gst_direct_control_binding_new_absolute
+ * @binding_type: The type of binding to create. Only "direct" is available for now.
  *
  * Creates a #GstControlBinding and adds it to the #GstElement concerned by the
  * property. Use the same syntax as #ges_track_element_lookup_child for
@@ -1489,8 +1484,13 @@ ges_track_element_set_control_source (GESTrackElement * object,
 
   if (direct || direct_absolute) {
     /* First remove existing binding */
-    if (ges_track_element_remove_control_binding (object, property_name)) {
-      GST_LOG ("Removed old binding for property %s", property_name);
+    binding =
+        (GstControlBinding *) g_hash_table_lookup (priv->bindings_hashtable,
+        property_name);
+    if (binding) {
+      GST_LOG ("Removing old binding %p for property %s", binding,
+          property_name);
+      gst_object_remove_control_binding (GST_OBJECT (element), binding);
     }
 
     if (direct_absolute)
@@ -1523,8 +1523,8 @@ ges_track_element_set_control_source (GESTrackElement * object,
  * Looks up the various controlled properties for that #GESTrackElement,
  * and returns the #GstControlBinding which controls @property_name.
  *
- * Returns: (transfer none) (nullable): the #GstControlBinding associated with
- * @property_name, or %NULL if that property is not controlled.
+ * Returns: (transfer none): the #GstControlBinding associated with @property_name, or %NULL
+ * if that property is not controlled.
  */
 GstControlBinding *
 ges_track_element_get_control_binding (GESTrackElement * object,
