@@ -20,16 +20,12 @@
 
 /**
  * SECTION:gestrack
- * @title: GESTrack
  * @short_description: Composition of objects
  *
  * Corresponds to one output format (i.e. audio OR video).
  *
  * Contains the compatible TrackElement(s).
  */
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
 
 #include "ges-internal.h"
 #include "ges-track.h"
@@ -38,7 +34,8 @@
 #include "ges-video-track.h"
 #include "ges-audio-track.h"
 
-#define CHECK_THREAD(track) g_assert(track->priv->valid_thread == g_thread_self())
+G_DEFINE_TYPE_WITH_CODE (GESTrack, ges_track, GST_TYPE_BIN,
+    G_IMPLEMENT_INTERFACE (GES_TYPE_META_CONTAINER, NULL));
 
 static GstStaticPadTemplate ges_track_src_pad_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -82,8 +79,6 @@ struct _GESTrackPrivate
 
   /* Virtual method to create GstElement that fill gaps */
   GESCreateElementForGapFunc create_element_for_gaps;
-
-  GThread *valid_thread;
 };
 
 enum
@@ -105,11 +100,6 @@ static guint ges_track_signals[LAST_SIGNAL] = { 0 };
 
 static GParamSpec *properties[ARG_LAST];
 
-G_DEFINE_TYPE_WITH_CODE (GESTrack, ges_track, GST_TYPE_BIN,
-    G_ADD_PRIVATE (GESTrack)
-    G_IMPLEMENT_INTERFACE (GES_TYPE_META_CONTAINER, NULL));
-
-
 static void composition_duration_cb (GstElement * composition, GParamSpec * arg
     G_GNUC_UNUSED, GESTrack * obj);
 
@@ -280,6 +270,14 @@ update_field (GQuark field_id, const GValue * value, GstStructure * original)
 }
 
 /* callbacks */
+static void
+sort_track_elements_cb (GESTrackElement * child,
+    GParamSpec * arg G_GNUC_UNUSED, GESTrack * track)
+{
+  g_sequence_sort (track->priv->trackelements_by_start,
+      (GCompareDataFunc) element_start_compare, NULL);
+}
+
 static void
 _ghost_nlecomposition_srcpad (GESTrack * track)
 {
@@ -372,6 +370,8 @@ remove_object_internal (GESTrack * track, GESTrackElement * object)
     }
   }
 
+  g_signal_handlers_disconnect_by_func (object, sort_track_elements_cb, NULL);
+
   ges_track_element_set_track (object, NULL);
   ges_timeline_element_set_timeline (GES_TIMELINE_ELEMENT (object), NULL);
 
@@ -561,6 +561,8 @@ ges_track_class_init (GESTrackClass * klass)
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_class = (GstElementClass *) klass;
 
+  g_type_class_add_private (klass, sizeof (GESTrackPrivate));
+
   gstelement_class->change_state = GST_DEBUG_FUNCPTR (ges_track_change_state);
 
   object_class->get_property = ges_track_get_property;
@@ -638,8 +640,8 @@ ges_track_class_init (GESTrackClass * klass)
   g_object_class_install_property (object_class, ARG_MIXING,
       properties[ARG_MIXING]);
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &ges_track_src_pad_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&ges_track_src_pad_template));
 
   /**
    * GESTrack::track-element-added:
@@ -679,8 +681,8 @@ ges_track_class_init (GESTrackClass * klass)
 static void
 ges_track_init (GESTrack * self)
 {
-  self->priv = ges_track_get_instance_private (self);
-  self->priv->valid_thread = g_thread_self ();
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_TRACK, GESTrackPrivate);
 
   self->priv->composition = gst_element_factory_make ("nlecomposition", NULL);
   self->priv->capsfilter = gst_element_factory_make ("capsfilter", NULL);
@@ -709,7 +711,7 @@ ges_track_init (GESTrack * self)
  * The newly created track will steal a reference to the caps. If you wish to
  * use those caps elsewhere, you will have to take an extra reference.
  *
- * Returns: (transfer floating): A new #GESTrack.
+ * Returns: A new #GESTrack.
  */
 GESTrack *
 ges_track_new (GESTrackType type, GstCaps * caps)
@@ -785,7 +787,6 @@ ges_track_set_caps (GESTrack * track, const GstCaps * caps)
   gint i;
 
   g_return_if_fail (GES_IS_TRACK (track));
-  CHECK_THREAD (track);
 
   GST_DEBUG ("track:%p, caps:%" GST_PTR_FORMAT, track, caps);
   g_return_if_fail (GST_IS_CAPS (caps));
@@ -816,7 +817,6 @@ ges_track_set_restriction_caps (GESTrack * track, const GstCaps * caps)
   GESTrackPrivate *priv;
 
   g_return_if_fail (GES_IS_TRACK (track));
-  CHECK_THREAD (track);
 
   GST_DEBUG ("track:%p, restriction caps:%" GST_PTR_FORMAT, track, caps);
   g_return_if_fail (GST_IS_CAPS (caps));
@@ -849,17 +849,8 @@ void
 ges_track_update_restriction_caps (GESTrack * self, const GstCaps * caps)
 {
   guint i;
-  GstCaps *new_restriction_caps;
+  GstCaps *new_restriction_caps = gst_caps_copy (self->priv->restriction_caps);
 
-  g_return_if_fail (GES_IS_TRACK (self));
-  CHECK_THREAD (self);
-
-  if (!self->priv->restriction_caps) {
-    ges_track_set_restriction_caps (self, caps);
-    return;
-  }
-
-  new_restriction_caps = gst_caps_copy (self->priv->restriction_caps);
   for (i = 0; i < gst_caps_get_size (caps); i++) {
     GstStructure *new = gst_caps_get_structure (caps, i);
 
@@ -873,7 +864,6 @@ ges_track_update_restriction_caps (GESTrack * self, const GstCaps * caps)
   }
 
   ges_track_set_restriction_caps (self, new_restriction_caps);
-  gst_caps_unref (new_restriction_caps);
 }
 
 /**
@@ -887,7 +877,6 @@ void
 ges_track_set_mixing (GESTrack * track, gboolean mixing)
 {
   g_return_if_fail (GES_IS_TRACK (track));
-  CHECK_THREAD (track);
 
   if (!track->priv->mixing_operation) {
     GST_DEBUG_OBJECT (track, "Track will be set to mixing = %d", mixing);
@@ -922,7 +911,7 @@ ges_track_set_mixing (GESTrack * track, gboolean mixing)
 /**
  * ges_track_add_element:
  * @track: a #GESTrack
- * @object: (transfer floating): the #GESTrackElement to add
+ * @object: (transfer full): the #GESTrackElement to add
  *
  * Adds the given object to the track. Sets the object's controlling track,
  * and thus takes ownership of the @object.
@@ -937,21 +926,16 @@ ges_track_add_element (GESTrack * track, GESTrackElement * object)
 {
   g_return_val_if_fail (GES_IS_TRACK (track), FALSE);
   g_return_val_if_fail (GES_IS_TRACK_ELEMENT (object), FALSE);
-  CHECK_THREAD (track);
 
   GST_DEBUG ("track:%p, object:%p", track, object);
 
   if (G_UNLIKELY (ges_track_element_get_track (object) != NULL)) {
     GST_WARNING ("Object already belongs to another track");
-    gst_object_ref_sink (object);
-    gst_object_unref (object);
     return FALSE;
   }
 
   if (G_UNLIKELY (!ges_track_element_set_track (object, track))) {
     GST_ERROR ("Couldn't properly add the object to the Track");
-    gst_object_ref_sink (object);
-    gst_object_unref (object);
     return FALSE;
   }
 
@@ -962,8 +946,6 @@ ges_track_add_element (GESTrack * track, GESTrackElement * object)
   if (G_UNLIKELY (!ges_nle_composition_add_object (track->priv->composition,
               ges_track_element_get_nleobject (object)))) {
     GST_WARNING ("Couldn't add object to the NleComposition");
-    gst_object_ref_sink (object);
-    gst_object_unref (object);
     return FALSE;
   }
 
@@ -977,6 +959,15 @@ ges_track_add_element (GESTrack * track, GESTrackElement * object)
   g_signal_emit (track, ges_track_signals[TRACK_ELEMENT_ADDED], 0,
       GES_TRACK_ELEMENT (object));
 
+  g_signal_connect (GES_TRACK_ELEMENT (object), "notify::start",
+      G_CALLBACK (sort_track_elements_cb), track);
+
+  g_signal_connect (GES_TRACK_ELEMENT (object), "notify::duration",
+      G_CALLBACK (sort_track_elements_cb), track);
+
+  g_signal_connect (GES_TRACK_ELEMENT (object), "notify::priority",
+      G_CALLBACK (sort_track_elements_cb), track);
+
   return TRUE;
 }
 
@@ -995,7 +986,6 @@ ges_track_get_elements (GESTrack * track)
   GList *ret = NULL;
 
   g_return_val_if_fail (GES_IS_TRACK (track), NULL);
-  CHECK_THREAD (track);
 
   g_sequence_foreach (track->priv->trackelements_by_start,
       (GFunc) add_trackelement_to_list_foreach, &ret);
@@ -1025,7 +1015,6 @@ ges_track_remove_element (GESTrack * track, GESTrackElement * object)
 
   g_return_val_if_fail (GES_IS_TRACK (track), FALSE);
   g_return_val_if_fail (GES_IS_TRACK_ELEMENT (object), FALSE);
-  CHECK_THREAD (track);
 
   priv = track->priv;
 
@@ -1033,6 +1022,7 @@ ges_track_remove_element (GESTrack * track, GESTrackElement * object)
 
   it = g_hash_table_lookup (priv->trackelements_iter, object);
   g_sequence_remove (it);
+  track_resort_and_fill_gaps (track);
 
   if (remove_object_internal (track, object) == TRUE) {
     ges_timeline_element_set_timeline (GES_TIMELINE_ELEMENT (object), NULL);
@@ -1059,7 +1049,6 @@ const GstCaps *
 ges_track_get_caps (GESTrack * track)
 {
   g_return_val_if_fail (GES_IS_TRACK (track), NULL);
-  CHECK_THREAD (track);
 
   return track->priv->caps;
 }
@@ -1070,13 +1059,12 @@ ges_track_get_caps (GESTrack * track)
  *
  * Get the #GESTimeline this track belongs to. Can be %NULL.
  *
- * Returns: (nullable): The #GESTimeline this track belongs to. Can be %NULL.
+ * Returns: The #GESTimeline this track belongs to. Can be %NULL.
  */
 const GESTimeline *
 ges_track_get_timeline (GESTrack * track)
 {
   g_return_val_if_fail (GES_IS_TRACK (track), NULL);
-  CHECK_THREAD (track);
 
   return track->priv->timeline;
 }
@@ -1116,7 +1104,6 @@ gboolean
 ges_track_commit (GESTrack * track)
 {
   g_return_val_if_fail (GES_IS_TRACK (track), FALSE);
-  CHECK_THREAD (track);
 
   track_resort_and_fill_gaps (track);
 
@@ -1139,7 +1126,6 @@ ges_track_set_create_element_for_gap_func (GESTrack * track,
     GESCreateElementForGapFunc func)
 {
   g_return_if_fail (GES_IS_TRACK (track));
-  CHECK_THREAD (track);
 
   track->priv->create_element_for_gaps = func;
 }
