@@ -45,6 +45,8 @@ GST_DEBUG_CATEGORY_STATIC (nlecomposition_debug);
 #define _do_init              \
   GST_DEBUG_CATEGORY_INIT (nlecomposition_debug,"nlecomposition", GST_DEBUG_FG_BLUE | GST_DEBUG_BOLD, "GNonLin Composition");
 #define nle_composition_parent_class parent_class
+G_DEFINE_TYPE_WITH_CODE (NleComposition, nle_composition, NLE_TYPE_OBJECT,
+    _do_init);
 
 enum
 {
@@ -66,6 +68,7 @@ enum
 {
   COMMIT_SIGNAL,
   COMMITED_SIGNAL,
+  QUERY_POSITION_SIGNAL,
   LAST_SIGNAL
 };
 
@@ -102,12 +105,6 @@ typedef struct
   NleUpdateStackReason reason;
 } UpdateCompositionData;
 
-typedef struct _Action
-{
-  GCClosure closure;
-  gint priority;
-} Action;
-
 struct _NleCompositionPrivate
 {
   gboolean dispose_has_run;
@@ -137,35 +134,20 @@ struct _NleCompositionPrivate
   /* List of NleObject whose start/duration will be the same as the composition */
   GList *expandables;
 
-  /* currently configured stack seek start/stop time.
-   * In forward playback:
-   *   - current_stack_start: The start of the current stack or the start value
-   *     of the seek if the stack has been seeked 'in the middle'
-   *   - current_stack_stop: The stop time of the current stack
-   *
-   * Reconstruct pipeline ONLY if seeking outside of those values
-   * FIXME : current_stack_start isn't always the earliest time before which the
-   * timeline doesn't need to be modified
+  /*
+     current segment seek start/stop time.
+     Reconstruct pipeline ONLY if seeking outside of those values
+     FIXME : segment_start isn't always the earliest time before which the
+     timeline doesn't need to be modified
    */
-  GstClockTime current_stack_start;
-  GstClockTime current_stack_stop;
+  GstClockTime segment_start;
+  GstClockTime segment_stop;
 
   /* Seek segment handler */
-  /* Represents the current segment that is being played,
-   * In forwards playback (logic is the same but swapping start and
-   * stop in backward playback):
-   *  - segment->start: start of the current segment being played,
-   *    at each stack change it will advance to the newly configured
-   *    stack start.
-   *  - segment->stop is the final stop of the segment being played.
-   *    if a seek with a stop time happened, it will be the stop time
-   *    otherwise, it will be the composition duration.
-   */
   GstSegment *segment;
+  GstSegment *outside_segment;
 
-  /* Segment representing the last seek. Simply initialized
-   * segment if no seek occured. */
-  GstSegment *seek_segment;
+  /* Next running base_time to set on outgoing segment */
   guint64 next_base_time;
 
   /*
@@ -179,7 +161,6 @@ struct _NleCompositionPrivate
   GMutex actions_lock;
   GCond actions_cond;
   GList *actions;
-  Action *current_action;
 
   gboolean running;
   gboolean initialized;
@@ -193,27 +174,28 @@ struct _NleCompositionPrivate
 
   /* 0 means that we already received the right caps or segment */
   gint seqnum_to_restart_task;
-  gboolean waiting_serialized_query_or_buffer;
+  gboolean waiting_for_buffer;
 
   gboolean tearing_down_stack;
-  gboolean suppress_child_error;
 
   NleUpdateStackReason updating_reason;
 };
 
+typedef struct _Action
+{
+  GCClosure closure;
+  gint priority;
+} Action;
+
 #define ACTION_CALLBACK(__action) (((GCClosure*) (__action))->callback)
 
 static guint _signals[LAST_SIGNAL] = { 0 };
 
 static GParamSpec *nleobject_properties[NLEOBJECT_PROP_LAST];
 
-G_DEFINE_TYPE_WITH_CODE (NleComposition, nle_composition, NLE_TYPE_OBJECT,
-    G_ADD_PRIVATE (NleComposition)
-    _do_init);
-
 #define OBJECT_IN_ACTIVE_SEGMENT(comp,element)      \
-  ((NLE_OBJECT_START(element) < comp->priv->current_stack_stop) &&  \
-   (NLE_OBJECT_STOP(element) >= comp->priv->current_stack_start))
+  ((NLE_OBJECT_START(element) < comp->priv->segment_stop) &&  \
+   (NLE_OBJECT_STOP(element) >= comp->priv->segment_start))
 
 static void nle_composition_dispose (GObject * object);
 static void nle_composition_finalize (GObject * object);
@@ -397,19 +379,14 @@ _execute_actions (NleComposition * comp)
 
     lact = g_list_first (priv->actions);
     priv->actions = g_list_remove_link (priv->actions, lact);
-    priv->current_action = lact->data;
     ACTIONS_UNLOCK (comp);
 
     GST_INFO_OBJECT (comp, "Invoking %p:%s",
         lact->data, GST_DEBUG_FUNCPTR_NAME ((ACTION_CALLBACK (lact->data))));
     g_closure_invoke (lact->data, NULL, 1, params, NULL);
     g_value_unset (&params[0]);
-
-    ACTIONS_LOCK (comp);
     g_closure_unref (lact->data);
     g_list_free (lact);
-    priv->current_action = NULL;
-    ACTIONS_UNLOCK (comp);
 
     GST_LOG_OBJECT (comp, "remaining actions [%d]",
         g_list_length (priv->actions));
@@ -549,16 +526,12 @@ _seek_pipeline_func (NleComposition * comp, SeekData * seekd)
 
   gst_segment_do_seek (priv->segment,
       rate, format, flags, cur_type, cur, stop_type, stop, NULL);
-  gst_segment_do_seek (priv->seek_segment,
+  gst_segment_do_seek (priv->outside_segment,
       rate, format, flags, cur_type, cur, stop_type, stop, NULL);
 
   GST_DEBUG_OBJECT (seekd->comp, "Segment now has flags:%d",
       priv->segment->flags);
 
-  /* FIXME: The idea was to avoid seeking on a stack if we know we will endup
-   * passed the end, but then we loose the flush, wich leads to hangs. Long
-   * term, we should just flush the stack instead to avoid the double seek. */
-#if 0
   if (priv->segment->start >= NLE_OBJECT_STOP (seekd->comp)) {
     GST_INFO_OBJECT (seekd->comp,
         "Start %" GST_TIME_FORMAT " > comp->stop: %" GST_TIME_FORMAT
@@ -567,7 +540,6 @@ _seek_pipeline_func (NleComposition * comp, SeekData * seekd)
     GST_FIXME_OBJECT (seekd->comp, "HANDLE error async!");
     return;
   }
-#endif
 
   _post_start_composition_update (seekd->comp,
       gst_event_get_seqnum (seekd->event), COMP_UPDATE_STACK_ON_SEEK);
@@ -612,9 +584,7 @@ _process_pending_entries (NleComposition * comp)
 
       _nle_composition_remove_object (comp, object);
     } else {
-      /* take a new ref on object as the current one will be released when
-       * object is removed from pending_io */
-      _nle_composition_add_object (comp, gst_object_ref (object));
+      _nle_composition_add_object (comp, object);
     }
   }
 
@@ -718,7 +688,7 @@ _remove_object_func (NleComposition * comp, ChildIOData * childio)
     return;
   }
 
-  g_hash_table_add (priv->pending_io, gst_object_ref (object));
+  g_hash_table_add (priv->pending_io, object);
 
   return;
 }
@@ -768,9 +738,7 @@ _add_object_func (NleComposition * comp, ChildIOData * childio)
     return;
   }
 
-  /* current reference is hold by the action and will be released with it,
-   * so take a new one */
-  g_hash_table_add (priv->pending_io, gst_object_ref (object));
+  g_hash_table_add (priv->pending_io, object);
 }
 
 static void
@@ -797,12 +765,8 @@ _free_action (gpointer udata, Action * action)
 
     gst_event_unref (seekd->event);
     g_slice_free (SeekData, seekd);
-  } else if (ACTION_CALLBACK (action) == _add_object_func) {
-    ChildIOData *iodata = (ChildIOData *) udata;
-
-    gst_object_unref (iodata->object);
-    g_slice_free (ChildIOData, iodata);
-  } else if (ACTION_CALLBACK (action) == _remove_object_func) {
+  } else if (ACTION_CALLBACK (action) == _remove_object_func ||
+      ACTION_CALLBACK (action) == _add_object_func) {
     g_slice_free (ChildIOData, udata);
   } else if (ACTION_CALLBACK (action) == _update_pipeline_func ||
       ACTION_CALLBACK (action) == _commit_func ||
@@ -812,7 +776,7 @@ _free_action (gpointer udata, Action * action)
 }
 
 static void
-_add_action_locked (NleComposition * comp, GCallback func,
+_add_action (NleComposition * comp, GCallback func,
     gpointer data, gint priority)
 {
   Action *action;
@@ -826,6 +790,7 @@ _add_action_locked (NleComposition * comp, GCallback func,
   action->priority = priority;
   g_closure_set_marshal ((GClosure *) action, g_cclosure_marshal_VOID__VOID);
 
+  ACTIONS_LOCK (comp);
   GST_INFO_OBJECT (comp, "Adding Action for function: %p:%s",
       action, GST_DEBUG_FUNCPTR_NAME (func));
 
@@ -838,75 +803,23 @@ _add_action_locked (NleComposition * comp, GCallback func,
       g_list_length (priv->actions));
 
   SIGNAL_NEW_ACTION (comp);
-}
-
-static void
-_add_action (NleComposition * comp, GCallback func,
-    gpointer data, gint priority)
-{
-  ACTIONS_LOCK (comp);
-  _add_action_locked (comp, func, data, priority);
   ACTIONS_UNLOCK (comp);
 }
 
 static void
 _add_seek_action (NleComposition * comp, GstEvent * event)
 {
-  SeekData *seekd;
-  GList *tmp;
-  guint32 seqnum = gst_event_get_seqnum (event);
-
-  ACTIONS_LOCK (comp);
-  /* Check if this is our current seqnum */
-  if (seqnum == comp->priv->next_eos_seqnum) {
-    GST_DEBUG_OBJECT (comp, "Not adding Action, same seqnum as previous seek");
-    ACTIONS_UNLOCK (comp);
-    return;
-  }
-
-  /* Check if this seqnum is already queued up but not handled yet */
-  for (tmp = comp->priv->actions; tmp != NULL; tmp = tmp->next) {
-    Action *act = tmp->data;
-
-    if (ACTION_CALLBACK (act) == G_CALLBACK (_seek_pipeline_func)) {
-      SeekData *tmp_data = ((GClosure *) act)->data;
-
-      if (gst_event_get_seqnum (tmp_data->event) == seqnum) {
-        GST_DEBUG_OBJECT (comp,
-            "Not adding Action, same seqnum as previous seek");
-        ACTIONS_UNLOCK (comp);
-        return;
-      }
-    }
-  }
-
-  /* Check if this seqnum is currently being handled */
-  if (comp->priv->current_action) {
-    Action *act = comp->priv->current_action;
-    if (ACTION_CALLBACK (act) == G_CALLBACK (_seek_pipeline_func)) {
-      SeekData *tmp_data = ((GClosure *) act)->data;
-
-      if (gst_event_get_seqnum (tmp_data->event) == seqnum) {
-        GST_DEBUG_OBJECT (comp,
-            "Not adding Action, same seqnum as previous seek");
-        ACTIONS_UNLOCK (comp);
-        return;
-      }
-    }
-  }
+  SeekData *seekd = g_slice_new0 (SeekData);
 
   GST_DEBUG_OBJECT (comp, "Adding Action");
 
-  seekd = g_slice_new0 (SeekData);
   seekd->comp = comp;
   seekd->event = event;
 
   comp->priv->next_eos_seqnum = 0;
   comp->priv->real_eos_seqnum = 0;
-  _add_action_locked (comp, G_CALLBACK (_seek_pipeline_func), seekd,
+  _add_action (comp, G_CALLBACK (_seek_pipeline_func), seekd,
       G_PRIORITY_DEFAULT);
-
-  ACTIONS_UNLOCK (comp);
 }
 
 static void
@@ -941,29 +854,22 @@ static void
 nle_composition_handle_message (GstBin * bin, GstMessage * message)
 {
   NleComposition *comp = (NleComposition *) bin;
-  NleCompositionPrivate *priv = comp->priv;
 
-  if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR &&
-      (priv->tearing_down_stack || priv->suppress_child_error)) {
-    GST_FIXME_OBJECT (comp, "Dropping %" GST_PTR_FORMAT " message from "
-        " %" GST_PTR_FORMAT " tearing down: %d, suppressing error: %d",
-        message, GST_MESSAGE_SRC (message), priv->tearing_down_stack,
-        priv->suppress_child_error);
-    goto drop;
-  } else if (comp->priv->tearing_down_stack) {
+  if (comp->priv->tearing_down_stack) {
+    if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR) {
+      GST_FIXME_OBJECT (comp, "Dropping %" GST_PTR_FORMAT " message from "
+          " %" GST_PTR_FORMAT " being teared down to READY",
+          message, GST_MESSAGE_SRC (message));
+    }
+
     GST_DEBUG_OBJECT (comp, "Dropping message %" GST_PTR_FORMAT " from "
         "object being teared down to READY!", message);
-    goto drop;
+    gst_message_unref (message);
+
+    return;
   }
 
   GST_BIN_CLASS (parent_class)->handle_message (bin, message);
-
-  return;
-
-drop:
-  gst_message_unref (message);
-
-  return;
 }
 
 static void
@@ -979,6 +885,8 @@ nle_composition_class_init (NleCompositionClass * klass)
   gstbin_class = (GstBinClass *) klass;
   nleobject_class = (NleObjectClass *) klass;
 
+  g_type_class_add_private (klass, sizeof (NleCompositionPrivate));
+
   gst_element_class_set_static_metadata (gstelement_class,
       "GNonLin Composition", "Filter/Editor", "Combines NLE objects",
       "Wim Taymans <wim.taymans@gmail.com>, Edward Hervey <bilboed@bilboed.com>,"
@@ -996,8 +904,8 @@ nle_composition_class_init (NleCompositionClass * klass)
   gstbin_class->handle_message =
       GST_DEBUG_FUNCPTR (nle_composition_handle_message);
 
-  gst_element_class_add_static_pad_template (gstelement_class,
-      &nle_composition_src_template);
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&nle_composition_src_template));
 
   /* Get the paramspec of the NleObject klass so we can do
    * fast notifies */
@@ -1013,6 +921,22 @@ nle_composition_class_init (NleCompositionClass * klass)
       0, NULL, NULL, g_cclosure_marshal_generic, G_TYPE_NONE, 1,
       G_TYPE_BOOLEAN);
 
+  /**
+   * NleComposition::query-position
+   * @comp: a #NleComposition
+   *
+   * A signal that *has* to be connected and which should return the current
+   * position of the pipeline.
+   *
+   * This signal is used in order to know the current position of the whole
+   * pipeline so it is user's responsability to give that answer as there
+   * is no other way to precisely know the position in the whole pipeline.
+   */
+  _signals[QUERY_POSITION_SIGNAL] =
+      g_signal_new ("query-position", G_TYPE_FROM_CLASS (klass),
+      G_SIGNAL_RUN_LAST | G_SIGNAL_NO_RECURSE,
+      0, NULL, NULL, g_cclosure_marshal_generic, G_TYPE_UINT64, 0, NULL);
+
   GST_DEBUG_REGISTER_FUNCPTR (_seek_pipeline_func);
   GST_DEBUG_REGISTER_FUNCPTR (_remove_object_func);
   GST_DEBUG_REGISTER_FUNCPTR (_add_object_func);
@@ -1035,12 +959,13 @@ nle_composition_init (NleComposition * comp)
   GST_OBJECT_FLAG_SET (comp, NLE_OBJECT_SOURCE);
   GST_OBJECT_FLAG_SET (comp, NLE_OBJECT_COMPOSITION);
 
-  priv = nle_composition_get_instance_private (comp);
+  priv = G_TYPE_INSTANCE_GET_PRIVATE (comp, NLE_TYPE_COMPOSITION,
+      NleCompositionPrivate);
   priv->objects_start = NULL;
   priv->objects_stop = NULL;
 
   priv->segment = gst_segment_new ();
-  priv->seek_segment = gst_segment_new ();
+  priv->outside_segment = gst_segment_new ();
 
   g_rec_mutex_init (&comp->task_rec_lock);
 
@@ -1049,8 +974,7 @@ nle_composition_init (NleComposition * comp)
   g_mutex_init (&priv->actions_lock);
   g_cond_init (&priv->actions_cond);
 
-  priv->pending_io = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-      gst_object_unref, NULL);
+  priv->pending_io = g_hash_table_new (g_direct_hash, g_direct_equal);
 
   comp->priv = priv;
 
@@ -1131,14 +1055,14 @@ nle_composition_finalize (GObject * object)
   g_hash_table_destroy (priv->objects_hash);
 
   gst_segment_free (priv->segment);
-  gst_segment_free (priv->seek_segment);
+  gst_segment_free (priv->outside_segment);
 
   g_rec_mutex_clear (&comp->task_rec_lock);
 
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+
   g_mutex_clear (&priv->actions_lock);
   g_cond_clear (&priv->actions_cond);
-
-  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 /* signal_duration_change
@@ -1191,12 +1115,12 @@ nle_composition_reset (NleComposition * comp)
 
   _assert_proper_thread (comp);
 
-  priv->current_stack_start = GST_CLOCK_TIME_NONE;
-  priv->current_stack_stop = GST_CLOCK_TIME_NONE;
+  priv->segment_start = GST_CLOCK_TIME_NONE;
+  priv->segment_stop = GST_CLOCK_TIME_NONE;
   priv->next_base_time = 0;
 
   gst_segment_init (priv->segment, GST_FORMAT_TIME);
-  gst_segment_init (priv->seek_segment, GST_FORMAT_TIME);
+  gst_segment_init (priv->outside_segment, GST_FORMAT_TIME);
 
   if (priv->current)
     g_node_destroy (priv->current);
@@ -1223,9 +1147,8 @@ ghost_event_probe_handler (GstPad * ghostpad G_GNUC_UNUSED,
   NleCompositionPrivate *priv = comp->priv;
   GstEvent *event;
 
-  if (GST_IS_BUFFER (info->data) ||
-      (GST_IS_QUERY (info->data) && GST_QUERY_IS_SERIALIZED (info->data))) {
-    if (priv->waiting_serialized_query_or_buffer) {
+  if (GST_IS_BUFFER (info->data)) {
+    if (priv->waiting_for_buffer) {
       GST_INFO_OBJECT (comp, "update_pipeline DONE");
       _restart_task (comp);
     }
@@ -1308,10 +1231,6 @@ ghost_event_probe_handler (GstPad * ghostpad G_GNUC_UNUSED,
       gst_event_unref (event);
     }
       break;
-    case GST_EVENT_TAG:
-      GST_DEBUG_OBJECT (comp, "Dropping tag: %" GST_PTR_FORMAT, info->data);
-      retval = GST_PAD_PROBE_DROP;
-      break;
     case GST_EVENT_EOS:
     {
       gint seqnum = gst_event_get_seqnum (event);
@@ -1379,13 +1298,12 @@ have_to_update_pipeline (NleComposition * comp,
       GST_TIME_FORMAT "--%" GST_TIME_FORMAT "]",
       GST_TIME_ARGS (priv->segment->start),
       GST_TIME_ARGS (priv->segment->stop),
-      GST_TIME_ARGS (priv->current_stack_start),
-      GST_TIME_ARGS (priv->current_stack_stop));
+      GST_TIME_ARGS (priv->segment_start), GST_TIME_ARGS (priv->segment_stop));
 
-  if (priv->segment->start < priv->current_stack_start)
+  if (priv->segment->start < priv->segment_start)
     return TRUE;
 
-  if (priv->segment->start >= priv->current_stack_stop)
+  if (priv->segment->start >= priv->segment_stop)
     return TRUE;
 
   return FALSE;
@@ -1406,7 +1324,7 @@ nle_composition_commit_func (NleObject * object, gboolean recurse)
  * Returns a seek event for the currently configured segment
  * and start/stop values
  *
- * The GstSegment and current_stack_start|stop must have been configured
+ * The GstSegment and segment_start|stop must have been configured
  * before calling this function.
  */
 static GstEvent *
@@ -1424,21 +1342,21 @@ get_new_seek_event (NleComposition * comp, gboolean initial,
     flags |= (GstSeekFlags) priv->segment->flags;
 
   GST_DEBUG_OBJECT (comp,
-      "private->segment->start:%" GST_TIME_FORMAT " current_stack_start%"
+      "private->segment->start:%" GST_TIME_FORMAT " segment_start%"
       GST_TIME_FORMAT, GST_TIME_ARGS (priv->segment->start),
-      GST_TIME_ARGS (priv->current_stack_start));
+      GST_TIME_ARGS (priv->segment_start));
 
   GST_DEBUG_OBJECT (comp,
-      "private->segment->stop:%" GST_TIME_FORMAT " current_stack_stop%"
+      "private->segment->stop:%" GST_TIME_FORMAT " segment_stop%"
       GST_TIME_FORMAT, GST_TIME_ARGS (priv->segment->stop),
-      GST_TIME_ARGS (priv->current_stack_stop));
+      GST_TIME_ARGS (priv->segment_stop));
 
   start = GST_CLOCK_TIME_IS_VALID (priv->segment->start)
-      ? MAX (priv->segment->start, priv->current_stack_start)
-      : priv->current_stack_start;
+      ? MAX (priv->segment->start, priv->segment_start)
+      : priv->segment_start;
   stop = GST_CLOCK_TIME_IS_VALID (priv->segment->stop)
-      ? MIN (priv->segment->stop, priv->current_stack_stop)
-      : priv->current_stack_stop;
+      ? MIN (priv->segment->stop, priv->segment_stop)
+      : priv->segment_stop;
 
   if (updatestoponly) {
     starttype = GST_SEEK_TYPE_NONE;
@@ -1463,40 +1381,16 @@ get_current_position (NleComposition * comp)
   NleCompositionPrivate *priv = comp->priv;
   gboolean res;
   gint64 value = GST_CLOCK_TIME_NONE;
-  GstObject *parent, *tmp;
 
   GstPad *peer;
 
-  parent = gst_object_get_parent (GST_OBJECT (comp));
-  while ((tmp = parent)) {
-    if (NLE_IS_COMPOSITION (parent)) {
-      GstClockTime parent_position =
-          get_current_position (NLE_COMPOSITION (parent));
+  g_signal_emit (comp, _signals[QUERY_POSITION_SIGNAL], 0, &value);
 
-      if (parent_position > NLE_OBJECT_STOP (comp)
-          || parent_position < NLE_OBJECT_START (comp)) {
-        GST_INFO_OBJECT (comp,
-            "Global position outside of subcomposition, returning TIME_NONE");
+  if (value >= 0) {
+    GST_DEBUG_OBJECT (comp, "Got position %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (value));
 
-        return GST_CLOCK_TIME_NONE;
-      }
-
-      value =
-          parent_position - NLE_OBJECT_START (comp) + NLE_OBJECT_INPOINT (comp);
-    }
-
-    if (GST_IS_PIPELINE (parent)) {
-      if (gst_element_query_position (GST_ELEMENT (parent), GST_FORMAT_TIME,
-              &value)) {
-
-        gst_object_unref (parent);
-        return value;
-      }
-    }
-
-
-    parent = gst_object_get_parent (GST_OBJECT (parent));
-    gst_object_unref (tmp);
+    return value;
   }
 
   /* Try querying position downstream */
@@ -1540,8 +1434,8 @@ get_current_position (NleComposition * comp)
 beach:
 
   if (!GST_CLOCK_TIME_IS_VALID (value)) {
-    if (GST_CLOCK_TIME_IS_VALID (comp->priv->current_stack_start)) {
-      value = comp->priv->current_stack_start;
+    if (GST_CLOCK_TIME_IS_VALID (comp->priv->segment_start)) {
+      value = comp->priv->segment_start;
     } else {
       GST_INFO_OBJECT (comp, "Current position is unknown, " "setting it to 0");
 
@@ -1588,13 +1482,6 @@ _seek_current_stack (NleComposition * comp, GstEvent * event,
 
   GST_INFO_OBJECT (comp, "Seeking itself %" GST_PTR_FORMAT, event);
 
-  if (!peer) {
-    GST_ERROR_OBJECT (comp, "Can't seek because no pad available - "
-        "no children in the composition ready to be used, the duration is 0, "
-        "or not committed yet");
-    return FALSE;
-  }
-
   if (flush_downstream) {
     priv->flush_seqnum = gst_event_get_seqnum (event);
     GST_INFO_OBJECT (comp, "sending flushes downstream with seqnum %d",
@@ -1624,7 +1511,7 @@ static gboolean
 seek_handling (NleComposition * comp, gint32 seqnum,
     NleUpdateStackReason update_stack_reason)
 {
-  GST_DEBUG_OBJECT (comp, "Seek handling update pipeline reason: %s",
+  GST_DEBUG_OBJECT (comp, "Seek hanlding update pipeline reason: %s",
       UPDATE_PIPELINE_REASONS[update_stack_reason]);
 
   if (have_to_update_pipeline (comp, update_stack_reason)) {
@@ -1662,23 +1549,13 @@ nle_composition_event_handler (GstPad * ghostpad, GstObject * parent,
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
-      /* Queue up a seek action if this seek event does not come from
-       * ourselves. Due to a possible race condition around the
-       * seeking_itself flag, we also check if the seek comes from
-       * our task thread. The seeking_itself flag only works as an
-       * optimization */
-      GST_OBJECT_LOCK (comp);
-      if (!priv->seeking_itself || (comp->task
-              && gst_task_get_state (comp->task) != GST_TASK_STOPPED
-              && g_thread_self () != comp->task->thread)) {
-        GST_OBJECT_UNLOCK (comp);
+      if (!priv->seeking_itself) {
         _add_seek_action (comp, event);
         event = NULL;
         GST_FIXME_OBJECT (comp, "HANDLE seeking errors!");
 
         return TRUE;
       }
-      GST_OBJECT_UNLOCK (comp);
       break;
     }
     case GST_EVENT_QOS:
@@ -1692,13 +1569,12 @@ nle_composition_event_handler (GstPad * ghostpad, GstObject * parent,
 
       GST_DEBUG_OBJECT (comp,
           "timestamp:%" GST_TIME_FORMAT " segment.start:%" GST_TIME_FORMAT
-          " segment.stop:%" GST_TIME_FORMAT " current_stack_start%"
-          GST_TIME_FORMAT " current_stack_stop:%" GST_TIME_FORMAT,
-          GST_TIME_ARGS (timestamp),
-          GST_TIME_ARGS (priv->seek_segment->start),
-          GST_TIME_ARGS (priv->seek_segment->stop),
-          GST_TIME_ARGS (priv->current_stack_start),
-          GST_TIME_ARGS (priv->current_stack_stop));
+          " segment.stop:%" GST_TIME_FORMAT " segment_start%" GST_TIME_FORMAT
+          " segment_stop:%" GST_TIME_FORMAT, GST_TIME_ARGS (timestamp),
+          GST_TIME_ARGS (priv->outside_segment->start),
+          GST_TIME_ARGS (priv->outside_segment->stop),
+          GST_TIME_ARGS (priv->segment_start),
+          GST_TIME_ARGS (priv->segment_stop));
 
       /* The problem with QoS events is the following:
        * At each new internal segment (i.e. when we re-arrange our internal
@@ -1727,16 +1603,16 @@ nle_composition_event_handler (GstPad * ghostpad, GstObject * parent,
        *
        */
 
-      if (GST_CLOCK_TIME_IS_VALID (priv->seek_segment->start)) {
+      if (GST_CLOCK_TIME_IS_VALID (priv->outside_segment->start)) {
         GstClockTimeDiff curdiff;
 
         /* We'll either create a new event or discard it */
         gst_event_unref (event);
 
         if (priv->segment->rate < 0.0)
-          curdiff = priv->seek_segment->stop - priv->current_stack_stop;
+          curdiff = priv->outside_segment->stop - priv->segment_stop;
         else
-          curdiff = priv->current_stack_start - priv->seek_segment->start;
+          curdiff = priv->segment_start - priv->outside_segment->start;
         GST_DEBUG ("curdiff %" GST_TIME_FORMAT, GST_TIME_ARGS (curdiff));
         if ((curdiff != 0) && ((timestamp < curdiff)
                 || (curdiff > timestamp + diff))) {
@@ -1827,8 +1703,7 @@ nle_composition_ghost_pad_set_target (NleComposition * comp, GstPad * target)
     priv->ghosteventprobe =
         gst_pad_add_probe (target,
         GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM | GST_PAD_PROBE_TYPE_EVENT_FLUSH |
-        GST_PAD_PROBE_TYPE_DATA_DOWNSTREAM |
-        GST_PAD_PROBE_TYPE_QUERY_DOWNSTREAM,
+        GST_PAD_PROBE_TYPE_DATA_DOWNSTREAM,
         (GstPadProbeCallback) ghost_event_probe_handler, comp, NULL);
     GST_DEBUG_OBJECT (comp, "added event probe %lu", priv->ghosteventprobe);
   }
@@ -2276,7 +2151,7 @@ _restart_task (NleComposition * comp)
         G_PRIORITY_HIGH);
 
   comp->priv->seqnum_to_restart_task = 0;
-  comp->priv->waiting_serialized_query_or_buffer = FALSE;
+  comp->priv->waiting_for_buffer = FALSE;
 
   comp->priv->updating_reason = COMP_UPDATE_STACK_NONE;
   GST_OBJECT_LOCK (comp);
@@ -2295,8 +2170,8 @@ _is_ready_to_restart_task (NleComposition * comp, GstEvent * event)
   if (comp->priv->seqnum_to_restart_task == seqnum) {
     gchar *name = g_strdup_printf ("%s-new-stack__%" GST_TIME_FORMAT "--%"
         GST_TIME_FORMAT "", GST_OBJECT_NAME (comp),
-        GST_TIME_ARGS (comp->priv->current_stack_start),
-        GST_TIME_ARGS (comp->priv->current_stack_stop));
+        GST_TIME_ARGS (comp->priv->segment_start),
+        GST_TIME_ARGS (comp->priv->segment_stop));
 
     GST_INFO_OBJECT (comp, "Got %s with proper seqnum"
         " done with stack reconfiguration %" GST_PTR_FORMAT,
@@ -2311,7 +2186,7 @@ _is_ready_to_restart_task (NleComposition * comp, GstEvent * event)
       return TRUE;
     }
 
-    priv->waiting_serialized_query_or_buffer = TRUE;
+    priv->waiting_for_buffer = TRUE;
     return FALSE;
 
   } else if (comp->priv->seqnum_to_restart_task) {
@@ -2352,23 +2227,9 @@ _commit_func (NleComposition * comp, UpdateCompositionData * ucompo)
     g_signal_emit (comp, _signals[COMMITED_SIGNAL], 0, TRUE);
 
   } else {
-    gboolean reverse;
-
     /* And update the pipeline at current position if needed */
-    update_start_stop_duration (comp);
 
-    reverse = (priv->segment->rate < 0.0);
-    if (!reverse) {
-      GST_DEBUG_OBJECT (comp,
-          "Setting segment->start to curpos:%" GST_TIME_FORMAT,
-          GST_TIME_ARGS (curpos));
-      priv->segment->start = curpos;
-    } else {
-      GST_DEBUG_OBJECT (comp,
-          "Setting segment->stop to curpos:%" GST_TIME_FORMAT,
-          GST_TIME_ARGS (curpos));
-      priv->segment->stop = curpos;
-    }
+    update_start_stop_duration (comp);
     update_pipeline (comp, curpos, ucompo->seqnum, COMP_UPDATE_STACK_ON_COMMIT);
 
     if (!priv->current) {
@@ -2390,18 +2251,18 @@ _update_pipeline_func (NleComposition * comp, UpdateCompositionData * ucompo)
 
   _post_start_composition_update (comp, ucompo->seqnum, ucompo->reason);
 
-  /* Set up a non-initial seek on current_stack_stop */
+  /* Set up a non-initial seek on segment_stop */
   reverse = (priv->segment->rate < 0.0);
   if (!reverse) {
     GST_DEBUG_OBJECT (comp,
-        "Setting segment->start to current_stack_stop:%" GST_TIME_FORMAT,
-        GST_TIME_ARGS (priv->current_stack_stop));
-    priv->segment->start = priv->current_stack_stop;
+        "Setting segment->start to segment_stop:%" GST_TIME_FORMAT,
+        GST_TIME_ARGS (priv->segment_stop));
+    priv->segment->start = priv->segment_stop;
   } else {
     GST_DEBUG_OBJECT (comp,
-        "Setting segment->stop to current_stack_start:%" GST_TIME_FORMAT,
-        GST_TIME_ARGS (priv->current_stack_start));
-    priv->segment->stop = priv->current_stack_start;
+        "Setting segment->stop to segment_start:%" GST_TIME_FORMAT,
+        GST_TIME_ARGS (priv->segment_start));
+    priv->segment->stop = priv->segment_start;
   }
 
   seek_handling (comp, ucompo->seqnum, COMP_UPDATE_STACK_ON_EOS);
@@ -2427,10 +2288,27 @@ _update_pipeline_func (NleComposition * comp, UpdateCompositionData * ucompo)
   _post_start_composition_update_done (comp, ucompo->seqnum, ucompo->reason);
 }
 
+static void
+_set_all_children_state (NleComposition * comp, GstState state)
+{
+  GList *tmp;
+
+  GST_DEBUG_OBJECT (comp, "Setting all children state to %s",
+      gst_element_state_get_name (state));
+
+  comp->priv->tearing_down_stack = TRUE;
+  gst_element_set_state (comp->priv->current_bin, state);
+  for (tmp = comp->priv->objects_start; tmp; tmp = tmp->next)
+    gst_element_set_state (tmp->data, state);
+
+  for (tmp = comp->priv->expandables; tmp; tmp = tmp->next)
+    gst_element_set_state (tmp->data, state);
+  comp->priv->tearing_down_stack = FALSE;
+}
+
 static GstStateChangeReturn
 nle_composition_change_state (GstElement * element, GstStateChange transition)
 {
-  GstStateChangeReturn res;
   NleComposition *comp = (NleComposition *) element;
 
   GST_DEBUG_OBJECT (comp, "%s => %s",
@@ -2454,53 +2332,23 @@ nle_composition_change_state (GstElement * element, GstStateChange transition)
 
       _remove_update_actions (comp);
       _remove_seek_actions (comp);
-      _deactivate_stack (comp, TRUE);
-      comp->priv->tearing_down_stack = TRUE;
+      _set_all_children_state (comp, GST_STATE_READY);
+      nle_composition_reset (comp);
+
+      _start_task (comp);
       break;
     case GST_STATE_CHANGE_READY_TO_NULL:
       _stop_task (comp);
 
       _remove_update_actions (comp);
       _remove_seek_actions (comp);
-      comp->priv->tearing_down_stack = TRUE;
+      _set_all_children_state (comp, GST_STATE_NULL);
       break;
     default:
       break;
   }
 
-  res = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  if (res == GST_STATE_CHANGE_FAILURE) {
-    GST_ERROR_OBJECT (comp, "state change failure %s => %s",
-        gst_element_state_get_name (GST_STATE_TRANSITION_CURRENT (transition)),
-        gst_element_state_get_name (GST_STATE_TRANSITION_NEXT (transition)));
-
-    comp->priv->tearing_down_stack = TRUE;
-    _stop_task (comp);
-    nle_composition_reset (comp);
-    gst_element_set_state (comp->priv->current_bin, GST_STATE_NULL);
-    comp->priv->tearing_down_stack = FALSE;
-
-    return res;
-  }
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      comp->priv->tearing_down_stack = FALSE;
-      nle_composition_reset (comp);
-
-      /* In READY we are still able to process actions. */
-      _start_task (comp);
-      break;
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      gst_element_set_state (comp->priv->current_bin, GST_STATE_NULL);
-      comp->priv->tearing_down_stack = FALSE;
-      break;
-    default:
-      break;
-  }
-
-  return res;
+  return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 }
 
 static gint
@@ -2693,7 +2541,7 @@ _relink_children_recursively (NleComposition * comp,
     GST_ELEMENT_ERROR (comp, STREAM, FAILED,
         ("The NleComposition structure is not valid"),
         ("Operation %" GST_PTR_FORMAT
-            " has no child objects to be connected to "
+            " has no child objects to be connected to"
             "current toplevel seek: %" GST_PTR_FORMAT, oper, toplevel_seek));
   }
   /* Make sure we have enough sinkpads */
@@ -2738,8 +2586,7 @@ _relink_single_node (NleComposition * comp, GNode * node,
   gst_bin_add (GST_BIN (comp->priv->current_bin), GST_ELEMENT (newobj));
   gst_element_sync_state_with_parent (GST_ELEMENT_CAST (newobj));
 
-  translated_seek = nle_object_translate_incoming_seek (newobj,
-      gst_event_ref (toplevel_seek));
+  translated_seek = nle_object_translate_incoming_seek (newobj, toplevel_seek);
 
   gst_element_send_event (GST_ELEMENT (newobj), translated_seek);
 
@@ -2782,7 +2629,7 @@ _deactivate_stack (NleComposition * comp, gboolean flush_downstream)
 {
   GstPad *ptarget;
 
-  GST_INFO_OBJECT (comp, "Deactivating current stack (flushing downstream: %d)",
+  GST_INFO_OBJECT (comp, "Deactivating current stack (flushing downstream: %d",
       flush_downstream);
   _set_current_bin_to_ready (comp, flush_downstream);
 
@@ -2885,8 +2732,8 @@ _activate_new_stack (NleComposition * comp)
   if (!priv->current) {
     if ((!priv->objects_start)) {
       nle_composition_reset_target_pad (comp);
-      priv->current_stack_start = 0;
-      priv->current_stack_stop = GST_CLOCK_TIME_NONE;
+      priv->segment_start = 0;
+      priv->segment_stop = GST_CLOCK_TIME_NONE;
     }
 
     GST_DEBUG_OBJECT (comp, "Nothing else in the composition"
@@ -2908,21 +2755,8 @@ _activate_new_stack (NleComposition * comp)
 
 resync_state:
   gst_element_set_locked_state (priv->current_bin, FALSE);
-
   GST_DEBUG ("going back to parent state");
-  priv->suppress_child_error = TRUE;
-  if (!gst_element_sync_state_with_parent (priv->current_bin)) {
-    gst_element_set_locked_state (priv->current_bin, TRUE);
-    gst_element_set_state (priv->current_bin, GST_STATE_NULL);
-    priv->suppress_child_error = FALSE;
-
-    GST_ELEMENT_ERROR (comp, CORE, STATE_CHANGE, (NULL),
-        ("Could not sync %" GST_PTR_FORMAT " state with parent",
-            priv->current_bin));
-    return FALSE;
-  }
-
-  priv->suppress_child_error = FALSE;
+  gst_element_sync_state_with_parent (priv->current_bin);
   GST_DEBUG ("gone back to parent state");
 
   return TRUE;
@@ -2939,9 +2773,9 @@ _set_real_eos_seqnum_from_seek (NleComposition * comp, GstEvent * event)
   gboolean reverse = (priv->segment->rate < 0);
   gint stack_seqnum = gst_event_get_seqnum (event);
 
-  if (reverse && GST_CLOCK_TIME_IS_VALID (priv->current_stack_start))
+  if (reverse && GST_CLOCK_TIME_IS_VALID (priv->segment_start))
     should_check_objects = TRUE;
-  else if (!reverse && GST_CLOCK_TIME_IS_VALID (priv->current_stack_stop))
+  else if (!reverse && GST_CLOCK_TIME_IS_VALID (priv->segment_stop))
     should_check_objects = TRUE;
 
   if (should_check_objects) {
@@ -2951,8 +2785,8 @@ _set_real_eos_seqnum_from_seek (NleComposition * comp, GstEvent * event)
       if (!NLE_IS_SOURCE (object))
         continue;
 
-      if ((!reverse && priv->current_stack_stop < object->stop) ||
-          (reverse && priv->current_stack_start > object->start)) {
+      if ((!reverse && priv->segment_stop < object->stop) ||
+          (reverse && priv->segment_start > object->start)) {
         priv->next_eos_seqnum = stack_seqnum;
         g_atomic_int_set (&priv->real_eos_seqnum, 0);
         return FALSE;
@@ -2966,46 +2800,6 @@ _set_real_eos_seqnum_from_seek (NleComposition * comp, GstEvent * event)
   return TRUE;
 }
 
-#ifndef GST_DISABLE_GST_DEBUG
-static gboolean
-_print_stack (GNode * node, gpointer res)
-{
-  NleObject *obj = NLE_OBJECT (node->data);
-
-  g_string_append_printf ((GString *) res,
-      "%*s [s=%" GST_TIME_FORMAT " - d=%" GST_TIME_FORMAT "] prio=%d\n",
-      g_node_depth (node) * 4, GST_OBJECT_NAME (obj),
-      GST_TIME_ARGS (NLE_OBJECT_START (obj)),
-      GST_TIME_ARGS (NLE_OBJECT_STOP (obj)), obj->priority);
-
-  return FALSE;
-}
-#endif
-
-static void
-_dump_stack (NleComposition * comp, GNode * stack)
-{
-#ifndef GST_DISABLE_GST_DEBUG
-  GString *res;
-
-  if (!stack)
-    return;
-
-  if (gst_debug_category_get_threshold (nlecomposition_debug) < GST_LEVEL_INFO)
-    return;
-
-  res = g_string_new (NULL);
-  g_string_append_printf (res, " ====> dumping stack [%" GST_TIME_FORMAT " - %"
-      GST_TIME_FORMAT "]:\n",
-      GST_TIME_ARGS (comp->priv->current_stack_start),
-      GST_TIME_ARGS (comp->priv->current_stack_stop));
-  g_node_traverse (stack, G_LEVEL_ORDER, G_TRAVERSE_ALL, -1, _print_stack, res);
-
-  GST_INFO_OBJECT (comp, "%s", res->str);
-  g_string_free (res, TRUE);
-#endif
-}
-
 /*
  * update_pipeline:
  * @comp: The #NleComposition
@@ -3067,14 +2861,14 @@ update_pipeline (NleComposition * comp, GstClockTime currenttime, gint32 seqnum,
   stack = get_clean_toplevel_stack (comp, &currenttime, &new_start, &new_stop);
   samestack = are_same_stacks (priv->current, stack);
 
-  /* set new current_stack_start/stop (the current zone over which the new stack
+  /* set new segment_start/stop (the current zone over which the new stack
    * is valid) */
   if (priv->segment->rate >= 0.0) {
-    priv->current_stack_start = currenttime;
-    priv->current_stack_stop = new_stop;
+    priv->segment_start = currenttime;
+    priv->segment_stop = new_stop;
   } else {
-    priv->current_stack_start = new_start;
-    priv->current_stack_stop = currenttime;
+    priv->segment_start = new_start;
+    priv->segment_stop = currenttime;
   }
 
 # if 0
@@ -3085,11 +2879,11 @@ update_pipeline (NleComposition * comp, GstClockTime currenttime, gint32 seqnum,
   gboolean startchanged, stopchanged;
 
   if (priv->segment->rate >= 0.0) {
-    startchanged = priv->current_stack_start != currenttime;
-    stopchanged = priv->current_stack_stop != new_stop;
+    startchanged = priv->segment_start != currenttime;
+    stopchanged = priv->segment_stop != new_stop;
   } else {
-    startchanged = priv->current_stack_start != new_start;
-    stopchanged = priv->current_stack_stop != currenttime;
+    startchanged = priv->segment_start != new_start;
+    stopchanged = priv->segment_stop != currenttime;
   }
 
   if (samestack) {
@@ -3109,19 +2903,14 @@ update_pipeline (NleComposition * comp, GstClockTime currenttime, gint32 seqnum,
 
   /* If stacks are different, unlink/relink objects */
   if (!samestack) {
-    _dump_stack (comp, stack);
     _deactivate_stack (comp, _have_to_flush_downstream (update_reason));
     _relink_new_stack (comp, stack, toplevel_seek);
   }
 
   /* Unlock all elements in new stack */
   GST_INFO_OBJECT (comp, "Setting current stack [%" GST_TIME_FORMAT " - %"
-      GST_TIME_FORMAT "]", GST_TIME_ARGS (priv->current_stack_start),
-      GST_TIME_ARGS (priv->current_stack_stop));
-
-  if (priv->current)
-    g_node_destroy (priv->current);
-
+      GST_TIME_FORMAT "]", GST_TIME_ARGS (priv->segment_start),
+      GST_TIME_ARGS (priv->segment_stop));
   priv->current = stack;
 
   if (priv->current) {
@@ -3265,7 +3054,7 @@ nle_composition_remove_object (GstBin * bin, GstElement * element)
   NleComposition *comp = (NleComposition *) bin;
 
   if (element == comp->priv->current_bin) {
-    GST_INFO_OBJECT (comp, "Removing internal bin");
+    GST_INFO_OBJECT (comp, "Adding internal bin");
     return GST_BIN_CLASS (parent_class)->remove_element (bin, element);
   }
 
