@@ -22,18 +22,14 @@
 #include "ges-launcher.h"
 
 static void
-_print_all_commands (gint nargs, gchar ** commands)
+_print_all_commands (void)
 {
-  gchar *help;
-
-  if (nargs == 0)
-    g_print ("Available ges-launch-1.0 commands:\n\n");
-
-  help = ges_command_line_formatter_get_help (nargs, commands);
-
-  g_print ("%s", help);
-
-  g_free (help);
+  /* Yeah I know very fancy */
+  g_print ("Available ges-launch-1.0 commands:\n\n");
+  g_print ("  %-9s %-11s %-15s %-10s\n\n", "+clip", "+effect", "+test-clip",
+      "set-");
+  g_print ("See ges-launch-1.0 help <command> or ges-launch-1.0 help <guide> "
+      "to read about a specific command or a given guide\n");
 }
 
 static void
@@ -48,15 +44,7 @@ _check_command_help (int argc, gchar ** argv)
  */
 
   if (!g_strcmp0 (argv[1], "help")) {
-    gint nargs = 0;
-    gchar **commands = NULL;
-
-    if (argc > 2) {
-      nargs = argc - 2;
-      commands = &argv[2];
-    }
-
-    _print_all_commands (nargs, commands);
+    _print_all_commands ();
     exit (0);
   }
 
@@ -86,11 +74,7 @@ main (int argc, gchar ** argv)
   launcher = ges_launcher_new ();
 
   ret = g_application_run (G_APPLICATION (launcher), argc, argv);
-
-  if (!ret)
-    ret = ges_launcher_get_exit_status (launcher);
-
-  g_object_unref (launcher);
-
-  return ret;
+  if (ret)
+    return ret;
+  return ges_launcher_get_exit_status (launcher);
 }
