@@ -51,8 +51,6 @@ typedef struct
   gboolean list_transitions;
   gboolean inspect_action_type;
   gchar *sanitized_timeline;
-  const gchar *video_track_caps;
-  const gchar *audio_track_caps;
 } ParsedOptions;
 
 struct _GESLauncherPrivate
@@ -66,7 +64,7 @@ struct _GESLauncherPrivate
   ParsedOptions parsed_options;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (GESLauncher, ges_launcher, G_TYPE_APPLICATION);
+G_DEFINE_TYPE (GESLauncher, ges_launcher, G_TYPE_APPLICATION);
 
 static const gchar *HELP_SUMMARY =
     "ges-launch-1.0 creates a multimedia timeline and plays it back,\n"
@@ -94,29 +92,6 @@ _parse_track_type (const gchar * option_name, const gchar * value,
   return TRUE;
 }
 
-static gboolean
-_set_track_restriction_caps (GESTrack * track, const gchar * caps_str)
-{
-  GstCaps *caps;
-
-  if (!caps_str)
-    return TRUE;
-
-  caps = gst_caps_from_string (caps_str);
-
-  if (!caps) {
-    g_printerr ("Could not create caps for %s from: %s",
-        G_OBJECT_TYPE_NAME (track), caps_str);
-
-    return FALSE;
-  }
-
-  ges_track_set_restriction_caps (track, caps);
-
-  gst_caps_unref (caps);
-  return TRUE;
-}
-
 static gboolean
 _timeline_set_user_options (GESLauncher * self, GESTimeline * timeline,
     const gchar * load_path)
@@ -147,9 +122,6 @@ retry:
     if (!has_video && opts->track_types & GES_TRACK_TYPE_VIDEO) {
       trackv = GES_TRACK (ges_video_track_new ());
 
-      if (!_set_track_restriction_caps (trackv, opts->video_track_caps))
-        return FALSE;
-
       if (opts->disable_mixing)
         ges_track_set_mixing (trackv, FALSE);
 
@@ -159,10 +131,6 @@ retry:
 
     if (!has_audio && opts->track_types & GES_TRACK_TYPE_AUDIO) {
       tracka = GES_TRACK (ges_audio_track_new ());
-
-      if (!_set_track_restriction_caps (tracka, opts->audio_track_caps))
-        return FALSE;
-
       if (opts->disable_mixing)
         ges_track_set_mixing (tracka, FALSE);
 
@@ -178,7 +146,6 @@ static void
 _project_loaded_cb (GESProject * project, GESTimeline * timeline,
     GESLauncher * self)
 {
-  gchar *project_uri = NULL;
   ParsedOptions *opts = &self->priv->parsed_options;
   GST_INFO ("Project loaded, playing it");
 
@@ -207,10 +174,9 @@ _project_loaded_cb (GESProject * project, GESTimeline * timeline,
     }
   }
 
-  project_uri = ges_project_get_uri (project);
-  _timeline_set_user_options (self, timeline, project_uri);
+  _timeline_set_user_options (self, timeline, ges_project_get_uri (project));
 
-  if (self->priv->parsed_options.load_path && project_uri
+  if (self->priv->parsed_options.load_path && ges_project_get_uri (project)
       && ges_validate_activate (GST_PIPELINE (self->priv->pipeline),
           opts->scenario, &opts->needs_set_state) == FALSE) {
     g_error ("Could not activate scenario %s", opts->scenario);
@@ -218,8 +184,6 @@ _project_loaded_cb (GESProject * project, GESTimeline * timeline,
     g_application_quit (G_APPLICATION (self));
   }
 
-  g_free (project_uri);
-
   if (!self->priv->seenerrors && opts->needs_set_state &&
       gst_element_set_state (GST_ELEMENT (self->priv->pipeline),
           GST_STATE_PLAYING) == GST_STATE_CHANGE_FAILURE) {
@@ -422,7 +386,7 @@ _run_pipeline (GESLauncher * self)
     }
 
     if (!_timeline_set_user_options (self, self->priv->timeline, NULL)) {
-      g_printerr ("Could not properly set tracks\n");
+      g_error ("Could not properly set tracks");
       return FALSE;
     }
   }
@@ -707,14 +671,6 @@ _local_command_line (GApplication * application, gchar ** arguments[],
           "Specify the track types to be created. "
           "When loading a project, only relevant tracks will be added to the timeline.",
         "<track-types>"},
-    {"video-caps", 0, 0, G_OPTION_ARG_STRING, &opts->video_track_caps,
-        "Specify the track restriction caps of the video track.",},
-    {"audio-caps", 0, 0, G_OPTION_ARG_STRING, &opts->audio_track_caps,
-        "Specify the track restriction caps of the audio track.",},
-    {"track-types", 't', 0, G_OPTION_ARG_CALLBACK, &_parse_track_type,
-          "Specify the track types to be created. "
-          "When loading a project, only relevant tracks will be added to the timeline.",
-        "<track-types>"},
 #ifdef HAVE_GST_VALIDATE
     {"set-scenario", 0, 0, G_OPTION_ARG_STRING, &opts->scenario,
           "ges-launch-1.0 exposes gst-validate functionalities, such as scenarios."
@@ -751,7 +707,6 @@ _local_command_line (GApplication * application, gchar ** arguments[],
   *exit_status = 0;
 
   if (!g_option_context_parse (ctx, &argc, &argv, &error)) {
-    gst_init (NULL, NULL);
     g_printerr ("Error initializing: %s\n", error->message);
     g_option_context_free (ctx);
     g_error_free (error);
@@ -865,12 +820,14 @@ ges_launcher_class_init (GESLauncherClass * klass)
   G_APPLICATION_CLASS (klass)->local_command_line = _local_command_line;
   G_APPLICATION_CLASS (klass)->startup = _startup;
   G_APPLICATION_CLASS (klass)->shutdown = _shutdown;
+  g_type_class_add_private (klass, sizeof (GESLauncherPrivate));
 }
 
 static void
 ges_launcher_init (GESLauncher * self)
 {
-  self->priv = ges_launcher_get_instance_private (self);
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+      GES_TYPE_LAUNCHER, GESLauncherPrivate);
   self->priv->parsed_options.track_types =
       GES_TRACK_TYPE_AUDIO | GES_TRACK_TYPE_VIDEO;
 }
